@@ -3,6 +3,20 @@
 ###
 ### Copyright 1997-2003  Jose C. Pinheiro <Jose.Pinheiro@pharma.novartis.com>,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+#
 
 gnls <-
   function(model,
@@ -95,7 +109,7 @@ gnls <-
     params <- list(params)
   }
   val <- NULL
-  for(i in seq(along = params)) {
+  for(i in seq_along(params)) {
     if (is.name(params[[i]][[2]])) {
       val <- c(val, list(params[[i]]))
     } else {
@@ -107,7 +121,7 @@ gnls <-
   }
   params <- as.list(val)
   pnames <- character(length(params))
-  for (i in seq(along = params)) {
+  for (i in seq_along(params)) {
     this <- eval(params[[i]])
     if (!inherits(this, "formula"))
       stop ("params must be a formula or list of formulae")
@@ -220,7 +234,7 @@ gnls <-
       plist[[nm]] <-
         model.matrix(asOneSidedFormula(params[[nm]][[3]]),
                  model.frame(asOneSidedFormula(params[[nm]][[3]]), dataModShrunk))
-      auxContr <- attr(plist[[nm]], "contr")
+      auxContr <- attr(plist[[nm]], "contrasts")
       contr <- c(contr, auxContr[is.na(match(names(auxContr), names(contr)))])
     }
   }
@@ -489,6 +503,7 @@ gnls <-
     Fitted[] <- Fitted[revOrderShrunk]
     grpShrunk[] <- grpShrunk[revOrderShrunk]
   }
+  names(Resid) <- names(Fitted) <- origOrderShrunk
   ## getting the approximate var-cov of the parameters
   ## first making Xy into single column array again
   attr(gnlsSt, "conLin")$Xy <- array(auxRes, c(NReal, 1))
@@ -523,7 +538,8 @@ gnls <-
 		 residuals = Resid,
 		 plist = plist,
                  pmap = pmap,
-                 parAssign = parAssign)
+                 parAssign = parAssign,
+                 na.action = attr(dataMod, "na.action"))
   if (inherits(data, "groupedData")) {
     ## saving labels and units for plots
     attr(estOut, "units") <- attr(data, "units")
@@ -711,7 +727,7 @@ predict.gnls <-
     params <- list(params)
   }
   val <- NULL
-  for(i in seq(along = params)) {
+  for(i in seq_along(params)) {
     if (is.name(params[[i]][[2]])) {
       val <- c(val, list(params[[i]]))
     } else {
@@ -825,7 +841,7 @@ Initialize.gnlsStruct <-
     object[] <- lapply(object, Initialize, data)
     theta <- lapply(object, coef)
     len <- unlist(lapply(theta, length))
-    num <- seq(along = len)
+    num <- seq_along(len)
     if (sum(len) > 0) {
       pmap <- outer(rep(num, len), num, "==")
     } else {
