@@ -22,7 +22,8 @@
 #define F77_FUNC(name,NAME) name ## _
 
 /* As F77_FUNC, but for C identifiers containing underscores. */
-#define F77_FUNC_(name,NAME) name ## __
+/* should be name # __ for GCC3 */
+#define F77_FUNC_(name,NAME) name ## _
 
 /* Define if F77 and FC dummy `main' functions are identical. */
 /* #undef FC_DUMMY_MAIN_EQ_F77 */
@@ -44,9 +45,18 @@
    be built. */
 /* #undef HAVE_AQUA */
 
+/* Define to 1 if you have the `argz_count' function. */
+/* #undef HAVE_ARGZ_COUNT */
+
 /* Define to 1 if you have the <argz.h> header file. */
 /* #undef HAVE_ARGZ_H */
 
+/* Define to 1 if you have the `argz_next' function. */
+/* #undef HAVE_ARGZ_NEXT */
+
+/* Define to 1 if you have the `argz_stringify' function. */
+/* #undef HAVE_ARGZ_STRINGIFY */
+
 /* Define to 1 if you have the <arpa/inet.h> header file. */
 /* #undef HAVE_ARPA_INET_H */
 
@@ -62,6 +72,9 @@
 /* Define if you have BSD networking headers and libraries. */
 /* #undef HAVE_BSD_NETWORKING */
 
+/* Define to 1 if the compiler understands __builtin_expect. */
+#define HAVE_BUILTIN_EXPECT 1
+
 /* Define this if you have support for C99 complex types. */
 #define HAVE_C99_COMPLEX 1
 
@@ -127,6 +140,10 @@
    */
 #define HAVE_DECL_PUTENV 1
 
+/* Define to 1 if you have the declaration of `realpath', and to 0 if you
+   don't. */
+#define HAVE_DECL_REALPATH 0
+
 /* Define to 1 if you have the declaration of `siglongjmp', and to 0 if you
    don't. */
 #define HAVE_DECL_SIGLONGJMP 1
@@ -183,9 +200,17 @@
 /* Define to 1 if you have the <errno.h> header file. */
 #define HAVE_ERRNO_H 1
 
+/* Define to 1 if you have the `execv' function. */
+#define HAVE_EXECV 1
+
 /* Define to 1 if you have the `expm1' function. */
 #define HAVE_EXPM1 1
 
+/* Define if your Fortran compiler appends an extra_underscore to external
+   names containing an underscore. */
+/* needs to be set for GCC3 */
+/* #undef HAVE_F77_EXTRA_UNDERSCORE */
+
 /* Define if your Fortran compiler appends an underscore to external names. */
 #define HAVE_F77_UNDERSCORE 1
 
@@ -198,9 +223,6 @@
 /* Define to 1 if you have the `fdopen' function. */
 #define HAVE_FDOPEN 1
 
-/* Define to 1 if you have the `finite' function. */
-#define HAVE_FINITE 1
-
 /* Define to 1 if you have the <floatingpoint.h> header file. */
 /* #undef HAVE_FLOATINGPOINT_H */
 
@@ -254,7 +276,7 @@
 /* #undef HAVE_GETTEXT */
 
 /* Define to 1 if you have the `gettimeofday' function. */
-/* #undef HAVE_GETTIMEOFDAY */
+#define HAVE_GETTIMEOFDAY 1
 
 /* Define to 1 if you have the `getuid' function. */
 /* #undef HAVE_GETUID */
@@ -263,6 +285,12 @@
    problem with getting the prototype of strptime(). */
 /* #undef HAVE_GLIBC2 */
 
+/* Define to 1 if you have the `glob' function. */
+/* #undef HAVE_GLOB */
+
+/* Define to 1 if you have the <glob.h> header file. */
+/* #undef HAVE_GLOB_H */
+
 /* Define to 1 if you have the <grp.h> header file. */
 /* #undef HAVE_GRP_H */
 
@@ -281,12 +309,6 @@
 /* Define to 1 if you have the <iconv.h> header file. */
 #define HAVE_ICONV_H 1
 
-/* Define to 1 if you have the <ieee754.h> header file. */
-/* #undef HAVE_IEEE754_H */
-
-/* Define to 1 if you have the <ieeefp.h> header file. */
-/* #undef HAVE_IEEEFP_H */
-
 /* Define if you have support for ftp/http access. */
 #define HAVE_INTERNET 1
 
@@ -296,7 +318,7 @@
 /* Define to 1 if the system has the type `intptr_t'. */
 #define HAVE_INTPTR_T 1
 
-/* Define if <inttypes.h> exists and doesn't clash with <sys/types.h>. */
+/* Define to 1 if you have the <inttypes.h> header file. */
 #define HAVE_INTTYPES_H 1
 
 /* Define if <inttypes.h> exists, doesn't clash with <sys/types.h>, and
@@ -304,13 +326,13 @@
 #define HAVE_INTTYPES_H_WITH_UINTMAX 1
 
 /* Define to 1 if you have the `isblank' function. */
-/* #undef HAVE_ISBLANK */
+#define HAVE_ISBLANK 1
 
 /* Define to 1 if you have the `isnan' function. */
 #define HAVE_ISNAN 1
 
 /* Define to 1 if you have the `iswblank' function. */
-/* #undef HAVE_ISWBLANK */
+#define HAVE_ISWBLANK 1
 
 /* Define to 1 if you have the `iswctype' function. */
 #define HAVE_ISWCTYPE 1
@@ -318,6 +340,9 @@
 /* Define if you have the JPEG headers and libraries. */
 #define HAVE_JPEG 1
 
+/* Define if KERN_USRSTACK sysctl is supported. */
+/* #undef HAVE_KERN_USRSTACK */
+
 /* Define if you have KeySym defined in X11. */
 /* #undef HAVE_KEYSYM */
 
@@ -330,6 +355,9 @@
 /* Define to 1 if you have the `cc_dynamic' library (-lcc_dynamic). */
 /* #undef HAVE_LIBCC_DYNAMIC */
 
+/* Define if __libc_stack_end is visible. */
+/* #undef HAVE_LIBC_STACK_END */
+
 /* Define to 1 if you have the `dl' library (-ldl). */
 /* #undef HAVE_LIBDL */
 
@@ -369,11 +397,8 @@
 /* Define if you have the 'long double' type. */
 #define HAVE_LONG_DOUBLE 1
 
-/* Define if you have the 'long long' type. */
-#define HAVE_LONG_LONG 1
-
-/* Define to 1 if you have the <malloc.h> header file. */
-#define HAVE_MALLOC_H 1
+/* Define to 1 if the system has the type `long long int'. */
+#define HAVE_LONG_LONG_INT 1
 
 /* Define to 1 if you have the `matherr' function. */
 /* #undef HAVE_MATHERR */
@@ -381,6 +406,9 @@
 /* Define to 1 if you have the `mbrtowc' function. */
 #define HAVE_MBRTOWC 1
 
+/* Define to 1 if the system has the type `mbstate_t'. */
+#define HAVE_MBSTATE_T 1
+
 /* Define to 1 if you have the `mbstowcs' function. */
 #define HAVE_MBSTOWCS 1
 
@@ -411,8 +439,9 @@
 /* Define to 1 if you have the <netinet/in.h> header file. */
 /* #undef HAVE_NETINET_IN_H */
 
-/* Define to 1 if you have the <nl_types.h> header file. */
-/* #undef HAVE_NL_TYPES_H */
+/* Define if you have <langinfo.h> and it defines the NL_LOCALE_NAME macro if
+   _GNU_SOURCE is defined. */
+/* #undef HAVE_NL_LOCALE_NAME */
 
 /* Define if module-loading does not need an underscore to be prepended to
    external names. */
@@ -438,14 +467,26 @@
 #define HAVE_POSIX_LEAPSECONDS 1
 
 /* Define if your printf() function supports format strings with positions. */
-/* #undef HAVE_POSIX_PRINTF */
+#define HAVE_POSIX_PRINTF 1
 
 /* Define if you have POSIX.1 compatible sigsetjmp/siglongjmp. */
 #define HAVE_POSIX_SETJMP 1
 
+/* Define if the <pthread.h> defines PTHREAD_MUTEX_RECURSIVE. */
+#define HAVE_PTHREAD_MUTEX_RECURSIVE 1
+
+/* Define if the POSIX multithreading library has read/write locks. */
+#define HAVE_PTHREAD_RWLOCK 1
+
 /* Define to 1 if you have the `putenv' function. */
 #define HAVE_PUTENV 1
 
+/* Define if putenv("FOO") can unset an environment variable */
+/* #undef HAVE_PUTENV_UNSET */
+
+/* Define if putenv("FOO=") can unset an environment variable */
+#define HAVE_PUTENV_UNSET2 1
+
 /* Define to 1 if you have the <pwd.h> header file. */
 /* #undef HAVE_PWD_H */
 
@@ -461,6 +502,9 @@
 /* Define to 1 if you have the `rint' function. */
 #define HAVE_RINT 1
 
+/* Define to 1 if you have the `rl_completion_matches' function. */
+/* #undef HAVE_RL_COMPLETION_MATCHES */
+
 /* Define to 1 if you have the `setenv' function. */
 /* #undef HAVE_SETENV */
 
@@ -494,6 +538,9 @@
 /* Define to 1 if you have the <stdarg.h> header file. */
 #define HAVE_STDARG_H 1
 
+/* Define to 1 if you have the <stdbool.h> header file. */
+#define HAVE_STDBOOL_H 1
+
 /* Define to 1 if you have the <stddef.h> header file. */
 #define HAVE_STDDEF_H 1
 
@@ -585,6 +632,9 @@
    to be built. */
 #define HAVE_TCLTK 1
 
+/* Define to 1 if you have the `time' function. */
+#define HAVE_TIME 1
+
 /* Define to 1 if you have the `times' function. */
 #define HAVE_TIMES 1
 
@@ -592,7 +642,7 @@
 #define HAVE_TIME_H 1
 
 /* Define to 1 if you have the `tsearch' function. */
-/* #undef HAVE_TSEARCH */
+#define HAVE_TSEARCH 1
 
 /* Define if you have the 'uintmax_t' type in <stdint.h> or <inttypes.h>. */
 #define HAVE_UINTMAX_T 1
@@ -609,8 +659,11 @@
 /* Define if you have the 'unsigned long long' type. */
 #define HAVE_UNSIGNED_LONG_LONG 1
 
+/* Define to 1 if the system has the type `unsigned long long int'. */
+#define HAVE_UNSIGNED_LONG_LONG_INT 1
+
 /* Define to 1 if you have the `vasprintf' function. */
-/* #undef HAVE_VASPRINTF */
+#define HAVE_VASPRINTF 1
 
 /* Define to 1 if you have the `va_copy' function. */
 #define HAVE_VA_COPY 1
@@ -618,6 +671,10 @@
 /* Defined if framework vecLib is present */
 /* #undef HAVE_VECLIB_FW */
 
+/* Define to 1 or 0, depending whether the compiler supports simple visibility
+   declarations. */
+#define HAVE_VISIBILITY 1
+
 /* Define to 1 if __attribute__((visibility())) is supported */
 /* #undef HAVE_VISIBILITY_ATTRIBUTE */
 
@@ -642,6 +699,9 @@
 /* Define to 1 if you have the `wcslen' function. */
 #define HAVE_WCSLEN 1
 
+/* Define to 1 if you have the `wcstod' function. */
+#define HAVE_WCSTOD 1
+
 /* Define to 1 if you have the `wcstombs' function. */
 #define HAVE_WCSTOMBS 1
 
@@ -663,63 +723,24 @@
 /* Define if calloc(0) returns a null pointer. */
 #define HAVE_WORKING_CALLOC 1
 
-/* Define if finite() is correct for -Inf/NaN/Inf. */
-#define HAVE_WORKING_FINITE 1
-
 /* Define if your ftell works correctly on files opened for append. */
 /* #undef HAVE_WORKING_FTELL */
 
 /* Define if isfinite() is correct for -Inf/NaN/Inf. */
 #define HAVE_WORKING_ISFINITE 1
 
-/* Define if log() is correct for 0/-1. */
-#define HAVE_WORKING_LOG 1
-
 /* Define if log1p() exists and is accurate enough. */
 #define HAVE_WORKING_LOG1P 1
 
+/* Define if sigaction() is complete enough for R's usage */
+/* #undef HAVE_WORKING_SIGACTION */
+
 /* Define if you have the X11 headers and libraries, and want the X11 GUI to
    be built. */
 /* #undef HAVE_X11 */
 
-/* Define to 1 if you have the `XmbDrawImageString' function. */
-/* #undef HAVE_XMBDRAWIMAGESTRING */
-
-/* Define to 1 if you have the `XmbDrawString' function. */
-/* #undef HAVE_XMBDRAWSTRING */
-
-/* Define to 1 if you have the `XmbLookupString' function. */
-/* #undef HAVE_XMBLOOKUPSTRING */
-
-/* Define to 1 if you have the `XmbTextEscapement' function. */
-/* #undef HAVE_XMBTEXTESCAPEMENT */
-
-/* Define to 1 if you have the `XmbTextExtents' function. */
-/* #undef HAVE_XMBTEXTEXTENTS */
-
-/* Define to 1 if you have the `Xutf8DrawImageString' function. */
-/* #undef HAVE_XUTF8DRAWIMAGESTRING */
-
-/* Define to 1 if you have the `Xutf8DrawString' function. */
-/* #undef HAVE_XUTF8DRAWSTRING */
-
-/* Define to 1 if you have the `Xutf8LookupString' function. */
-/* #undef HAVE_XUTF8LOOKUPSTRING */
-
-/* Define to 1 if you have the `Xutf8TextEscapement' function. */
-/* #undef HAVE_XUTF8TEXTESCAPEMENT */
-
-/* Define to 1 if you have the `Xutf8TextExtents' function. */
-/* #undef HAVE_XUTF8TEXTEXTENTS */
-
-/* Define to 1 if you have the `__argz_count' function. */
-/* #undef HAVE___ARGZ_COUNT */
-
-/* Define to 1 if you have the `__argz_next' function. */
-/* #undef HAVE___ARGZ_NEXT */
-
-/* Define to 1 if you have the `__argz_stringify' function. */
-/* #undef HAVE___ARGZ_STRINGIFY */
+/* Define if you have the X11/Xmu headers and libraries. */
+/* #undef HAVE_X11_Xmu */
 
 /* Define to 1 if you have the `__fsetlocking' function. */
 /* #undef HAVE___FSETLOCKING */
@@ -728,7 +749,7 @@
 #define HAVE___VA_COPY 1
 
 /* Define as const if the declaration of iconv() needs const. */
-/* #undef ICONV_CONST */
+#define ICONV_CONST const
 
 /* Define if `iconv' accepts "UTF-8", "latin1" and "UCS-*". */
 #define ICONV_LATIN1 1
@@ -751,6 +772,12 @@
 /* Define to disable Valgrind instrumentation */
 #define NVALGRIND 1
 
+/* Define if using GNU-style Objective C runtime. */
+/* #undef OBJC_GNU_RUNTIME */
+
+/* Define if using NeXT/Apple-style Objective C runtime. */
+/* #undef OBJC_NEXT_RUNTIME */
+
 /* Name of package */
 #define PACKAGE "R"
 
@@ -761,20 +788,26 @@
 #define PACKAGE_NAME "R"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "R 2.3.1"
+#define PACKAGE_STRING "R @VERSION@"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "R"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "2.3.1"
+#define PACKAGE_VERSION "@VERSION@"
 
 /* Define if <inttypes.h> exists and defines unusable PRI* macros. */
 /* #undef PRI_MACROS_BROKEN */
 
+/* Define if the pthread_in_use() detection is hard. */
+/* #undef PTHREAD_IN_USE_DETECTION_HARD */
+
 /* Define as the return type of signal handlers (`int' or `void'). */
 #define RETSIGTYPE void
 
+/* Define this to use architecture-dependent subdirectories of this name. */
+#define R_ARCH ""
+
 /* Define this to be the name of the CPU of your system. */
 #define R_CPU "i386"
 
@@ -782,6 +815,9 @@
    compiler calls it, or to nothing if it is not supported. */
 #define R_INLINE inline
 
+/* Define this to enable memory profiling. */
+/* #undef R_MEMORY_PROFILING */
+
 /* Define this to be the name of the OS of your system. */
 #define R_OS "mingw32"
 
@@ -804,25 +840,31 @@
    */
 #define SHLIB_EXT ".dll"
 
-/* The size of a `int', as computed by sizeof. */
+/* The size of `double', as computed by sizeof. */
+#define SIZEOF_DOUBLE 8
+
+/* The size of `int', as computed by sizeof. */
 #define SIZEOF_INT 4
 
-/* The size of a `long', as computed by sizeof. */
+/* The size of `long', as computed by sizeof. */
 #define SIZEOF_LONG 4
 
-/* The size of a `long double', as computed by sizeof. */
+/* The size of `long double', as computed by sizeof. */
 #define SIZEOF_LONG_DOUBLE 12
 
-/* The size of a `long long', as computed by sizeof. */
+/* The size of `long long', as computed by sizeof. */
 #define SIZEOF_LONG_LONG 8
 
+/* The size of `size_t', as computed by sizeof. */
+#define SIZEOF_SIZE_T 4
+
 /* Define as the maximum value of type 'size_t', if the system doesn't define
    it. */
 /* #undef SIZE_MAX */
 
 /* If using the C implementation of alloca, define if you know the
    direction of stack growth for your system; otherwise it will be
-   automatically deduced at run-time.
+   automatically deduced at runtime.
 	STACK_DIRECTION > 0 => grows toward higher addresses
 	STACK_DIRECTION < 0 => grows toward lower addresses
 	STACK_DIRECTION = 0 => direction of growth unknown */
@@ -840,10 +882,6 @@
 /* Define this to enable support for UTF-8 locales. */
 /* #undef SUPPORT_UTF8 */
 
-/* Define if your system is SunOS4, which is famous for broken header files.
-   */
-/* #undef SunOS4 */
-
 /* Define to enable provoking compile errors on write barrier violation. */
 /* #undef TESTING_WRITE_BARRIER */
 
@@ -854,6 +892,30 @@
    */
 /* #undef USE_BUILTIN_RINT */
 
+/* Define if the POSIX multithreading library can be used. */
+/* #define USE_POSIX_THREADS 1 */
+
+/* Define if references to the POSIX multithreading library should be made
+   weak. */
+/* #define USE_POSIX_THREADS_WEAK 1 */
+
+/* Define if the GNU Pth multithreading library can be used. */
+/* #undef USE_PTH_THREADS */
+
+/* Define if references to the GNU Pth multithreading library should be made
+   weak. */
+/* #undef USE_PTH_THREADS_WEAK */
+
+/* Define if the old Solaris multithreading library can be used. */
+/* #undef USE_SOLARIS_THREADS */
+
+/* Define if references to the old Solaris multithreading library should be
+   made weak. */
+/* #undef USE_SOLARIS_THREADS_WEAK */
+
+/* Define if the Win32 multithreading API can be used. */
+/* #undef USE_WIN32_THREADS */
+
 /* Define according to your operating system type. */
 /* #undef Unix */
 
@@ -861,7 +923,7 @@
 #define VALGRIND_LEVEL 0
 
 /* Version number of package */
-#define VERSION "2.3.1"
+#define VERSION "@VERSION@"
 
 /* Define to 1 if your processor stores words with the most significant byte
    first (like Motorola and SPARC, unlike Intel and VAX). */
@@ -876,6 +938,11 @@
 /* Number of bits in a file offset, on hosts where this is settable. */
 /* #undef _FILE_OFFSET_BITS */
 
+/* Enable GNU extensions on systems that have them.  */
+#ifndef _GNU_SOURCE
+# define _GNU_SOURCE 1
+#endif
+
 /* Define to 1 to make fseeko visible on some hosts (e.g. glibc 2.2). */
 /* #undef _LARGEFILE_SOURCE */
 
@@ -895,9 +962,6 @@
 /* #undef inline */
 #endif
 
-/* Define to `long' if <sys/types.h> does not define. */
-/* #undef off_t */
-
 /* Define to `int' if <sys/types.h> does not define. */
 /* #undef pid_t */
 
@@ -905,10 +969,7 @@
    doesn't define it. */
 /* #undef ptrdiff_t */
 
-/* Define to empty if the C compiler doesn't support this keyword. */
-/* #undef signed */
-
-/* Define to `unsigned' if <sys/types.h> does not define. */
+/* Define to `unsigned int' if <sys/types.h> does not define. */
 /* #undef size_t */
 
 /* Define to unsigned long or unsigned long long if <stdint.h> and
@@ -917,3 +978,35 @@
 
 
 #endif /* not R_CONFIG_H */
+
+
+#define __libc_lock_t                   gl_lock_t
+#define __libc_lock_define              gl_lock_define
+#define __libc_lock_define_initialized  gl_lock_define_initialized
+#define __libc_lock_init                gl_lock_init
+#define __libc_lock_lock                gl_lock_lock
+#define __libc_lock_unlock              gl_lock_unlock
+#define __libc_lock_recursive_t                   gl_recursive_lock_t
+#define __libc_lock_define_recursive              gl_recursive_lock_define
+#define __libc_lock_define_initialized_recursive  gl_recursive_lock_define_initialized
+#define __libc_lock_init_recursive                gl_recursive_lock_init
+#define __libc_lock_lock_recursive                gl_recursive_lock_lock
+#define __libc_lock_unlock_recursive              gl_recursive_lock_unlock
+#define glthread_in_use  libintl_thread_in_use
+#define glthread_lock_init     libintl_lock_init
+#define glthread_lock_lock     libintl_lock_lock
+#define glthread_lock_unlock   libintl_lock_unlock
+#define glthread_lock_destroy  libintl_lock_destroy
+#define glthread_rwlock_init     libintl_rwlock_init
+#define glthread_rwlock_rdlock   libintl_rwlock_rdlock
+#define glthread_rwlock_wrlock   libintl_rwlock_wrlock
+#define glthread_rwlock_unlock   libintl_rwlock_unlock
+#define glthread_rwlock_destroy  libintl_rwlock_destroy
+#define glthread_recursive_lock_init     libintl_recursive_lock_init
+#define glthread_recursive_lock_lock     libintl_recursive_lock_lock
+#define glthread_recursive_lock_unlock   libintl_recursive_lock_unlock
+#define glthread_recursive_lock_destroy  libintl_recursive_lock_destroy
+#define glthread_once                 libintl_once
+#define glthread_once_call            libintl_once_call
+#define glthread_once_singlethreaded  libintl_once_singlethreaded
+
