@@ -2,7 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  file preferences.h
  *  Copyright (C) 2000  Guido Masarotto and Brian Ripley
- *                2004-6  R Core Development Team
+ *                2004-7  R Core Development Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
  
 /*                configuration editor                        */
@@ -37,9 +37,11 @@ struct structGUI
 	cbb, cbl, grx, gry;
     rgb bg, fg, user, hlt;
     rect MDIsize;
+    char language[20];
+    int buffered;
 };
 typedef struct structGUI *Gui;
 
-int loadRconsole(Gui gui, char *optf);
+int loadRconsole(Gui gui, const char *optf);
 void getActive(Gui gui);
 void applyGUI(Gui newGUI);
