@@ -1,3 +1,19 @@
+#  File src/library/base/R/dates.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ## First shot at adding a "Date" class to base R.
 ## Representation is the number of whole days since 1970-01-01.
 
@@ -250,7 +266,7 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
         r1 <- as.POSIXlt(from)
         if(valid == 4) {
             if(missing(to)) { # years
-                yr <- seq.int(r1$year, by = by, length = length.out)
+                yr <- seq.int(r1$year, by = by, length.out = length.out)
             } else {
                 to <- as.POSIXlt(to)
                 yr <- seq.int(r1$year, to$year, by)
@@ -259,7 +275,7 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
             res <- .Internal(POSIXlt2Date(r1))
         } else if(valid == 3) { # months
             if(missing(to)) {
-                mon <- seq.int(r1$mon, by = by, length = length.out)
+                mon <- seq.int(r1$mon, by = by, length.out = length.out)
             } else {
                 to <- as.POSIXlt(to)
                 mon <- seq.int(r1$mon, 12*(to$year - r1$year) + to$mon, by)
@@ -297,13 +313,27 @@ cut.Date <-
 		start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
 	    incr <- 7
 	}
-	if(valid == 3) { start$mday <- 1; incr <- 31 }
-	if(valid == 4) { start$mon <- 0; incr <- 366 }
+	if(valid == 3) {
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        end <- as.POSIXlt(end + (31 * 86400))
+        end$mday <- 1
+        breaks <- as.Date(seq(start, end, "months"))
+    } else if(valid == 4) {
+        start$mon <- 0
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        end <- as.POSIXlt(end + (366 * 86400))
+        end$mon <- 0
+        end$mday <- 1
+        breaks <- as.Date(seq(start, end, "years"))
+    } else {
         start <- .Internal(POSIXlt2Date(start))
         if (length(by2) == 2) incr <- incr * as.integer(by2[1])
-	maxx <- max(x, na.rm = TRUE)
-	breaks <- seq.int(start, maxx + incr, breaks)
-	breaks <- breaks[1:(1+max(which(breaks < maxx)))]
+	    maxx <- max(x, na.rm = TRUE)
+        breaks <- seq.int(start, maxx + incr, breaks)
+        breaks <- breaks[1:(1+max(which(breaks < maxx)))]
+      }
     } else stop("invalid specification of 'breaks'")
     res <- cut(unclass(x), unclass(breaks), labels = labels,
                right = right, ...)
@@ -340,7 +370,7 @@ round.Date <- function(x, ...)
 }
 
 ## must avoid truncating dates prior to 1970-01-01 forwards.
-trunc.Date <- function(x) round(x - 0.4999999)
+trunc.Date <- function(x, ...) round(x - 0.4999999)
 
 rep.Date <- function(x, ...)
 {
@@ -364,3 +394,7 @@ diff.Date <- function (x, lag = 1, differences = 1, ...)
         r <- r[i1] - r[-length(r):-(length(r) - lag + 1)]
     r
 }
+
+# ---- additions in 2.6.0 -----
+
+is.numeric.Date <- function(x) FALSE
