@@ -1,3 +1,19 @@
+#  File src/library/base/R/datetime.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 Sys.time <- function()
     structure(.Internal(Sys.time()), class = c("POSIXt", "POSIXct"))
 
@@ -14,15 +30,15 @@ as.POSIXlt <- function(x, tz = "")
             if(is.na(xx)) f <- "%Y-%m-%d" # all NAs
         }
 	if(is.na(xx) ||
-           !is.na(strptime(xx, f <- "%Y-%m-%d %H:%M:%S")) ||
-	   !is.na(strptime(xx, f <- "%Y/%m/%d %H:%M:%S")) ||
+           !is.na(strptime(xx, f <- "%Y-%m-%d %H:%M:%OS")) ||
+	   !is.na(strptime(xx, f <- "%Y/%m/%d %H:%M:%OS")) ||
 	   !is.na(strptime(xx, f <- "%Y-%m-%d %H:%M")) ||
 	   !is.na(strptime(xx, f <- "%Y/%m/%d %H:%M")) ||
 	   !is.na(strptime(xx, f <- "%Y-%m-%d")) ||
 	   !is.na(strptime(xx, f <- "%Y/%m/%d")))
         {
 	    res <- strptime(x, f)
-            if(nchar(tz)) attr(res, "tzone") <- tz
+            if(nzchar(tz)) attr(res, "tzone") <- tz
             return(res)
         }
 	stop("character string is not in a standard unambiguous format")
@@ -32,12 +48,12 @@ as.POSIXlt <- function(x, tz = "")
     if(inherits(x, "Date")) return(.Internal(Date2POSIXlt(x)))
     tzone <- attr(x, "tzone")
     if(inherits(x, "date") || inherits(x, "dates")) x <- as.POSIXct(x)
-    if(is.character(x)) return(fromchar(x))
+    if(is.character(x)) return(fromchar(unclass(x))) # precaution PR7826
     if(is.factor(x))	return(fromchar(as.character(x)))
     if(is.logical(x) && all(is.na(x))) x <- as.POSIXct.default(x)
     if(!inherits(x, "POSIXct"))
-	stop(paste("Don't know how to convert `", deparse(substitute(x)),
-		   "' to class \"POSIXlt\"", sep=""))
+        stop(gettextf("do not know how to convert '%s' to class \"POSIXlt\"",
+                      deparse(substitute(x))))
     if(missing(tz) && !is.null(tzone)) tz <- tzone[1]
     .Internal(as.POSIXlt(x, tz))
 }
@@ -54,8 +70,8 @@ as.POSIXct.date <- function(x, ...)
     if(inherits(x, "date")) {
         x <- (x - 3653) * 86400 # origin 1960-01-01
         return(structure(x, class = c("POSIXt", "POSIXct")))
-    } else stop(paste("`", deparse(substitute(x)),
-                      "' is not a \"dates\" object", sep=""))
+    } else stop(gettextf("'%s' is not a \"date\" object",
+                         deparse(substitute(x)) ))
 }
 
 ## convert from package chron
@@ -67,8 +83,8 @@ as.POSIXct.dates <- function(x, ...)
         if(length(z) == 3 && is.numeric(z))
             x  <- x + as.numeric(ISOdate(z[3], z[1], z[2], 0))
         return(structure(x, class = c("POSIXt", "POSIXct")))
-    } else stop(paste("`", deparse(substitute(x)),
-                      "' is not a \"dates\" object", sep=""))
+    } else stop(gettextf("'%s' is not a \"dates\" object",
+                         deparse(substitute(x)) ))
 }
 
 as.POSIXct.POSIXlt <- function(x, tz = "")
@@ -86,26 +102,38 @@ as.POSIXct.default <- function(x, tz = "")
 	return(as.POSIXct(as.POSIXlt(x), tz))
     if(is.logical(x) && all(is.na(x)))
         return(structure(as.numeric(x), class = c("POSIXt", "POSIXct")))
-    stop(paste("Don't know how to convert `", deparse(substitute(x)),
-	       "' to class \"POSIXct\"", sep=""))
+    stop(gettextf("do not know how to convert '%s' to class \"POSIXlt\"",
+                  deparse(substitute(x))))
 }
 
+as.double.POSIXlt <- function(x, ...) as.POSIXct(x)
+
 format.POSIXlt <- function(x, format = "", usetz = FALSE, ...)
 {
     if(!inherits(x, "POSIXlt")) stop("wrong class")
     if(format == "") {
         ## need list [ method here.
         times <- unlist(unclass(x)[1:3])
+        secs <- x$sec; secs <- secs[!is.na(secs)]
+        np <- getOption("digits.secs")
+        if(is.null(np)) np <- 0 else np <- min(6, np)
+        if(np >= 1) {
+            for (i in (1:np)- 1) if(all( abs(secs - round(secs, i)) < 1e-6 )) {
+                np <- i
+                break
+            }
+        }
         format <- if(all(times[!is.na(times)] == 0)) "%Y-%m-%d"
-        else "%Y-%m-%d %H:%M:%S"
+        else if(np == 0) "%Y-%m-%d %H:%M:%S"
+        else paste("%Y-%m-%d %H:%M:%OS", np, sep="")
     }
     .Internal(format.POSIXlt(x, format, usetz))
 }
 
 strftime <- format.POSIXlt
 
-strptime <- function(x, format)
-    .Internal(strptime(x, format))
+strptime <- function(x, format, tz = "")
+    .Internal(strptime(as.character(x), format, tz))
 
 
 format.POSIXct <- function(x, format = "", tz = "", usetz = FALSE, ...)
@@ -152,12 +180,13 @@ summary.POSIXlt <- function(object, digits = 15, ...)
     if (nargs() == 1) return(e1)
     # only valid if one of e1 and e2 is a scalar.
     if(inherits(e1, "POSIXt") && inherits(e2, "POSIXt"))
-        stop("binary + is not defined for POSIXt objects")
+        stop("binary + is not defined for \"POSIXt\" objects")
     if(inherits(e1, "POSIXlt")) e1 <- as.POSIXct(e1)
     if(inherits(e2, "POSIXlt")) e2 <- as.POSIXct(e2)
     if (inherits(e1, "difftime")) e1 <- coerceTimeUnit(e1)
     if (inherits(e2, "difftime")) e2 <- coerceTimeUnit(e2)
-    structure(unclass(e1) + unclass(e2), class = c("POSIXt", "POSIXct"))
+    structure(unclass(e1) + unclass(e2), class = c("POSIXt", "POSIXct"),
+              tzone = check_tzones(e1, e2))
 }
 
 "-.POSIXt" <- function(e1, e2)
@@ -170,7 +199,7 @@ summary.POSIXlt <- function(object, digits = 15, ...)
     }
     if(!inherits(e1, "POSIXt"))
         stop("Can only subtract from POSIXt objects")
-    if (nargs() == 1) stop("unary - is not defined for POSIXt objects")
+    if (nargs() == 1) stop("unary - is not defined for \"POSIXt\" objects")
     if(inherits(e2, "POSIXt")) return(difftime(e1, e2))
     if (inherits(e2, "difftime")) e2 <- unclass(coerceTimeUnit(e2))
     if(!is.null(attr(e2, "class")))
@@ -181,36 +210,54 @@ summary.POSIXlt <- function(object, digits = 15, ...)
 Ops.POSIXt <- function(e1, e2)
 {
     if (nargs() == 1)
-        stop(paste("unary", .Generic, "not defined for POSIXt objects"))
+        stop("unary ", .Generic, " not defined for \"POSIXt\" objects")
     boolean <- switch(.Generic, "<" = , ">" = , "==" = ,
                       "!=" = , "<=" = , ">=" = TRUE, FALSE)
-    if (!boolean) stop(paste(.Generic, "not defined for POSIXt objects"))
-    if(inherits(e1, "POSIXlt")) e1 <- as.POSIXct(e1)
-    if(inherits(e2, "POSIXlt")) e2 <- as.POSIXct(e2)
+    if (!boolean) stop(.Generic, " not defined for \"POSIXt\" objects")
+    if(inherits(e1, "POSIXlt") || is.character(e1)) e1 <- as.POSIXct(e1)
+    if(inherits(e2, "POSIXlt") || is.character(e1)) e2 <- as.POSIXct(e2)
+    check_tzones(e1, e2)
     NextMethod(.Generic)
 }
 
 Math.POSIXt <- function (x, ...)
 {
-    stop(paste(.Generic, "not defined for POSIXt objects"))
+    stop(.Generic, " not defined for POSIXt objects")
 }
 
-Summary.POSIXct <- function (x, ...)
+check_tzones <- function(...)
+{
+    tzs <- unique(sapply(list(...), function(x) {
+        y <- attr(x, "tzone")
+        if(is.null(y)) "" else y
+    }))
+    tzs <- tzs[tzs != ""]
+    if(length(tzs) > 1)
+        warning("'tzone' attributes are inconsistent")
+    if(length(tzs)) tzs[1] else NULL
+}
+
+Summary.POSIXct <- function (..., na.rm)
 {
     ok <- switch(.Generic, max = , min = , range = TRUE, FALSE)
-    if (!ok) stop(paste(.Generic, "not defined for POSIXct objects"))
+    if (!ok) stop(.Generic, " not defined for \"POSIXct\" objects")
+    args <- list(...)
+    tz <- do.call("check_tzones", args)
     val <- NextMethod(.Generic)
-    class(val) <- oldClass(x)
+    class(val) <- oldClass(args[[1]])
+    attr(val, "tzone") <- tz
     val
 }
 
-Summary.POSIXlt <- function (x, ...)
+Summary.POSIXlt <- function (..., na.rm)
 {
     ok <- switch(.Generic, max = , min = , range = TRUE, FALSE)
-    if (!ok) stop(paste(.Generic, "not defined for POSIXlt objects"))
-    x <- as.POSIXct(x)
-    val <- NextMethod(.Generic)
-    as.POSIXlt(structure(val, class = c("POSIXt", "POSIXct")))
+    if (!ok) stop(.Generic, " not defined for \"POSIXlt\" objects")
+    args <- list(...)
+    tz <- do.call("check_tzones", args)
+    args <- lapply(args, as.POSIXct)
+    val <- do.call(.Generic, c(args, na.rm = na.rm))
+    as.POSIXlt(structure(val, class = c("POSIXt", "POSIXct"), tzone = tz))
 }
 
 "[.POSIXct" <-
@@ -220,6 +267,7 @@ function(x, ..., drop = TRUE)
     class(x) <- NULL
     val <- NextMethod("[")
     class(val) <- cl
+    attr(val, "tzone") <- attr(x, "tzone")
     val
 }
 
@@ -230,6 +278,7 @@ function(x, ..., drop = TRUE)
     class(x) <- NULL
     val <- NextMethod("[[")
     class(val) <- cl
+    attr(val, "tzone") <- attr(x, "tzone")
     val
 }
 
@@ -238,27 +287,25 @@ function(x, ..., value) {
     if(!as.logical(length(value))) return(x)
     value <- as.POSIXct(value)
     cl <- oldClass(x)
+    tz <- attr(x, "tzone")
     class(x) <- class(value) <- NULL
     x <- NextMethod(.Generic)
     class(x) <- cl
+    attr(x, "tzone") <- tz
     x
 }
 
 as.character.POSIXt <- function(x, ...) format(x, ...)
 
-str.POSIXt <- function(object, ...) {
-    cl <- oldClass(object)
-    cat("`", cl[min(2, length(cl))],"', format:", sep = "")
-    str(format(object), ...)
-}
-
 as.data.frame.POSIXct <- as.data.frame.vector
 
 is.na.POSIXlt <- function(x) is.na(as.POSIXct(x))
 
+## <FIXME> check the argument validity
+## This is documented to remove the timezone
 c.POSIXct <- function(..., recursive=FALSE)
     structure(c(unlist(lapply(list(...), unclass))),
-              class=c("POSIXt","POSIXct"))
+              class = c("POSIXt", "POSIXct"))
 
 ## we need conversion to POSIXct as POSIXlt objects can be in different tz.
 c.POSIXlt <- function(..., recursive=FALSE)
@@ -266,27 +313,29 @@ c.POSIXlt <- function(..., recursive=FALSE)
 
 ## force absolute comparisons
 all.equal.POSIXct <- function(target, current, ..., scale=1)
+{
+    check_tzones(target, current)
     NextMethod("all.equal")
-
+}
 
 
 ISOdatetime <- function(year, month, day, hour, min, sec, tz="")
 {
     x <- paste(year, month, day, hour, min, sec, sep="-")
-    as.POSIXct(strptime(x, "%Y-%m-%d-%H-%M-%S"), tz=tz)
+    as.POSIXct(strptime(x, "%Y-%m-%d-%H-%M-%OS", tz=tz), tz=tz)
 }
 
 ISOdate <- function(year, month, day, hour=12, min=0, sec=0, tz="GMT")
     ISOdatetime(year, month, day, hour, min, sec, tz)
 
-as.matrix.POSIXlt <- function(x)
+as.matrix.POSIXlt <- function(x, ...)
 {
-    as.matrix(as.data.frame(unclass(x)))
+    as.matrix(as.data.frame(unclass(x)), ...)
 }
 
 mean.POSIXct <- function (x, ...)
     structure(mean(unclass(x), ...), class = c("POSIXt", "POSIXct"),
-              tzone=attr(x, "tzone"))
+              tzone = attr(x, "tzone"))
 
 mean.POSIXlt <- function (x, ...)
     as.POSIXlt(mean(as.POSIXct(x), ...))
@@ -322,22 +371,59 @@ difftime <-
 
 ## "difftime" constructor
 ## Martin Maechler, Date: 16 Sep 2002
-as.difftime <- function(tim, format="%X")
+## Numeric input version Peter Dalgaard, December 2006
+as.difftime <- function(tim, format="%X", units="auto")
 {
-    difftime(strptime(tim, format=format),
-             strptime("0:0:0", format="%X"))
+    if (inherits(tim, "difftime")) return(tim)
+    if (is.character(tim)){
+        difftime(strptime(tim, format=format),
+             strptime("0:0:0", format="%X"), units=units)
+    } else {
+        if (!is.numeric(tim)) stop("'tim' is not character or numeric")
+	if (units=="auto") stop("need explicit units for numeric conversion")
+        if (!(units %in% c("secs", "mins", "hours", "days", "weeks")))
+	    stop("invalid units specified")
+        structure(tim, units=units, class="difftime")
+    }
 }
 
+### For now, these have only difftime methods, but you never know...
+units <- function(x) UseMethod("units")
+
+"units<-" <- function(x, value) UseMethod("units<-")
+
+units.difftime <- function(x) attr(x, "units")
+
+"units<-.difftime" <- function(x, value)
+{
+    from <- units(x)
+    if (from == value) return(x)
+    if (!(value %in% c("secs", "mins", "hours", "days", "weeks")))
+        stop("invalid units specified")
+    sc <- cumprod(c(secs=1, mins=60, hours=60, days=24, weeks=7))
+    newx <- as.vector(x)*sc[from]/sc[value]
+    structure(newx, units=value, class="difftime")
+}
+
+as.double.difftime <- function(x, units="auto", ...) {
+    if (units != "auto")
+        units(x) <- units
+    as.double(as.vector(x))
+}
+
+as.data.frame.difftime <- as.data.frame.vector
+
+format.difftime <- function(x,...) paste(format(unclass(x),...), units(x))
+
+
+
 print.difftime <- function(x, digits = getOption("digits"), ...)
 {
-    if(is.array(x)) {
+    if(is.array(x) || length(x) > 1) {
         cat("Time differences in ", attr(x, "units"), "\n", sep="")
         y <- unclass(x); attr(y, "units") <- NULL
         print(y)
-    } else if(length(x) > 1)
-        cat("Time differences of ",
-            paste(format(unclass(x), digits=digits), collapse = ", "), " ",
-            attr(x, "units"), "\n", sep="")
+    }
     else
         cat("Time difference of ", format(unclass(x), digits=digits), " ",
             attr(x, "units"), "\n", sep="")
@@ -345,7 +431,7 @@ print.difftime <- function(x, digits = getOption("digits"), ...)
     invisible(x)
 }
 
-round.difftime <- function (x, digits = 0)
+round.difftime <- function (x, digits = 0, ...)
 {
    units <- attr(x, "units")
    structure(NextMethod(), units=units, class="difftime")
@@ -365,12 +451,16 @@ Ops.difftime <- function(e1, e2)
 {
     coerceTimeUnit <- function(x)
     {
-        switch(attr(x,"units"),
+        switch(attr(x, "units"),
                secs = x, mins = 60*x, hours = 60*60*x,
                days = 60*60*24*x, weeks = 60*60*24*7*x)
     }
-    if (nargs() == 1)
-        stop(paste("unary", .Generic, "not defined for difftime objects"))
+    if (nargs() == 1) {
+        switch(.Generic, "+"= {}, "-" = {e1[] <- -unclass(e1)},
+               stop("unary ", .Generic, " not defined for \"difftime\" objects")
+               )
+        return(e1)
+    }
     boolean <- switch(.Generic, "<" = , ">" = , "==" = ,
                       "!=" = , "<=" = , ">=" = TRUE, FALSE)
     if (boolean) {
@@ -396,8 +486,8 @@ Ops.difftime <- function(e1, e2)
             structure(NextMethod(.Generic), units="secs", class="difftime")
         }
     } else {
-        ## `*' is covered by a specific method
-        stop(paste(.Generic, "not defined for difftime objects"))
+        ## '*' is covered by a specific method
+        stop(.Generic, " not defined for \"difftime\" objects")
     }
 }
 
@@ -405,7 +495,7 @@ Ops.difftime <- function(e1, e2)
 {
     ## need one scalar, one difftime.
     if(inherits(e1, "difftime") && inherits(e2, "difftime"))
-        stop("both arguments of * cannot be difftime objects")
+        stop("both arguments of * cannot be \"difftime\" objects")
     if(inherits(e2, "difftime")) {tmp <- e1; e1 <- e2; e2 <- tmp}
     structure(e2 * unclass(e1), units = attr(e1, "units"),
               class = "difftime")
@@ -415,14 +505,14 @@ Ops.difftime <- function(e1, e2)
 {
     ## need one scalar, one difftime.
     if(inherits(e2, "difftime"))
-        stop("second argument of / cannot be a difftime object")
+        stop("second argument of / cannot be a \"difftime\" object")
     structure(unclass(e1) / e2, units = attr(e1, "units"),
               class = "difftime")
 }
 
 Math.difftime <- function (x, ...)
 {
-    stop(paste(.Generic, "not defined for difftime objects"))
+    stop(.Generic, " not defined for \"difftime\" objects")
 }
 
 mean.difftime <- function (x, ..., na.rm = FALSE)
@@ -442,7 +532,7 @@ mean.difftime <- function (x, ..., na.rm = FALSE)
     }
 }
 
-Summary.difftime <- function (x, ..., na.rm = FALSE)
+Summary.difftime <- function (..., na.rm)
 {
     coerceTimeUnit <- function(x)
     {
@@ -451,8 +541,8 @@ Summary.difftime <- function (x, ..., na.rm = FALSE)
                          days = 60*60*24*x, weeks = 60*60*24*7*x))
     }
     ok <- switch(.Generic, max = , min = , range = TRUE, FALSE)
-    if (!ok) stop(paste(.Generic, "not defined for difftime objects"))
-    args <- c(lapply(list(x, ...), coerceTimeUnit), na.rm = na.rm)
+    if (!ok) stop(.Generic, " not defined for \"difftime\" objects")
+    args <- c(lapply(list(...), coerceTimeUnit), na.rm = na.rm)
     structure(do.call(.Generic, args), units="secs", class="difftime")
 }
 
@@ -462,34 +552,35 @@ Summary.difftime <- function (x, ..., na.rm = FALSE)
 seq.POSIXt <-
     function(from, to, by, length.out = NULL, along.with = NULL, ...)
 {
-    if (missing(from)) stop("`from` must be specified")
-    if (!inherits(from, "POSIXt")) stop("`from' must be a POSIXt object")
-        if(length(as.POSIXct(from)) != 1) stop("`from' must be of length 1")
+    if (missing(from)) stop("'from' must be specified")
+    if (!inherits(from, "POSIXt")) stop("'from' must be a POSIXt object")
+    cfrom <- as.POSIXct(from)
+    if(length(cfrom) != 1) stop("'from' must be of length 1")
+    tz <- attr(cfrom , 'tzone')
     if (!missing(to)) {
-        if (!inherits(to, "POSIXt")) stop("`to' must be a POSIXt object")
-        if (length(as.POSIXct(to)) != 1) stop("`to' must be of length 1")
-        if (to <= from) stop("`to' must be later than `from'")
+        if (!inherits(to, "POSIXt")) stop("'to' must be a POSIXt object")
+        if (length(as.POSIXct(to)) != 1) stop("'to' must be of length 1")
     }
     if (!missing(along.with)) {
         length.out <- length(along.with)
-    }  else if (!missing(length.out)) {
-        if (length(length.out) != 1) stop("`length.out' must be of length 1")
+    }  else if (!is.null(length.out)) {
+        if (length(length.out) != 1) stop("'length.out' must be of length 1")
         length.out <- ceiling(length.out)
     }
     status <- c(!missing(to), !missing(by), !is.null(length.out))
     if(sum(status) != 2)
-        stop("exactly two of `to', `by' and `length.out' / `along.with' must be specified")
+        stop("exactly two of 'to', 'by' and 'length.out' / 'along.with' must be specified")
     if (missing(by)) {
-        from <- unclass(as.POSIXct(from))
+        from <- unclass(cfrom)
         to <- unclass(as.POSIXct(to))
         ## Till (and incl.) 1.6.0 :
         ##- incr <- (to - from)/length.out
         ##- res <- seq.default(from, to, incr)
-        res <- seq.default(from, to, length.out = length.out)
-        return(structure(res, class = c("POSIXt", "POSIXct")))
+        res <- seq.int(from, to, length.out = length.out)
+        return(structure(res, class = c("POSIXt", "POSIXct"), tzone=tz))
     }
 
-    if (length(by) != 1) stop("`by' must be of length 1")
+    if (length(by) != 1) stop("'by' must be of length 1")
     valid <- 0
     if (inherits(by, "difftime")) {
         by <- switch(attr(by,"units"), secs = 1, mins = 60, hours = 3600,
@@ -497,47 +588,47 @@ seq.POSIXt <-
     } else if(is.character(by)) {
         by2 <- strsplit(by, " ", fixed=TRUE)[[1]]
         if(length(by2) > 2 || length(by2) < 1)
-            stop("invalid `by' string")
+            stop("invalid 'by' string")
         valid <- pmatch(by2[length(by2)],
                         c("secs", "mins", "hours", "days", "weeks",
                           "months", "years", "DSTdays"))
-        if(is.na(valid)) stop("invalid string for `by'")
+        if(is.na(valid)) stop("invalid string for 'by'")
         if(valid <= 5) {
             by <- c(1, 60, 3600, 86400, 7*86400)[valid]
             if (length(by2) == 2) by <- by * as.integer(by2[1])
         } else
             by <- if(length(by2) == 2) as.integer(by2[1]) else 1
-    } else if(!is.numeric(by)) stop("invalid mode for `by'")
-    if(is.na(by)) stop("`by' is NA")
+    } else if(!is.numeric(by)) stop("invalid mode for 'by'")
+    if(is.na(by)) stop("'by' is NA")
 
     if(valid <= 5) {
         from <- unclass(as.POSIXct(from))
         if(!is.null(length.out))
-            res <- seq.default(from, by=by, length.out=length.out)
+            res <- seq.int(from, by=by, length.out=length.out)
         else {
             to <- unclass(as.POSIXct(to))
             ## defeat test in seq.default
-            res <- seq.default(0, to - from, by) + from
+            res <- seq.int(0, to - from, by) + from
         }
-        return(structure(res, class=c("POSIXt", "POSIXct")))
+        return(structure(res, class=c("POSIXt", "POSIXct"), tzone=tz))
     } else {  # months or years or DSTdays
         r1 <- as.POSIXlt(from)
         if(valid == 7) {
             if(missing(to)) { # years
-                yr <- seq(r1$year, by = by, length = length.out)
+                yr <- seq.int(r1$year, by = by, length.out = length.out)
             } else {
                 to <- as.POSIXlt(to)
-                yr <- seq(r1$year, to$year, by)
+                yr <- seq.int(r1$year, to$year, by)
             }
             r1$year <- yr
             r1$isdst <- -1
             res <- as.POSIXct(r1)
         } else if(valid == 6) { # months
             if(missing(to)) {
-                mon <- seq(r1$mon, by = by, length = length.out)
+                mon <- seq.int(r1$mon, by = by, length.out = length.out)
             } else {
                 to <- as.POSIXlt(to)
-                mon <- seq(r1$mon, 12*(to$year - r1$year) + to$mon, by)
+                mon <- seq.int(r1$mon, 12*(to$year - r1$year) + to$mon, by)
             }
             r1$mon <- mon
             r1$isdst <- -1
@@ -548,7 +639,7 @@ seq.POSIXt <-
                 length.out <- 2 + floor((unclass(as.POSIXct(to)) -
                                          unclass(as.POSIXct(from)))/86400)
             }
-            r1$mday <- seq(r1$mday, by = by, length = length.out)
+            r1$mday <- seq.int(r1$mday, by = by, length.out = length.out)
             r1$isdst <- -1
             res <- as.POSIXct(r1)
             ## now correct if necessary.
@@ -562,7 +653,7 @@ cut.POSIXt <-
     function (x, breaks, labels = NULL, start.on.monday = TRUE,
               right = FALSE, ...)
 {
-    if(!inherits(x, "POSIXt")) stop("`x' must be a date-time object")
+    if(!inherits(x, "POSIXt")) stop("'x' must be a date-time object")
     x <- as.POSIXct(x)
 
     if (inherits(breaks, "POSIXt")) {
@@ -572,12 +663,12 @@ cut.POSIXt <-
     } else if(is.character(breaks) && length(breaks) == 1) {
         by2 <- strsplit(breaks, " ", fixed=TRUE)[[1]]
         if(length(by2) > 2 || length(by2) < 1)
-            stop("invalid specification of `breaks'")
+            stop("invalid specification of 'breaks'")
 	valid <-
 	    pmatch(by2[length(by2)],
 		   c("secs", "mins", "hours", "days", "weeks",
 		     "months", "years", "DSTdays"))
-	if(is.na(valid)) stop("invalid specification of `breaks'")
+	if(is.na(valid)) stop("invalid specification of 'breaks'")
 	start <- as.POSIXlt(min(x, na.rm=TRUE))
 	incr <- 1
 	if(valid > 1) { start$sec <- 0; incr <- 59.99 }
@@ -589,14 +680,28 @@ cut.POSIXt <-
 		start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
 	    incr <- 7*86400
 	}
-	if(valid == 6) { start$mday <- 1; incr <- 31*86400 }
-	if(valid == 7) { start$mon <- 0; incr <- 366*86400 }
         if(valid == 8) incr <- 25*3600
+	if(valid == 6) {
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        end <- as.POSIXlt(end + (31 * 86400))
+        end$mday <- 1
+        breaks <- seq(start, end, "months")
+    } else if(valid == 7) {
+        start$mon <- 0
+        start$mday <- 1
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        end <- as.POSIXlt(end + (366 * 86400))
+        end$mon <- 0
+        end$mday <- 1
+        breaks <- seq(start, end, "years")
+    } else {
         if (length(by2) == 2) incr <- incr * as.integer(by2[1])
-	maxx <- max(x, na.rm = TRUE)
-	breaks <- seq(start, maxx + incr, breaks)
-	breaks <- breaks[1:(1+max(which(breaks < maxx)))]
-    } else stop("invalid specification of `breaks'")
+	    maxx <- max(x, na.rm = TRUE)
+        breaks <- seq.int(start, maxx + incr, breaks)
+        breaks <- breaks[1:(1+max(which(breaks < maxx)))]
+      }
+    } else stop("invalid specification of 'breaks'")
     res <- cut(unclass(x), unclass(breaks), labels = labels,
                right = right, ...)
     if(is.null(labels)) levels(res) <- as.character(breaks[-length(breaks)])
@@ -607,7 +712,8 @@ julian <- function(x, ...) UseMethod("julian")
 
 julian.POSIXt <- function(x, origin = as.POSIXct("1970-01-01", tz="GMT"), ...)
 {
-    if(length(origin) != 1) stop("`origin' must be of length one")
+    origin <- as.POSIXct(origin)
+    if(length(origin) != 1) stop("'origin' must be of length one")
     res <- difftime(as.POSIXct(x), origin, units = "days")
     structure(res, "origin" = origin)
 }
@@ -631,7 +737,7 @@ quarters.POSIXt <- function(x, ...)
     paste("Q", x+1, sep = "")
 }
 
-trunc.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"))
+trunc.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"), ...)
 {
     units <- match.arg(units)
     x <- as.POSIXlt(x)
@@ -676,9 +782,9 @@ round.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"))
     x
 }
 
-as.data.frame.POSIXlt <- function(x, row.names = NULL, optional = FALSE)
+as.data.frame.POSIXlt <- function(x, row.names = NULL, optional = FALSE, ...)
 {
-    value <- as.data.frame.POSIXct(as.POSIXct(x), row.names, optional)
+    value <- as.data.frame.POSIXct(as.POSIXct(x), row.names, optional, ...)
     if (!optional)
         names(value) <- deparse(substitute(x))[[1]]
     value
@@ -686,16 +792,15 @@ as.data.frame.POSIXlt <- function(x, row.names = NULL, optional = FALSE)
 
 # ---- additions in 1.8.0 -----
 
-rep.POSIXct <- function(x, times,  ...)
+rep.POSIXct <- function(x, ...)
 {
     y <- NextMethod()
     structure(y, class=c("POSIXt", "POSIXct"), tzone = attr(x, "tzone"))
 }
 
-rep.POSIXlt <- function(x, times, ...)
+rep.POSIXlt <- function(x, ...)
 {
-    y <- if(missing(times)) lapply(x, rep, ...)
-       else lapply(x, rep, times=times, ...)
+    y <- lapply(x, rep, ...)
     attributes(y) <- attributes(x)
     y
 }
@@ -703,12 +808,12 @@ rep.POSIXlt <- function(x, times, ...)
 diff.POSIXt <- function (x, lag = 1, differences = 1, ...)
 {
     ismat <- is.matrix(x)
-    xlen <- if (ismat) dim(x)[1] else length(x)
+    r <- if(inherits(x, "POSIXlt")) as.POSIXct(x) else x
+    xlen <- if (ismat) dim(x)[1] else length(r)
     if (length(lag) > 1 || length(differences) > 1 || lag < 1 || differences < 1)
-        stop("`lag' and `differences' must be integers >= 1")
+        stop("'lag' and 'differences' must be integers >= 1")
     if (lag * differences >= xlen)
         return(structure(numeric(0), class="difftime", units="secs"))
-    r <- x
     i1 <- -1:-lag
     if (ismat) for (i in 1:differences) r <- r[i1, , drop = FALSE] -
             r[-nrow(r):-(nrow(r) - lag + 1), , drop = FALSE]
@@ -716,3 +821,24 @@ diff.POSIXt <- function (x, lag = 1, differences = 1, ...)
         r <- r[i1] - r[-length(r):-(length(r) - lag + 1)]
     r
 }
+
+# ---- additions in 2.2.0 -----
+
+duplicated.POSIXlt <- function(x, incomparables = FALSE, ...)
+{
+    x <- as.POSIXct(x)
+    NextMethod("duplicated", x)
+}
+
+unique.POSIXlt <- function(x, incomparables = FALSE, ...)
+    x[!duplicated(x, incomparables, ...)]
+
+# ---- additions in 2.4.0 -----
+
+sort.POSIXlt <- function(x, decreasing = FALSE, na.last = NA, ...)
+    x[order(as.POSIXct(x), na.last = na.last, decreasing = decreasing)]
+
+
+# ---- additions in 2.6.0 -----
+
+is.numeric.POSIXt <- function(x) FALSE
