@@ -1,8 +1,23 @@
-factor <- function (x, levels = sort(unique.default(x), na.last = TRUE),
-		    labels=levels, exclude = NA, ordered = is.ordered(x))
+#  File src/library/base/R/factor.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
+factor <- function (x = character(), levels = sort(unique.default(x),
+                    na.last = TRUE), labels=levels, exclude = NA,
+                    ordered = is.ordered(x))
 {
-    if(is.null(x))
-	x <- list()
     exclude <- as.vector(exclude, typeof(x))
     levels <- levels[is.na(match(levels, exclude))]
     f <- match(x, levels)
@@ -12,7 +27,7 @@ factor <- function (x, levels = sort(unique.default(x), na.last = TRUE),
 	if (nl == length(levels))
 	    as.character(labels)
 	else if(nl == 1)
-	    paste(labels, seq(along = levels), sep = "")
+	    paste(labels, seq_along(levels), sep = "")
 	else
 	    stop(gettextf("invalid labels; length %d should be 1 or %d",
                           nl, length(levels)), domain = NA)
@@ -26,16 +41,18 @@ as.factor <- function(x) if (is.factor(x)) x else factor(x)
 ## Help old S users:
 category <- function(...) .Defunct()
 
-levels <- function(x) attr(x, "levels")
+levels <- function(x) UseMethod("levels")
+levels.default <- function(x) attr(x, "levels")
 nlevels <- function(x) length(levels(x))
 
-"levels<-" <- function(x, value) UseMethod("levels<-")
+## now a primitive
+## "levels<-" <- function(x, value) UseMethod("levels<-")
 
-"levels<-.default" <- function(x, value)
-{
-    attr(x, "levels") <- value
-    x
-}
+## "levels<-.default" <- function(x, value)
+## {
+##     attr(x, "levels") <- value
+##     x
+## }
 
 "levels<-.factor" <- function(x, value)
 {
@@ -63,17 +80,20 @@ nlevels <- function(x) length(levels(x))
 
 as.vector.factor <- function(x, mode="any")
 {
-    if(mode== "any" || mode== "character" || mode== "logical" || mode== "list")
+    if(mode=="list") as.list(x)
+    else if(mode== "any" || mode== "character" || mode== "logical")
 	as.vector(levels(x)[x], mode)
     else
 	as.vector(unclass(x), mode)
 }
 
-as.character.factor <- function(x,...)
+as.character.factor <- function(x,...) levels(x)[x]
+
+as.list.factor <- function(x,...)
 {
-    cx <- levels(x)[x]
-    if("NA" %in% levels(x)) cx[is.na(x)] <- "<NA>"
-    cx
+    res <- vector("list", length(x))
+    for(i in seq_along(x)) res[[i]] <- x[i]
+    res
 }
 
 ## for `factor' *and* `ordered' :
@@ -82,9 +102,16 @@ print.factor <- function (x, quote = FALSE, max.levels = NULL,
 {
     ord <- is.ordered(x)
     if (length(x) <= 0)
-        cat(if(ord)"ordered" else "factor","(0)\n",sep="")
-    else
-        print(as.character(x), quote = quote, ...)
+        cat(if(ord)"ordered" else "factor", "(0)\n", sep = "")
+    else {
+        ## The idea here is to preserve all relevant attributes such as
+        ## names and dims
+        xx <- x
+        class(xx) <- NULL
+        levels(xx) <- NULL
+        xx[] <- as.character(x)
+        print(xx, quote = quote, ...)
+    }
     maxl <- if(is.null(max.levels)) TRUE else max.levels
     if (maxl) {
         n <- length(lev <- encodeString(levels(x), quote=ifelse(quote, '"', '')))
@@ -92,9 +119,9 @@ print.factor <- function (x, quote = FALSE, max.levels = NULL,
         T0 <- "Levels: "
         if(is.logical(maxl))
             maxl <- { ## smart default
-                width <- width - (nchar(T0, type="w") + 3 + 1 + 3)
+                width <- width - (nchar(T0, "w") + 3 + 1 + 3)
                                         # 3='...', 3=#lev, 1=extra
-                lenl <- cumsum(nchar(lev, type="w") + nchar(colsep, type="w"))
+                lenl <- cumsum(nchar(lev, "w") + nchar(colsep, "w"))
                 if(n <= 1 || lenl[n] <= width) n
                 else max(1, which(lenl > width)[1] - 1)
             }
@@ -110,7 +137,7 @@ print.factor <- function (x, quote = FALSE, max.levels = NULL,
 Math.factor <- function(x, ...) {
     stop(.Generic, " not meaningful for factors")
 }
-Summary.factor <- function(x, ...) {
+Summary.factor <- function(..., na.rm) {
     stop(.Generic, " not meaningful for factors")
 }
 Ops.factor <- function(e1, e2)
@@ -121,33 +148,33 @@ Ops.factor <- function(e1, e2)
 	return(rep.int(NA, max(length(e1), if(!missing(e2))length(e2))))
     }
     nas <- is.na(e1) | is.na(e2)
-    if (nchar(.Method[1])) {
+    if (nzchar(.Method[1])) {
 	l1 <- levels(e1)
 	e1 <- l1[e1]
     }
-    if (nchar(.Method[2])) {
+    if (nzchar(.Method[2])) {
 	l2 <- levels(e2)
 	e2 <- l2[e2]
     }
-    if (all(nchar(.Method)) && (length(l1) != length(l2) ||
-				!all(sort(l2) == sort(l1))))
+    if (all(nzchar(.Method) > 0) &&
+        (length(l1) != length(l2) || !all(sort.int(l2) == sort.int(l1))))
 	stop("level sets of factors are different")
     value <- NextMethod(.Generic)
     value[nas] <- NA
     value
 }
 
-"[.factor" <- function(x, i, drop=FALSE)
+"[.factor" <- function(x, ..., drop = FALSE)
 {
     y <- NextMethod("[")
     attr(y,"contrasts")<-attr(x,"contrasts")
     ## NB factor has levels before class in attribute list (PR#6799)
-    attr(y,"levels")<-attr(x,"levels")
+    attr(y,"levels") <- attr(x,"levels")
     class(y) <- oldClass(x)
     if ( drop ) factor(y) else y
 }
 
-"[<-.factor" <- function(x, i, value)
+"[<-.factor" <- function(x, ..., value)
 {
     lx <- levels(x)
     cx <- oldClass(x)
@@ -158,14 +185,20 @@ Ops.factor <- function(e1, e2)
     if (any(is.na(m) & !is.na(value)))
 	warning("invalid factor level, NAs generated")
     class(x) <- NULL
-    if (missing(i))
-	x[] <- m
-    else
-        x[i] <- m
+    x[...] <- m
     attr(x,"levels") <- lx
     class(x) <- cx
     x
 }
+"[[.factor" <- function(x, ...)
+{
+    y <- NextMethod("[[")
+    attr(y,"contrasts") <- attr(x,"contrasts")
+    ## NB factor has levels before class in attribute list (PR#6799)
+    attr(y,"levels") <- attr(x,"levels")
+    class(y) <- oldClass(x)
+    y
+}
 
 ## ordered factors ...
 
@@ -190,15 +223,16 @@ function (e1, e2)
     nas <- is.na(e1) | is.na(e2)
     ord1 <- FALSE
     ord2 <- FALSE
-    if (nchar(.Method[1])) {
+    if (nzchar(.Method[1])) {
 	l1 <- levels(e1)
 	ord1 <- TRUE
     }
-    if (nchar(.Method[2])) {
+    if (nzchar(.Method[2])) {
 	l2 <- levels(e2)
 	ord2 <- TRUE
     }
-    if (all(nchar(.Method)) && (length(l1) != length(l2) || !all(l2 == l1)))
+    if (all(nzchar(.Method)) &&
+        (length(l1) != length(l2) || !all(l2 == l1)))
 	stop("level sets of factors are different")
     if (ord1 && ord2) {
 	e1 <- as.integer(e1) # was codes, but same thing for ordered factor.
