@@ -1,9 +1,30 @@
-strtrim <- function(x, width) .Internal(strtrim(x, width))
+#  File src/library/base/R/strwrap.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
+strtrim <- function(x, width)
+{
+    if(!is.character(x)) x <- as.character(x)
+    .Internal(strtrim(x, width))
+}
 
 strwrap <-
 function(x, width = 0.9 * getOption("width"), indent = 0, exdent = 0,
-         prefix = "", simplify = TRUE) {
-
+         prefix = "", simplify = TRUE)
+{
+    if(!is.character(x)) x <- as.character(x)
     ## Useful variables.
     indentString <- paste(rep.int(" ", indent), collapse = "")
     exdentString <- paste(rep.int(" ", exdent), collapse = "")
@@ -12,15 +33,15 @@ function(x, width = 0.9 * getOption("width"), indent = 0, exdent = 0,
     ## Now z[[i]][[j]] is a character vector of all "words" in
     ## paragraph j of x[i].
 
-    for(i in seq(along = z)) {
+    for(i in seq_along(z)) {
         yi <- character(0)
-        for(j in seq(along = z[[i]])) {
+        for(j in seq_along(z[[i]])) {
             ## Format paragraph j in x[i].
             words <- z[[i]][[j]]
             nc <- nchar(words, type="w")
 	    if(any(is.na(nc))) {
 		## use byte count as a reasonable substitute
-		nc0 <- nchar(words)
+		nc0 <- nchar(words, type="b")
 		nc[is.na(nc)] <- nc0[is.na(nc)]
 	    }
 
@@ -53,7 +74,7 @@ function(x, width = 0.9 * getOption("width"), indent = 0, exdent = 0,
             ## such that the words in line k are the ones in the k-th
             ## index block.
             while(length(lens) > 0) {
-                k <- max(sum(lens < maxLength), 1)
+                k <- max(sum(lens <= maxLength), 1)
                 if(first) {
                     first <- FALSE
                     maxLength <- maxLength + indent - exdent
@@ -92,7 +113,10 @@ function(x, width = 0.9 * getOption("width"), indent = 0, exdent = 0,
                               sep = "")
             yi <- c(yi, s, prefix)
         }
-        y <- c(y, list(yi[-length(yi)]))
+        y <- if(length(yi))
+            c(y, list(yi[-length(yi)]))
+        else
+            c(y, "")
     }
 
     if(simplify) y <- unlist(y)
