@@ -1,10 +1,8 @@
-require(graphics)
-if(dev.cur() <= 1) get(getOption("device"))()
+#### --- Hershey Vector Fonts ---
 
-opar <- par(ask = interactive() &&
-            (.Device %in% c("X11", "GTK", "gnome", "windows", "quartz")))
+require(grDevices); require(graphics)
+opar <- par(ask = dev.interactive(orNone = TRUE))
 
-## --- Hershey Vector Fonts
 
 ######
 # create tables of vector font functionality
