@@ -1,9 +1,7 @@
-require(graphics); require(datasets)
-if(dev.cur() <= 1) get(getOption("device"))()
+require(datasets)
+require(grDevices); require(graphics)
+opar <- par(ask = dev.interactive(orNone = TRUE))
 
-opar <- par(ask = interactive() &&
-            (.Device %in% c("X11", "GTK", "windows","quartz")))
-# For source
 
 x <- 10*(1:nrow(volcano)); x.at <- seq(100, 800, by=100)
 y <- 10*(1:ncol(volcano)); y.at <- seq(100, 600, by=100)
