@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 2001-3 Paul Murrell
- *                2003-5 The R Development Core Team
+ *                2003-7 The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,13 +13,15 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #include "grid.h"
+#include <math.h>
+#include <float.h>
+#include <string.h>
 
 int isUnitArithmetic(SEXP ua) {
     return inherits(ua, "unit.arithmetic");
@@ -35,15 +37,12 @@ int isUnitList(SEXP ul) {
 SEXP unit(double value, int unit) 
 {
     SEXP u, units, classname;
-    PROTECT(u = allocVector(REALSXP, 1));
-    REAL(u)[0] = value;
-    PROTECT(units = allocVector(INTSXP, 1));
-    INTEGER(units)[0] = unit;
+    PROTECT(u = ScalarReal(value));
+    PROTECT(units = ScalarInteger(unit));
     /* NOTE that we do not set the "unit" attribute */
     setAttrib(u, install("valid.unit"), units);
     setAttrib(u, install("data"), R_NilValue);
-    PROTECT(classname = allocVector(STRSXP, 1));
-    SET_STRING_ELT(classname, 0, mkChar("unit"));
+    PROTECT(classname = mkString("unit"));
     classgets(u, classname);
     UNPROTECT(3);
     return u;
@@ -77,18 +76,21 @@ SEXP unitData(SEXP unit, int index) {
     SEXP data = getAttrib(unit, install("data"));
     if (isNull(data))
 	result = R_NilValue;
-    else {
+    else if(TYPEOF(data) == VECSXP) {
 	/* Recycle data if necessary 
 	 */
 	int n = LENGTH(data);
 	result = VECTOR_ELT(data, index % n);
+    } else {
+	warning("unit attribute 'data' is of incorrect type");
+	return R_NilValue;
     }
     return result;
 }
 
 /* Accessor functions for unit arithmetic object
  */
-char* fName(SEXP ua) {
+const char* fName(SEXP ua) {
     return CHAR(STRING_ELT(getListElement(ua, "fname"), 0));
 }
 
@@ -257,7 +259,12 @@ double pureNullUnitValue(SEXP unit, int index)
 	else 
 	    error(_("Unimplemented unit function"));
     } else if (isUnitList(unit)) {
-	result = unitValue(VECTOR_ELT(unit, index), 0);
+	/*
+	 * Recycle if necessary;  it is up to the calling code
+	 * to limit indices to unit length if desired
+	 */
+	int n = unitLength(unit);
+	result = unitValue(VECTOR_ELT(unit, index % n), 0);
     } else
 	result = unitValue(unit, index);
     return result;
@@ -270,7 +277,12 @@ int pureNullUnit(SEXP unit, int index, GEDevDesc *dd) {
     if (isUnitArithmetic(unit)) 
 	result = pureNullUnitArithmetic(unit, index, dd);
     else if (isUnitList(unit)) {
-	result = pureNullUnit(VECTOR_ELT(unit, index), 0, dd);
+	/*
+	 * Recycle if necessary;  it is up to the calling code
+	 * to limit indices to unit length if desired
+	 */
+	int n = unitLength(unit);
+	result = pureNullUnit(VECTOR_ELT(unit, index % n), 0, dd);
     } else {  /* Just a plain unit */
 	/* Special case:  if "grobwidth" or "grobheight" unit
 	 * and width/height(grob) is pure null
@@ -408,24 +420,52 @@ int pureNullUnitArithmetic(SEXP unit, int index, GEDevDesc *dd) {
  * set.gpar operations, which will impact on grid state variables
  * BUT that's ok(ish) because we save and restore the relevant state
  * variables in here so that the overall effect is NULL.
+ *
+ * FIXME:  OTOH, the calls back to R Code may also perform
+ * viewport operations.  Again, we restore state as much as possible,
+ * but this can "pollute" the viewport tree in some cases.
  */
 
-double evaluateGrobWidthUnit(SEXP grob, 
-			     double vpheightCM, double vpwidthCM,
-			     int nullLMode, int nullAMode,
-			     GEDevDesc *dd) 
+double evaluateGrobUnit(double value, SEXP grob,
+			double vpwidthCM, double vpheightCM,
+			int nullLMode, int nullAMode,
+			/*
+			 * Evaluation type
+			 * 0 = x, 1 = y, 2 = width, 3 = height
+			 */
+			int evalType,
+			GEDevDesc *dd) 
 {
     double vpWidthCM, vpHeightCM;
     double rotationAngle;
     LViewportContext vpc;
     R_GE_gcontext gc;
-    LTransform transform;
+    LTransform transform, savedTransform;
     SEXP currentvp, currentgp;
-    SEXP widthPreFn, widthFn, widthPostFn, findGrobFn;
-    SEXP R_fcall0, R_fcall1, R_fcall2, R_fcall3;
+    SEXP preFn,  postFn, findGrobFn;
+    SEXP evalFnx = R_NilValue, evalFny = R_NilValue;
+    SEXP R_fcall0, R_fcall1, R_fcall2x, R_fcall2y, R_fcall3;
     SEXP savedgpar, savedgrob;
-    SEXP width;
-    double result;
+    SEXP unitx = R_NilValue, unity = R_NilValue;
+    double result = 0.0;
+    Rboolean protectedGrob = FALSE;
+    /*
+     * We are just doing calculations, not drawing, so
+     * we don't want anything recorded on the graphics engine DL
+     *
+     * FIXME:  This should probably be done via a GraphicsEngine.h
+     * function call rather than directly playing with dd->recordGraphics
+     */
+    Rboolean record = dd->recordGraphics;
+    dd->recordGraphics = FALSE;
+    /*
+     * Save the current viewport transform 
+     * (use to convert location relative to current viewport)
+     */
+    currentvp = gridStateElement(dd, GSS_VP);
+    getViewportTransform(currentvp, dd, 
+			 &vpWidthCM, &vpHeightCM, 
+			 savedTransform, &rotationAngle);
     /* 
      * Save the current gpar state and restore it at the end
      */
@@ -437,9 +477,21 @@ double evaluateGrobWidthUnit(SEXP grob,
     /*
      * Set up for calling R functions 
      */
-    PROTECT(widthPreFn = findFun(install("preDraw"), R_gridEvalEnv));
-    PROTECT(widthFn = findFun(install("width"), R_gridEvalEnv));
-    PROTECT(widthPostFn = findFun(install("postDraw"), R_gridEvalEnv));
+    PROTECT(preFn = findFun(install("preDraw"), R_gridEvalEnv));
+    switch(evalType) {
+    case 0:
+    case 1:
+	PROTECT(evalFnx = findFun(install("xDetails"), R_gridEvalEnv));
+	PROTECT(evalFny = findFun(install("yDetails"), R_gridEvalEnv));
+	break;
+    case 2:
+	PROTECT(evalFnx = findFun(install("width"), R_gridEvalEnv));
+	break;
+    case 3:
+	PROTECT(evalFny = findFun(install("height"), R_gridEvalEnv));
+	break;
+    }
+    PROTECT(postFn = findFun(install("postDraw"), R_gridEvalEnv));
     /*
      * If grob is actually a gPath, use it to find an actual grob
      */
@@ -456,20 +508,23 @@ double evaluateGrobWidthUnit(SEXP grob,
 					 R_gridEvalEnv));
 	    PROTECT(R_fcall0 = lang2(findGrobFn, 
 				     getListElement(grob, "name")));
-	    grob = eval(R_fcall0, R_gridEvalEnv);
+	    PROTECT(grob = eval(R_fcall0, R_gridEvalEnv));
 	} else {
 	    PROTECT(findGrobFn = findFun(install("findGrobinChildren"), 
 					 R_gridEvalEnv));
 	    PROTECT(R_fcall0 = lang3(findGrobFn, 
 				     getListElement(grob, "name"),
 				     getListElement(savedgrob, "children")));
-	    grob = eval(R_fcall0, R_gridEvalEnv);
+	    PROTECT(grob = eval(R_fcall0, R_gridEvalEnv));
 	}
-	UNPROTECT(2);
+	/*
+	 * Flag to make sure we UNPROTECT these at the end
+	 */
+	protectedGrob = TRUE;
     }
     /* Call preDraw(grob) 
      */
-    PROTECT(R_fcall1 = lang2(widthPreFn, grob));
+    PROTECT(R_fcall1 = lang2(preFn, grob));
     eval(R_fcall1, R_gridEvalEnv);
     /* 
      * The call to preDraw may have pushed viewports and/or
@@ -492,157 +547,164 @@ double evaluateGrobWidthUnit(SEXP grob,
 			 &vpWidthCM, &vpHeightCM, 
 			 transform, &rotationAngle);
     fillViewportContextFromViewport(currentvp, &vpc);
-    /* Call width(grob)
-     * to get the unit representing the with
+    /* Call whatever(grob)
+     * to get the unit representing the x/y/width/height
      */
-    PROTECT(R_fcall2 = lang2(widthFn, grob));
-    PROTECT(width = eval(R_fcall2, R_gridEvalEnv));
+    switch (evalType) {
+    case 0:
+    case 1:
+	/*
+	 * When evaluating grobX/grobY, the value of the unit
+	 * is an angle that gets passed to xDetails/yDetails
+	 */
+	{
+	    SEXP val;
+	    PROTECT(val = ScalarReal(value));
+	    PROTECT(R_fcall2x = lang3(evalFnx, grob, val));
+	    PROTECT(unitx = eval(R_fcall2x, R_gridEvalEnv));
+	    PROTECT(R_fcall2y = lang3(evalFny, grob, val));
+	    PROTECT(unity = eval(R_fcall2y, R_gridEvalEnv));
+	}
+	break;
+    case 2:
+	PROTECT(R_fcall2x = lang2(evalFnx, grob));
+	PROTECT(unitx = eval(R_fcall2x, R_gridEvalEnv));
+	break;
+    case 3:
+	PROTECT(R_fcall2y = lang2(evalFny, grob));
+	PROTECT(unity = eval(R_fcall2y, R_gridEvalEnv));
+	break;
+    }
     /* 
-     * Transform the width
+     * Transform the unit
      * NOTE:  We transform into INCHES so can produce final answer in terms
      * of NPC for original context
      */
     /* Special case for "null" units
      */
-    if (pureNullUnit(width, 0, dd)) {
-	result = evaluateNullUnit(pureNullUnitValue(width, 0), vpWidthCM,
-				  nullLMode, nullAMode);
-    } else {
-	gcontextFromgpar(currentgp, 0, &gc);
-	result = transformWidthtoINCHES(width, 0, vpc, &gc,
-					vpWidthCM, vpHeightCM,
-					dd);
+    gcontextFromgpar(currentgp, 0, &gc, dd);
+    switch(evalType) {
+    case 0:
+    case 1:
+	if (evalType && pureNullUnit(unity, 0, dd)) {
+	    result = evaluateNullUnit(pureNullUnitValue(unity, 0), 
+				      vpWidthCM,
+				      nullLMode, nullAMode);
+	} else if (pureNullUnit(unitx, 0, dd)) {
+	    result = evaluateNullUnit(pureNullUnitValue(unitx, 0), 
+				      vpWidthCM,
+				      nullLMode, nullAMode);
+	} else {
+	    /*
+	     * Transform to device (to allow for viewports in grob)
+	     * then adjust relative to current viewport.
+	     */
+	    double xx, yy;
+	    LLocation lin, lout;
+	    LTransform invt;
+	    invTransform(savedTransform, invt);
+	    transformLocn(unitx, unity, 0,
+			  vpc, &gc,
+			  vpWidthCM, vpHeightCM, dd,
+			  transform, &xx, &yy);
+	    location(xx, yy, lin);
+	    trans(lin, invt, lout);
+	    xx = locationX(lout);
+	    yy = locationY(lout);
+	    if (evalType)
+		result = yy;
+	    else
+		result = xx;
+	}
+	break;
+    case 2:
+	if (pureNullUnit(unitx, 0, dd)) {
+	    result = evaluateNullUnit(pureNullUnitValue(unitx, 0), 
+				      vpWidthCM,
+				      nullLMode, nullAMode);
+	} else {
+	    result = transformWidthtoINCHES(unitx, 0, vpc, &gc,
+					    vpWidthCM, vpHeightCM,
+					    dd);
+	}
+	break;
+    case 3:
+	if (pureNullUnit(unity, 0, dd)) {
+	    result = evaluateNullUnit(pureNullUnitValue(unity, 0), 
+				      vpWidthCM,
+				      nullLMode, nullAMode);
+	} else {
+	    result = transformHeighttoINCHES(unity, 0, vpc, &gc,
+					     vpWidthCM, vpHeightCM,
+					     dd);
+	}
+	break;
     }
     /* Call postDraw(grob)
      */
-    PROTECT(R_fcall3 = lang2(widthPostFn, grob));
+    PROTECT(R_fcall3 = lang2(postFn, grob));
     eval(R_fcall3, R_gridEvalEnv);
     /* 
      * Restore the saved gpar state and grob
      */
     setGridStateElement(dd, GSS_GPAR, savedgpar);
     setGridStateElement(dd, GSS_CURRGROB, savedgrob);
-    UNPROTECT(9);
+    if (protectedGrob) 
+	UNPROTECT(3);
+    switch(evalType) {
+    case 0:
+    case 1:
+	UNPROTECT(13);
+	break;
+    case 2:
+    case 3:
+	UNPROTECT(9);
+    }
     /* Return the transformed width
      */
+    /*
+     * If there is an error or user-interrupt in the above
+     * evaluation, dd->recordGraphics is set to TRUE
+     * on all graphics devices (see GEonExit(); called in errors.c)
+     */
+    dd->recordGraphics = record;
     return result;
 }
 
-/* See evaluateGrobWidthUnit for detailed comments
- */
+double evaluateGrobXUnit(double value, SEXP grob, 
+			 double vpheightCM, double vpwidthCM,
+			 int nullLMode, int nullAMode,
+			 GEDevDesc *dd) 
+{
+    return evaluateGrobUnit(value, grob, vpheightCM, vpwidthCM, 
+			    nullLMode, nullAMode, 0, dd);
+}
+
+double evaluateGrobYUnit(double value, SEXP grob, 
+			 double vpheightCM, double vpwidthCM,
+			 int nullLMode, int nullAMode,
+			 GEDevDesc *dd) 
+{
+    return evaluateGrobUnit(value, grob, vpheightCM, vpwidthCM, 
+			    nullLMode, nullAMode, 1, dd);
+}
+
+double evaluateGrobWidthUnit(SEXP grob, 
+			     double vpheightCM, double vpwidthCM,
+			     int nullLMode, int nullAMode,
+			     GEDevDesc *dd) 
+{
+    return evaluateGrobUnit(1, grob, vpheightCM, vpwidthCM, 
+			    nullLMode, nullAMode, 2, dd);
+}
+
 double evaluateGrobHeightUnit(SEXP grob, 
 			     double vpheightCM, double vpwidthCM,
 			     int nullLMode, int nullAMode,
 			     GEDevDesc *dd) 
 {
-    double vpWidthCM, vpHeightCM;
-    double rotationAngle;
-    LViewportContext vpc;
-    R_GE_gcontext gc;
-    LTransform transform;
-    SEXP currentvp, currentgp;
-    SEXP heightPreFn, heightFn, heightPostFn, findGrobFn;
-    SEXP R_fcall0, R_fcall1, R_fcall2, R_fcall3;
-    SEXP savedgpar, savedgrob;
-    SEXP height;
-    double result;
-    /* 
-     * Save the current gpar state and restore it at the end
-     */
-    PROTECT(savedgpar = gridStateElement(dd, GSS_GPAR));
-    /*
-     * Save the current grob and restore it at the end
-     */
-    PROTECT(savedgrob = gridStateElement(dd, GSS_CURRGROB));
-    PROTECT(heightPreFn = findFun(install("preDraw"), R_gridEvalEnv));
-    PROTECT(heightFn = findFun(install("height"), R_gridEvalEnv));
-    PROTECT(heightPostFn = findFun(install("postDraw"), R_gridEvalEnv));
-    /*
-     * If grob is actually a gPath, use it to find an actual grob
-     */
-    if (inherits(grob, "gPath")) {
-	/* 
-	 * If the current grob is NULL then we are at the top level
-	 * and we search the display list, otherwise we search the 
-	 * children of the current grob
-	 *
-	 * NOTE: assume here that only gPath of depth == 1 are valid
-	 */
-	if (isNull(savedgrob)) {
-	    PROTECT(findGrobFn = findFun(install("findGrobinDL"), 
-					 R_gridEvalEnv));
-	    PROTECT(R_fcall0 = lang2(findGrobFn, 
-				     getListElement(grob, "name")));
-	    grob = eval(R_fcall0, R_gridEvalEnv);
-	} else {
-	    PROTECT(findGrobFn = findFun(install("findGrobinChildren"), 
-					 R_gridEvalEnv));
-	    PROTECT(R_fcall0 = lang3(findGrobFn, 
-				     getListElement(grob, "name"),
-				     getListElement(savedgrob, "children")));
-	    grob = eval(R_fcall0, R_gridEvalEnv);
-	}
-	UNPROTECT(2);
-    }
-    /* Call preDraw(grob) 
-     */
-    PROTECT(R_fcall1 = lang2(heightPreFn, grob));
-    eval(R_fcall1, R_gridEvalEnv);
-    /* 
-     * The call to preDraw may have pushed viewports and/or
-     * enforced gpar settings, SO we need to re-establish the
-     * current viewport and gpar settings before evaluating the
-     * height unit.
-     * 
-     * NOTE:  we are really relying on the grid state to be coherent
-     * when we do stuff like this (i.e., not to have changed since
-     * we started evaluating the unit [other than the changes we may
-     * have deliberately made above by calling preDraw]).  In other
-     * words we are relying on no other drawing occurring at the 
-     * same time as we are doing this evaluation.  In other other
-     * words, we are relying on there being only ONE process
-     * (i.e., NOT multi-threaded).
-     */
-    currentvp = gridStateElement(dd, GSS_VP);
-    currentgp = gridStateElement(dd, GSS_GPAR);
-    getViewportTransform(currentvp, dd, 
-			 &vpWidthCM, &vpHeightCM, 
-			 transform, &rotationAngle);
-    fillViewportContextFromViewport(currentvp, &vpc);
-    /* Call height(grob)
-     * to get the unit representing the with
-     */
-    PROTECT(R_fcall2 = lang2(heightFn, grob));
-    PROTECT(height = eval(R_fcall2, R_gridEvalEnv));
-    /* 
-     * Transform the height
-     * NOTE:  We transform into INCHES so can produce final answer in terms
-     * of NPC for original context
-     */
-    /* Special case for "null" units
-     */
-    if (pureNullUnit(height, 0, dd)) {
-	result = evaluateNullUnit(pureNullUnitValue(height, 0), vpHeightCM,
-				  nullLMode, nullAMode);
-    } else {
-	gcontextFromgpar(currentgp, 0, &gc);
-	result = transformHeighttoINCHES(height, 0, vpc, &gc,
-					 vpWidthCM, vpHeightCM,
-					 dd);
-    }
-    /* Call postDraw(grob)
-     */
-    PROTECT(R_fcall3 = lang2(heightPostFn, grob));
-    eval(R_fcall3, R_gridEvalEnv);
-    /* 
-     * Restore the saved gpar state and grob
-     */
-    setGridStateElement(dd, GSS_GPAR, savedgpar);
-    setGridStateElement(dd, GSS_CURRGROB, savedgrob);
-    UNPROTECT(9);
-    /* Return the transformed height
-     */
-    return result;
+    return evaluateGrobUnit(1, grob, vpheightCM, vpwidthCM, 
+			    nullLMode, nullAMode, 3, dd);
 }
 
 /**************************
@@ -672,6 +734,7 @@ double transform(double value, int unit, SEXP data,
 	result = result/2.54;
 	break;
     case L_INCHES: 
+	result = result;
 	break;
     /* FIXME:  The following two assume that the pointsize specified
      * by the user is actually the pointsize provided by the
@@ -742,12 +805,20 @@ double transform(double value, int unit, SEXP data,
 					     gc, dd),
 				 GE_INCHES, dd);
 	break;
+    case L_GROBX:
+	result = evaluateGrobXUnit(value, data, thisCM, otherCM,
+				   nullLMode, nullAMode, dd);
+	break;	
+    case L_GROBY:
+	result = evaluateGrobYUnit(value, data, otherCM, thisCM,
+				   nullLMode, nullAMode, dd);
+	break;	
     case L_GROBWIDTH:
-	result = value*evaluateGrobWidthUnit(data, otherCM, thisCM, 
+	result = value*evaluateGrobWidthUnit(data, thisCM, otherCM,
 					     nullLMode, nullAMode, dd);
 	break;
     case L_GROBHEIGHT:
-	result = value*evaluateGrobHeightUnit(data, thisCM, otherCM, 
+	result = value*evaluateGrobHeightUnit(data, otherCM, thisCM,
 					      nullLMode, nullAMode, dd);
 	break;
     case L_NULL:
@@ -756,6 +827,30 @@ double transform(double value, int unit, SEXP data,
     default:
 	error(_("Illegal unit or unit not yet implemented"));
     }
+    /*
+     * For physical units, scale the result by GSS_SCALE (a "zoom" factor)
+     */
+    switch (unit) {
+    case L_INCHES:
+    case L_CM:
+    case L_MM:
+    case L_POINTS:
+    case L_PICAS:
+    case L_BIGPOINTS:
+    case L_DIDA:
+    case L_CICERO:
+    case L_SCALEDPOINTS:
+      result = result * REAL(gridStateElement(dd, GSS_SCALE))[0];
+      break;
+    default:
+      /*
+       * No need to scale relative coordinates (NPC, NATIVE, NULL)
+       * CHAR and LINES already scaled because of scaling in gcontextFromGPar()
+       * Ditto STRINGWIDTH/HEIGHT
+       * GROBWIDTH/HEIGHT recurse into here so scaling already done
+       */
+      break;
+    }
     return result;
 }
 
@@ -808,13 +903,14 @@ double transformX(SEXP x, int index,
 	    nullamode = nullAMode;
 	result = unitValue(x, index);
 	unit = unitUnit(x, index);
-	data = unitData(x, index);
+	PROTECT(data = unitData(x, index));
 	result = transformLocation(result, unit, data, 
 				   vpc.xscalemin, vpc.xscalemax, gc,
 				   widthCM, heightCM, 
 				   nullLMode, 
 				   nullamode,
 				   dd);
+	UNPROTECT(1);
     }
     return result;
 }
@@ -849,13 +945,14 @@ double transformY(SEXP y, int index,
 	    nullamode = nullAMode;
 	result = unitValue(y, index);
 	unit = unitUnit(y, index);
-	data = unitData(y, index);
+	PROTECT(data = unitData(y, index));
 	result = transformLocation(result, unit, data, 
 				   vpc.yscalemin, vpc.yscalemax, gc,
 				   heightCM, widthCM, 
 				   nullLMode, 
 				   nullamode,
 				   dd);
+	UNPROTECT(1);
     } 
     return result;
 }
@@ -909,13 +1006,14 @@ double transformWidth(SEXP width, int index,
 	    nullamode = nullAMode;
 	result = unitValue(width, index);
 	unit = unitUnit(width, index);
-	data = unitData(width, index);
+	PROTECT(data = unitData(width, index));
 	result = transformDimension(result, unit, data, 
 				    vpc.xscalemin, vpc.xscalemax, gc,
 				    widthCM, heightCM, 
 				    nullLMode, 
 				    nullamode,
 				    dd);
+	UNPROTECT(1);
     }
     return result;
 }
@@ -950,13 +1048,14 @@ double transformHeight(SEXP height, int index,
 	    nullamode = nullAMode;
 	result = unitValue(height, index);
 	unit = unitUnit(height, index);
-	data = unitData(height, index);
+	PROTECT(data = unitData(height, index));
 	result = transformDimension(result, unit, data, 
 				    vpc.yscalemin, vpc.yscalemax, gc,
 				    heightCM, widthCM, 
 				    nullLMode, 
 				    nullamode,
 				    dd);
+	UNPROTECT(1);
     }
     return result;
 }
@@ -1466,12 +1565,36 @@ double transformFromINCHES(double value, int unit,
     case L_MYSTRINGWIDTH:
     case L_STRINGHEIGHT:
     case L_MYSTRINGHEIGHT:
+    case L_GROBX:
+    case L_GROBY:
     case L_GROBWIDTH:
     case L_GROBHEIGHT:
     case L_NULL:
     default:
 	error(_("Illegal unit or unit not yet implemented"));
     }
+    /*
+     * For physical units, reverse the scale by GSS_SCALE (a "zoom" factor)
+     */
+    switch (unit) {
+    case L_INCHES:
+    case L_CM:
+    case L_MM:
+    case L_POINTS:
+    case L_PICAS:
+    case L_BIGPOINTS:
+    case L_DIDA:
+    case L_CICERO:
+    case L_SCALEDPOINTS:
+      result = result / REAL(gridStateElement(dd, GSS_SCALE))[0];
+      break;
+    default:
+      /*
+       * No need to scale relative coordinates (NPC, NATIVE, NULL)
+       * All other units forbidden anyway
+       */
+      break;
+    }
     return result;
 }
 
@@ -1552,12 +1675,15 @@ static UnitTab UnitTable[] = {
     { "strheight",     15 },
 
     { "char",          18 },
-    { "grobwidth",     19 },
-    { "grobheight",    20 },
-    { "mylines",       21 },
-    { "mychar",        22 },
-    { "mystrwidth",        23 },
-    { "mystrheight",        24 },
+    { "grobx",         19 },
+    { "groby",         20 },
+    { "grobwidth",     21 },
+    { "grobheight",    22 },
+
+    { "mylines",       23 },
+    { "mychar",        24 },
+    { "mystrwidth",    25 },
+    { "mystrheight",   26 },
 
     { NULL,            -1 }
 };
