@@ -1,3 +1,19 @@
+#  File src/library/stats/R/cor.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 #### cor() , cov() and var() : Based on the same C code
 
 ## cor() and cov() only differ by one single letter :
@@ -40,8 +56,8 @@ function(x, y=NULL, use="all.obs", method = c("pearson", "kendall", "spearman"))
          if (is.null(y)) {
              ncy <- ncx <- ncol(x)
              r <- matrix(0, nrow=ncx, ncol=ncy)
-             for (i in 2:ncx) {
-                 for (j in 1:(i-1)) {
+             for (i in seq.int(2, length.out = ncx - 1)) {
+                 for (j in seq_len(i - 1)) {
                      x2 <- x[,i]
                      y2 <- x[,j]
                      ok <- complete.cases(x2, y2)
