@@ -1,3 +1,19 @@
+#  File src/library/stats/R/mantelhaen.test.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 mantelhaen.test <-
 function(x, y = NULL, z = NULL,
          alternative = c("two.sided", "less", "greater"),
@@ -61,7 +77,7 @@ function(x, y = NULL, z = NULL,
                           sum(apply(rbind(s.x, s.y), 2, prod)
                               / (n^2 * (n - 1))))
             PARAMETER <- 1
-            PVAL <- pchisq(STATISTIC, PARAMETER, lower = FALSE)
+            PVAL <- pchisq(STATISTIC, PARAMETER, lower.tail = FALSE)
             names(STATISTIC) <- "Mantel-Haenszel X-squared"
             names(PARAMETER) <- "df"
             METHOD <- paste("Mantel-Haenszel chi-squared test",
@@ -88,7 +104,7 @@ function(x, y = NULL, z = NULL,
                 switch(alternative,
                        less = c(0, ESTIMATE * exp(qnorm(conf.level) * sd)),
                        greater = c(ESTIMATE * exp(qnorm(conf.level,
-                                   lower = FALSE) * sd), Inf),
+                                   lower.tail = FALSE) * sd), Inf),
                        two.sided = {
                            ESTIMATE * exp(c(1, -1) *
                                           qnorm((1 - conf.level) / 2) * sd)
@@ -120,13 +136,12 @@ function(x, y = NULL, z = NULL,
             ## Density of the *central* product hypergeometric
             ## distribution on its support: store for once as this is
             ## needed quite a bit.
-            dc <- .C("d2x2xk",
+            dc <- .C(R_d2x2xk,
                      as.integer(K),
                      as.double(m),
                      as.double(n),
                      as.double(t),
-                     d = double(hi - lo + 1),
-                     PACKAGE = "stats")$d
+                     d = double(hi - lo + 1))$d
             logdc <- log(dc)
 
             dn2x2xk <- function(ncp) {
@@ -168,7 +183,7 @@ function(x, y = NULL, z = NULL,
             PVAL <-
                 switch(alternative,
                        less = pn2x2xk(s, 1),
-                       greater = pn2x2xk(s, 1, upper = TRUE),
+                       greater = pn2x2xk(s, 1, upper.tail = TRUE),
                        two.sided = {
                            ## Note that we need a little fuzz.
                            relErr <- 1 + 10 ^ (-7)
@@ -212,14 +227,14 @@ function(x, y = NULL, z = NULL,
             ncp.L <- function(x, alpha) {
                 if(x == lo)
                     return(0)
-                p <- pn2x2xk(x, 1, upper = TRUE)
+                p <- pn2x2xk(x, 1, upper.tail = TRUE)
                 if(p > alpha)
                     uniroot(function(t)
-                            pn2x2xk(x, t, upper = TRUE) - alpha,
+                            pn2x2xk(x, t, upper.tail = TRUE) - alpha,
                             c(0, 1))$root
             else if (p < alpha)
                 1 / uniroot(function(t)
-                            pn2x2xk(x, 1/t, upper = TRUE) - alpha,
+                            pn2x2xk(x, 1/t, upper.tail = TRUE) - alpha,
                             c(.Machine$double.eps, 1))$root
             else
                 1
@@ -256,7 +271,7 @@ function(x, y = NULL, z = NULL,
         ## (Thanks to Torsten Hothorn for spotting this.)
         df <- (I - 1) * (J - 1)
         n <- m <- double(length = df)
-        V <- matrix(0, nr = df, nc = df)
+        V <- matrix(0, nrow = df, ncol = df)
         for (k in 1 : K) {
             f <- x[ , , k]              # frequencies in stratum k
             ntot <- sum(f)              # n_{..k}
@@ -275,7 +290,7 @@ function(x, y = NULL, z = NULL,
         n <- n - m
         STATISTIC <- c(crossprod(n, qr.solve(V, n)))
         PARAMETER <- df
-        PVAL <- pchisq(STATISTIC, PARAMETER, lower = FALSE)
+        PVAL <- pchisq(STATISTIC, PARAMETER, lower.tail = FALSE)
         names(STATISTIC) <- "Cochran-Mantel-Haenszel M^2"
         names(PARAMETER) <- "df"
         METHOD <- "Cochran-Mantel-Haenszel test"
