@@ -1,3 +1,19 @@
+#  File src/library/tools/R/Rd.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ### * Rd_pp
 
 Rd_pp <-
@@ -10,7 +26,7 @@ function(lines)
 
     ## Re-encode if necessary (and possible).
     encoding <-
-        .get_Rd_metadata_from_Rd_lines(lines[!is.na(nchar(lines, "c"))],
+        .get_Rd_metadata_from_Rd_lines(lines[!is.na(nchar(lines, "c", TRUE))],
                                        "encoding")
     if(length(encoding)) {
         if((Sys.getlocale("LC_CTYPE") != "C")
@@ -22,11 +38,10 @@ function(lines)
     }
     else {
         ## No \encoding metadata.
-        ## Determine whether we can assume Latin1.
-        if(!all(.is_ISO_8859(lines)))
-            encoding <- NA
+        ## Determine if ASCII
+        if(!all(.is_ASCII(lines))) encoding <- NA
     }
-    if(any(is.na(nchar(lines, "c")))) {
+    if(any(is.na(nchar(lines, "c", TRUE)))) {
         ## Ouch, invalid in the current locale.
         ## (Can only happen in a MBCS locale.)
         ## Try re-encoding from Latin1.
@@ -72,7 +87,7 @@ function(lines)
     skipList <- integer(0)
     skipLevel <- 0
     skipIndices <- ppLineIndices
-    for(i in seq(along = ppTypes)) {
+    for(i in seq_along(ppTypes)) {
         if(!is.na(skip <- ppTypes[i])) {
             if(skipLevel == 0 && skip > 0) {
                 skipStart <- ppLineIndices[i]
@@ -85,8 +100,8 @@ function(lines)
         else {
             if(skipLevel == 1 && skipList[1] > 0) {
                 skipIndices <- c(skipIndices,
-                                 seq(from = skipStart,
-                                     to = ppLineIndices[i]))
+                                 seq.int(from = skipStart,
+                                         to = ppLineIndices[i]))
                 skipLevel <- 0
             }
             else
@@ -192,7 +207,7 @@ function(RdFiles)
                  "Keywords", "Encoding")
     contents <- vector("list", length(RdFiles) * length(entries))
     dim(contents) <- c(length(RdFiles), length(entries))
-    for(i in seq(along = RdFiles)) {
+    for(i in seq_along(RdFiles)) {
         contents[i, ] <- Rdinfo(RdFiles[i])
     }
     colnames(contents) <- entries
@@ -388,10 +403,10 @@ function(package, dir, lib.loc = NULL)
         db <- list()
         for(f in docsFiles) {
             valid_lines <- lines <- .read_Rd_lines_quietly(f)
-            valid_lines[is.na(nchar(lines, "c"))] <- ""
-            eofPos <- grep("\\eof$", valid_lines)
+            valid_lines[is.na(nchar(lines, "c", TRUE))] <- ""
+            eofPos <- grep("^\\\\eof$", valid_lines)
             db <- c(db, split(lines[-eofPos],
-                              rep(seq(along = eofPos),
+                              rep(seq_along(eofPos),
                                   times = diff(c(0, eofPos)))[-eofPos]))
         }
         ## If this was installed using a recent enough version of R CMD
@@ -546,9 +561,10 @@ function(package, dir, lib.loc = NULL)
     if(!missing(package)) {
         dir <- .find.package(package, lib.loc)
         rds <- file.path(dir, "Meta", "Rd.rds")
-        if(file_test("-f", rds))
-            sort(unlist(.readRDS(rds)$Aliases))
-        else
+        if(file_test("-f", rds)) {
+            aliases <- .readRDS(rds)$Aliases
+            if(length(aliases)) sort(unlist(aliases)) else character()
+        } else
             character()
         ## <NOTE>
         ## Alternatively, we could get the aliases from the help index
@@ -572,7 +588,9 @@ function(package, dir, lib.loc = NULL)
             db <- Rd_db(dir = dir)
             db <- lapply(db, Rd_pp)
             aliases <- lapply(db, .get_Rd_metadata_from_Rd_lines, "alias")
-            sort(unique(unlist(aliases, use.names = FALSE)))
+            if(length(aliases))
+                sort(unique(unlist(aliases, use.names = FALSE)))
+            else character()
         }
         else
             character()
@@ -771,7 +789,7 @@ function(txt)
 .get_Rd_xrefs <-
 function(txt)
 {
-    out <- matrix(character(), nr = 0, nc = 2)
+    out <- matrix(character(), nrow = 0, ncol = 2)
     if(length(txt) != 1) return(out)
     while((pos <-
            regexpr("\\\\link(\\[[^[]+\\])?\\{", txt)) != -1) {
@@ -806,7 +824,7 @@ function(txt, cmd, FUN)
     if(length(txt) != 1) return(character())
 
     ## Vectorized in 'cmd':
-    pattern <- sprintf("\\\\%s\\{", paste(cmd, collapse = "|"))
+    pattern <- sprintf("\\\\(%s)\\{", paste(cmd, collapse = "|"))
 
     out <- character()
     while((pos <- regexpr(pattern, txt)) != -1) {
