@@ -1,3 +1,19 @@
+#  File src/library/utils/R/help.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 help <-
 function(topic, offline = FALSE, package = NULL, lib.loc = NULL,
          verbose = getOption("verbose"),
@@ -20,9 +36,14 @@ function(topic, offline = FALSE, package = NULL, lib.loc = NULL,
         return(help("help", package = "utils", lib.loc = .Library))
     }
 
-    ischar <- try(is.character(topic), silent = TRUE)
+    ischar <- try(is.character(topic) && length(topic) == 1, silent = TRUE)
     if(inherits(ischar, "try-error")) ischar <- FALSE
-    if(!ischar) topic <- deparse(substitute(topic))
+    ## if this was not a length-one character vector, try for the name.
+    if(!ischar) {
+        ## we only want names here, but some reserved words parse
+        ## to other types, e.g. NA, Inf
+        topic <- deparse(substitute(topic))
+    }
 
     type <- if(offline)
         "latex"
@@ -74,11 +95,12 @@ function(topic, offline = FALSE, package = NULL, lib.loc = NULL,
 print.help_files_with_topic <-
 function(x, ...)
 {
-    if (.Platform$GUI=="AQUA") {
+    if (.Platform$GUI == "AQUA") {
         .Internal(aqua.custom.print("help-files", x))
 	return(invisible(x))
     }
     topic <- attr(x, "topic")
+    type <- attr(x, "type")
     paths <- as.character(x)
     if(!length(paths)) {
         writeLines(c(gettextf("No documentation for '%s' in specified packages and libraries:",
@@ -98,25 +120,40 @@ function(x, ...)
                                     c(gettext("Library"),
                                       dirname(paths)),
                                     indent = 22))))
-    }
-    else {
+    } else {
         if(length(paths) > 1) {
             file <- paths[1]
+            p <- paths
             msg <- gettextf("Help on topic '%s' was found in the following packages:",
                             topic)
             paths <- dirname(dirname(paths))
-            writeLines(c(strwrap(msg), "",
-                         paste(" ",
-                               formatDL(c(gettext("Package"),
-                                          basename(paths)),
-                                        c(gettext("Library"),
-                                          dirname(paths)),
-                                        indent = 22)),
-                         gettext("\nUsing the first match ...")))
+            txt <- formatDL(c("Package", basename(paths)),
+                            c("Library", dirname(paths)),
+                            indent = 22)
+            writeLines(c(strwrap(msg), "", paste(" ", txt), ""))
+            if(interactive()) {
+                fp <- file.path(paths, "Meta", "Rd.rds")
+                tp <- basename(p)
+                titles <- tp
+                if(type == "html") tp <- tools::file_path_sans_ext(tp)
+                for (i in seq_along(fp)) {
+                    tmp <- try(.readRDS(fp[i]))
+                    titles[i] <- if(inherits(tmp, "try-error"))
+                        "unknown title" else
+                    tmp[tools::file_path_sans_ext(tmp$File) == tp[i], "Title"]
+                }
+                txt <- paste(titles, " {", basename(paths), "}", sep="")
+                ## FIXME: use html page for HTML help.
+                res <- menu(txt, title = gettext("Choose one"),
+                            graphics = getOption("menu.graphics"))
+                if(res > 0) file <- p[res]
+            } else {
+                writeLines(gettext("\nUsing the first match ..."))
+            }
         }
         else
             file <- paths
-        type <- attr(x, "type")
+
         if(type == "html") {
             if(file.exists(file))
                 .show_help_on_topic_as_HTML(file, topic)
@@ -125,7 +162,7 @@ function(x, ...)
         }
         else if(type == "chm") {
             ## unneeded but harmless under Unix
-            chm.dll <- file.path(R.home("bin"), "Rchtml.dll")
+            chm.dll <- file.path(R.home("modules"), "Rchtml.dll")
             if(!file.exists(chm.dll))
                 stop("Compiled HTML is not installed")
             if(!is.loaded("Rchtml")) dyn.load(chm.dll)
@@ -137,17 +174,31 @@ function(x, ...)
                 err <- .C("Rchtml", hlpfile, basename(file),
                           err = integer(1), PACKAGE = "Rchtml")$err
                 if(err) stop("CHM file could not be displayed")
-            } else
-                stop(gettextf("No CHM help for '%s' in package '%s' is available:\nthe CHM file is for the package is missing", topic, thispkg), domain = NA)
+            } else {
+            	warning(gettextf("No CHM help for '%s' in package '%s' is available:\nthe CHM file for the package is missing", topic, thispkg), domain = NA)
+            	att <- attributes(x)
+            	x <- sub("/chm/([^/]*$)", "/help/\\1", x)
+            	attributes(x) <- att
+            	attr(x, "type") <- "help"
+            	print(x)
+            }
         }
         else if(type == "help") {
             zfile <- zip.file.extract(file, "Rhelp.zip")
-            if(file.exists(zfile))
+            if(file.exists(zfile)) {
+                first <- readLines(zfile, n = 1)
+                enc <- if(length(grep("\\(.*\\)$", first)) > 0)
+                    sub("[^(]*\\((.*)\\)$", "\\1", first) else ""
+                if(enc == "utf8") enc <- "UTF-8"
+                ## allow for 'smart' quotes on Windows, which work
+                ## in all but CJK encodings
+                if(.Platform$OS.type == "windows" && enc == ""
+                   && l10n_info()$codepage < 1000) enc <- "CP1252"
                 file.show(zfile,
                           title = gettextf("R Help on '%s'", topic),
                           delete.file = (zfile != file),
-                          pager = attr(x, "pager"))
-            else
+                          pager = attr(x, "pager"), encoding = enc)
+            } else
                 stop(gettextf("No text help for '%s' is available:\ncorresponding file is missing", topic), domain = NA)
         }
         else if(type == "latex") {
