@@ -1,3 +1,19 @@
+#  File src/library/utils/R/packageStatus.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 packageStatus <- function(lib.loc = NULL, repositories = NULL, method,
                           type = getOption("pkgType"))
 {
@@ -5,7 +21,7 @@ packageStatus <- function(lib.loc = NULL, repositories = NULL, method,
     {
         vers <- package_version(x)
 	max <- vers[1]
-        for (i in seq(along=vers)) if (max < vers[i]) max <- vers[i]
+        for (i in seq_along(vers)) if (max < vers[i]) max <- vers[i]
 	which(vers == max)[1]
     }
 
