@@ -1,29 +1,47 @@
-available.packages <-
-    function(contriburl = contrib.url(getOption("repos")), method)
-{
-    .checkRversion <- function(x) {
-        if(is.na(xx <- x["Depends"])) return(TRUE)
-        xx <- tools:::.split_dependencies(xx)
-        if(length(z <- xx[["R"]]) > 1)
-            eval(parse(text=paste("currentR", z$op, "z$version")))
-        else TRUE
-    }
+#  File src/library/utils/R/packages.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
 
-    flds <- c("Package", "Version", "Priority", "Bundle",
-              "Depends", "Imports", "Suggests", "Contains")
-    res <- matrix(as.character(NA), 0, length(flds) + 1)
-    colnames(res) <- c(flds, "Repository")
+available.packages <-
+    function(contriburl = contrib.url(getOption("repos")), method,
+             fields = NULL)
+{
+    requiredFields <-
+        tools:::.get_standard_repository_db_fields()
+    if (is.null(fields))
+	fields <- requiredFields
+    else {
+	stopifnot(is.character(fields))
+	fields <- unique(c(requiredFields, fields))
+    }
+    res <- matrix(NA_character_, 0, length(fields) + 1,
+		  dimnames = list(NULL, c(fields, "Repository")))
     for(repos in contriburl) {
         localcran <- length(grep("^file:", repos)) > 0
         if(localcran) {
             ## see note in download.packages
-            tmpf <- paste(substring(repos, 6), "PACKAGES", sep = "/")
-            tmpf <- sub("^//", "", tmpf)
-            if(.Platform$OS.type == "windows") {
-                if(length(grep("[A-Za-z]:", tmpf)))
-                    tmpf <- substring(tmpf, 2)
+            if(substring(repos, 1, 8) == "file:///") {
+                tmpf <- paste(substring(repos, 8), "PACKAGES", sep = "/")
+                if(.Platform$OS.type == "windows") {
+                    if(length(grep("^/[A-Za-z]:", tmpf)))
+                        tmpf <- substring(tmpf, 2)
+                }
+            } else {
+                tmpf <- paste(substring(repos, 6), "PACKAGES", sep = "/")
             }
-            res0 <- read.dcf(file = tmpf, fields = flds)
+            res0 <- read.dcf(file = tmpf)
             if(length(res0)) rownames(res0) <- res0[, "Package"]
         } else {
             dest <- file.path(tempdir(),
@@ -57,26 +75,36 @@ available.packages <-
                             call. = FALSE, immediate. = TRUE, domain = NA)
                     next
                 }
-                res0 <- read.dcf(file = tmpf, fields = flds)
+                res0 <- read.dcf(file = tmpf)
                 if(length(res0)) rownames(res0) <- res0[, "Package"]
                 .saveRDS(res0, dest, compress = TRUE)
                 unlink(tmpf)
                 on.exit()
             } # end of download vs cached
         } # end of localcran vs online
-        res0 <- cbind(res0, Repository = repos)
-        res <- rbind(res, res0)
+        if (length(res0)) {
+            missingFields <- fields[!(fields %in% colnames(res0))]
+            if (length(missingFields)) {
+                toadd <- matrix(NA_character_, nrow=nrow(res0),
+                                ncol=length(missingFields),
+                                dimnames=list(NULL, missingFields))
+                res0 <- cbind(res0, toadd)
+            }
+            res0 <- cbind(res0[, fields, drop = FALSE],
+                          Repository = repos)
+            res <- rbind(res, res0)
+        }
     }
     ## ignore packages which don't fit our version of R
     if(length(res)) {
+        currentR <- getRversion()
         .checkRversion <- function(x) {
             if(is.na(xx <- x["Depends"])) return(TRUE)
             xx <- tools:::.split_dependencies(xx)
-            if(length(z <- xx[["R"]]) > 1)
+            if(length(z <- xx[["R", exact=TRUE]]) > 1)
                 eval(parse(text=paste("currentR", z$op, "z$version")))
             else TRUE
         }
-        currentR <- getRversion()
         res <- res[apply(res, 1, .checkRversion), , drop=FALSE]
     }
     res
@@ -95,8 +123,7 @@ simplifyRepos <- function(repos, type)
 update.packages <- function(lib.loc = NULL, repos = getOption("repos"),
                             contriburl = contrib.url(repos, type),
                             method, instlib = NULL, ask = TRUE,
-                            available = NULL, destdir = NULL,
-			    installWithVers = FALSE,
+                            available = NULL, ...,
                             checkBuilt = FALSE, type = getOption("pkgType"))
 {
     ask  # just a check that it is valid before we start work
@@ -113,7 +140,7 @@ update.packages <- function(lib.loc = NULL, repos = getOption("repos"),
                 simplifyRepos(old[k, "Repository"], type))
             cat("\n")
             answer <- substr(readline("Update (y/N/c)?  "), 1, 1)
-            if(answer == "c" | answer == "c") {
+            if(answer == "c" | answer == "C") {
                 cat("cancelled by user\n")
                 return(invisible())
             }
@@ -154,12 +181,12 @@ update.packages <- function(lib.loc = NULL, repos = getOption("repos"),
 
     if(!is.null(update)) {
         if(is.null(instlib)) instlib <-  update[,"LibPath"]
-
-        install.packages(update[,"Package"], instlib,
-                         contriburl = contriburl,
-                         method = method,
-                         available = available, destdir = destdir,
-                         installWithVers = installWithVers, type = type)
+        ## do this a library at a time, to handle dependencies correctly.
+        libs <- unique(instlib)
+        for(l in libs)
+            install.packages(update[instlib == l ,"Package"], l,
+                             contriburl = contriburl, method = method,
+                             available = available, ..., type = type)
     }
 }
 
@@ -171,12 +198,13 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
         lib.loc <- .libPaths()
 
     instp <- installed.packages(lib.loc = lib.loc)
-    if(is.null(dim(instp)))
-        stop(gettextf("no installed packages for (invalid?) 'lib.loc=%s'",
-                      lib.loc), domain = NA)
+    if(NROW(instp) == 0) return(NULL)
+    ##    stop(gettextf("no installed packages for (invalid?) 'lib.loc=%s'",
+    ##                  lib.loc), domain = NA)
     if(is.null(available))
         available <- available.packages(contriburl = contriburl,
                                         method = method)
+    available <- .remove_stale_dups(available)
 
     ## For bundles it is sufficient to install the first package
     ## contained in the bundle, as this will install the complete bundle
@@ -198,7 +226,7 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
     update <- NULL
 
     currentR <- minorR <- getRversion()
-    minorR[[1]][3] <- 0 # set patchlevel to 0
+    minorR[[c(1,3)]] <- 0 # set patchlevel to 0
     for(k in 1:nrow(instp)) {
         if (instp[k, "Priority"] %in% "base") next
         z <- match(instp[k, "Package"], available[,"Package"])
@@ -210,7 +238,7 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
            package_version(instp[k, "Version"])) next
         deps <- onRepos["Depends"]
         if(!is.na(deps)) {
-            Rdeps <- tools:::.split_dependencies(deps)[["R"]]
+            Rdeps <- tools:::.split_dependencies(deps)[["R", exact=TRUE]]
             if(length(Rdeps) > 1) {
                 target <- Rdeps$version
                 res <- eval(parse(text=paste("currentR", Rdeps$op, "target")))
@@ -229,8 +257,9 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
 }
 
 new.packages <- function(lib.loc = NULL, repos = getOption("repos"),
-                         contriburl = contrib.url(repos),
-                         method, available = NULL, ask = FALSE)
+                         contriburl = contrib.url(repos, type),
+                         method, available = NULL, ask = FALSE,
+                         ..., type = getOption("pkgType"))
 {
     ask  # just a check that it is valid before we start work
     if(is.null(lib.loc)) lib.loc <- .libPaths()
@@ -270,6 +299,11 @@ new.packages <- function(lib.loc = NULL, repos = getOption("repos"),
     installed <- unique(instp[, "Package"])
     poss <- sort(unique(available[ ,"Package"])) # sort in local locale
     res <- setdiff(poss, installed)
+
+    ## We need to treat gnomeGUI separately
+    if(.Platform$OS.type == "unix" &&
+       file.exists(paste(R.home(), "/bin/exec", Sys.getenv("R_ARCH"), "/Rgnome", sep=""))) res <- setdiff(res, "gnomeGUI")
+
     update <- character(0)
     if(is.character(ask) && ask == "graphics") {
         if(.Platform$OS.type == "unix"
@@ -288,61 +322,78 @@ new.packages <- function(lib.loc = NULL, repos = getOption("repos"),
                             , res)]
     if(length(update)) {
         install.packages(update, lib = lib.loc[1], contriburl = contriburl,
-                         method = method, available = available)
-        # now check if they were installed and update 'res'
-        updated <- update[update %in% list.files(lib.loc[1])]
+                         method = method, available = available, ...)
+        # Now check if they were installed and update 'res'
+        dirs <- list.files(lib.loc[1])
+        updated <- update[update %in% dirs]
+        # Need to check separately for bundles
+        av <- available[update, , drop = FALSE]
+        bundles <- av[!is.na(av[, "Contains"]), , drop=FALSE]
+        for(bundle in rownames(bundles)) {
+            contains <- strsplit(bundles[bundle, "Contains"],
+                                 "[[:space:]]+")[[1]]
+            if(all(contains %in% dirs)) updated <- c(updated, bundle)
+        }
         res <- res[!res %in% updated]
     }
     res
 }
 
 installed.packages <-
-    function(lib.loc = NULL, priority = NULL,  noCache = FALSE)
+    function(lib.loc = NULL, priority = NULL,  noCache = FALSE,
+             fields = NULL)
 {
     if(is.null(lib.loc))
         lib.loc <- .libPaths()
-    pkgFlds <- c("Version", "Priority", "Bundle", "Contains", "Depends",
-                 "Suggests", "Imports", "Built")
     if(!is.null(priority)) {
         if(!is.character(priority))
             stop("'priority' must be character or NULL")
         if(any(b <- priority %in% "high"))
             priority <- c(priority[!b], "recommended","base")
     }
-    retval <- matrix("", 0, 2+length(pkgFlds))
+    requiredFields <-
+        c(tools:::.get_standard_repository_db_fields(), "Built")
+    if (is.null(fields))
+	fields <- requiredFields
+    else {
+	stopifnot(is.character(fields))
+	fields <- unique(c(requiredFields, fields))
+    }
+    ## Don't retain 'Package' and 'LibPath' fields as these are used to
+    ## record name and path of installed packages.
+    fields <- fields[! fields %in% c("Package", "LibPath")]
+    retval <- matrix("", 0, 2 + length(fields))
     for(lib in lib.loc) {
         dest <- file.path(tempdir(),
-                          paste("libloc_", URLencode(lib, TRUE), ".rds",
+                          paste("libloc_", URLencode(lib, TRUE),
+                                paste(fields, collapse=","),
+                                ".rds",
                                 sep=""))
         if(!noCache && file.exists(dest) &&
             file.info(dest)$mtime > file.info(lib.loc)$mtime) {
             retval <- rbind(retval, .readRDS(dest))
         } else {
-            ret0 <- character()
-            ## this excludes packages without DESCRIPTION files
-            pkgs <- .packages(all.available = TRUE, lib.loc = lib)
-            for(p in pkgs){
-                desc <- packageDescription(p, lib = lib, fields = pkgFlds,
-                                           encoding = NA)
-                ## this gives NA if the package has no Version field
-                if (is.logical(desc)) {
-                    desc <- rep(as.character(NA), length(pkgFlds))
-                    names(desc) <- pkgFlds
-                } else {
-                    desc <- unlist(desc)
-                    Rver <- strsplit(strsplit(desc["Built"], ";")[[1]][1],
+            pkgs <- list.files(lib)
+            ret0 <- matrix(NA_character_, length(pkgs), 2+length(fields))
+            for(i in seq_along(pkgs)) {
+                pkgpath <- file.path(lib, pkgs[i])
+                if(file.access(pkgpath, 5)) next
+                pkgpath <- file.path(pkgpath, "DESCRIPTION")
+                if(file.access(pkgpath, 4)) next
+                desc <- read.dcf(pkgpath, fields = fields)[1,]
+                Rver <- strsplit(strsplit(desc["Built"], ";")[[1]][1],
                                      "[ \t]+")[[1]][2]
-                    desc["Built"] <- Rver
-                }
-                ret0 <- rbind(ret0, c(p, lib, desc))
+                desc["Built"] <- Rver
+                ret0[i, ] <- c(sub("_.*", "", pkgs[i]), lib, desc)
             }
+            ret0 <- ret0[!is.na(ret0[,1]), ]
             if(length(ret0)) {
                 retval <- rbind(retval, ret0)
                 .saveRDS(ret0, dest, compress = TRUE)
             }
         }
     }
-    colnames(retval) <- c("Package", "LibPath", pkgFlds)
+    colnames(retval) <- c("Package", "LibPath", fields)
     if(length(retval) && !is.null(priority)) {
         keep <- !is.na(pmatch(retval[,"Priority"], priority,
                               duplicates.ok = TRUE))
@@ -377,14 +428,14 @@ remove.packages <- function(pkgs, lib, version) {
     is_bundle <- pkgs %in% have[, "Bundle"]
     pkgs0 <- pkgs; pkgs <- pkgs[!is_bundle]
     if(hv) {
-        names(version) <- pkgs0;
+        names(version) <- pkgs0
         if(length(pkgs)) pkgs <- manglePackageName(pkgs, version[!is_bundle])
     }
     for(p in pkgs0[is_bundle]) {
         ## for consistency with packages, need unversioned names
         ## and let .find.packages() figure out what to do.
         add <- have[have[, "Bundle"] %in% p, "Package"]
-        add <- unique(sub("_[0-9.\-]*$", "", add))
+        add <- unique(sub("_[0-9.-]*$", "", add))
         if(hv) add <- manglePackageName(add, version[p])
         pkgs <- c(pkgs, add)
     }
@@ -425,6 +476,7 @@ download.packages <- function(pkgs, destdir, available = NULL,
                 keep[duplicated(keep)] <- FALSE
                 ok[ok][!keep] <- FALSE
             }
+            if (length(grep("mac\\.binary\\..",type))) type <- "mac.binary"
             fn <- paste(p, "_", available[ok, "Version"],
                         switch(type,
                                "source" = ".tar.gz",
@@ -433,18 +485,25 @@ download.packages <- function(pkgs, destdir, available = NULL,
                         sep="")
             repos <- available[ok, "Repository"]
             if(length(grep("^file:", repos)) > 0) { # local repository
-                ## We need to derive the file name from the URL
-                ## This is tricky as so many forms have been allowed,
-                ## and indeed external methods may do even more.
-                fn <- paste(substring(repos, 6), fn, sep = "/")
-                fn <- sub("^//", "", fn)
-                fn <- URLdecode(fn)
-                ## This should leave us with a path beginning with /
-                if(.Platform$OS.type == "windows") {
-                    if(length(grep("[A-Za-z]:", fn)))
-                        fn <- substring(fn, 2)
+                ## This could be file: + file path or a file:/// URL.
+                if(substring(repos, 1, 8) == "file:///") {
+                    ## We need to derive the file name from the URL
+                    ## This is tricky as so many forms have been allowed,
+                    ## and indeed external methods may do even more.
+                    fn <- paste(substring(repos, 8), fn, sep = "/")
+                    ## This leaves a path beginning with /
+                    if(.Platform$OS.type == "windows") {
+                        if(length(grep("^/[A-Za-z]:", fn)))
+                            fn <- substring(fn, 2)
+                    }
+                } else {
+                    fn <- paste(substring(repos, 6), fn, sep = "/")
                 }
-                retval <- rbind(retval, c(p, fn))
+                if(file.exists(fn))
+                    retval <- rbind(retval, c(p, fn))
+                else
+                    warning(gettextf("package '%s' does not exist on the local repository", p),
+                            domain = NA, immediate. = TRUE)
             } else {
                 url <- paste(repos, fn, sep="/")
                 destfile <- file.path(destdir, fn)
@@ -478,22 +537,43 @@ contrib.url <- function(repos, type = getOption("pkgType"))
     }
     if("@CRAN@" %in% repos) stop("trying to use CRAN without setting a mirror")
 
-    ver <- paste(R.version$major, substring(R.version$minor, 1, 1), sep = ".")
-    res <-
-        switch(type,
-               "source" = paste(gsub("/$", "", repos), "src", "contrib", sep="/"),
-               "mac.binary" = paste(gsub("/$", "", repos), "bin", "macosx", R.version$arch, "contrib", ver, sep = "/"),
-               "win.binary" = paste(gsub("/$", "", repos), "bin", "windows", "contrib", ver, sep="/")
+    ver <- paste(R.version$major,
+                 strsplit(R.version$minor, ".", fixed=TRUE)[[1]][1], sep = ".")
+    mac.subtype <- "universal"
+    if (length(grep("mac\\.binary\\..",type))) {
+        mac.subtype <- substring(type, 12)
+        type <- "mac.binary"
+    }
+    res <- switch(type,
+		"source" = paste(gsub("/$", "", repos), "src", "contrib", sep="/"),
+                "mac.binary" = paste(gsub("/$", "", repos), "bin", "macosx", mac.subtype, "contrib", ver, sep = "/"),
+                "win.binary" = paste(gsub("/$", "", repos), "bin", "windows", "contrib", ver, sep="/")
                )
-    names(res) <- names(repos)
     res
 }
 
 
-chooseCRANmirror <- function(graphics = TRUE)
+getCRANmirrors <- function(all=FALSE, local.only=FALSE)
+{
+    m <- NULL
+    if(!local.only) {
+        ## try to handle explicitly failure to connect to CRAN.
+        con <- url("http://cran.r-project.org/CRAN_mirrors.csv")
+        m <- try(open(con, "r"), silent = TRUE)
+        if(!inherits(m, "try-error")) m <- try(read.csv(con, as.is=TRUE))
+        close(con)
+    }
+    if(is.null(m) || inherits(m, "try-error"))
+        m <- read.csv(file.path(R.home("doc"), "CRAN_mirrors.csv"), as.is=TRUE)
+    if(!all) m <- m[as.logical(m$OK), ]
+    m
+}
+
+
+chooseCRANmirror <- function(graphics = getOption("menu.graphics"))
 {
     if(!interactive()) stop("cannot choose a CRAN mirror non-interactively")
-    m <- read.csv(file.path(R.home("doc"), "CRAN_mirrors.csv"), as.is=TRUE)
+    m <- getCRANmirrors(all=FALSE, local.only=FALSE)
     res <- menu(m[,1], graphics, "CRAN mirror")
     if(res > 0) {
         URL <- m[res, "URL"]
@@ -504,13 +584,14 @@ chooseCRANmirror <- function(graphics = TRUE)
     invisible()
 }
 
-setRepositories <- function(graphics=TRUE)
+setRepositories <-
+    function(graphics = getOption("menu.graphics"), ind = NULL)
 {
     if(!interactive()) stop("cannot set repositories non-interactively")
     p <- file.path(Sys.getenv("HOME"), ".R", "repositories")
     if(!file.exists(p))
         p <- file.path(R.home("etc"), "repositories")
-    a <- read.delim(p, header=TRUE,
+    a <- read.delim(p, header=TRUE, comment.char="#",
                     colClasses=c(rep("character", 3), rep("logical", 4)))
     thisType <- a[[getOption("pkgType")]]
     a <- a[thisType, 1:3]
@@ -532,30 +613,33 @@ setRepositories <- function(graphics=TRUE)
 
     default <- a[["default"]]
 
-    res <- integer(0)
-    if(graphics) {
-        ## return a list of row numbers.
-        if(.Platform$OS.type == "windows" || .Platform$GUI == "AQUA")
-            res <- match(select.list(a[, 1], a[default, 1], multiple = TRUE,
-                                     "Repositories"), a[, 1])
-        else if(.Platform$OS.type == "unix" &&
-                capabilities("tcltk") && capabilities("X11"))
-            res <- match(tcltk::tk_select.list(a[, 1], a[default, 1],
-                                            multiple = TRUE, "Repositories"),
-                         a[, 1])
-    }
-    if(!length(res)) {
-        ## text-mode fallback
-        cat(gettext("--- Please select repositories for use in this session ---\n"))
-        nc <- length(default)
-        cat("", paste(seq(len=nc), ": ",
-                      ifelse(default, "+", " "), " ", a[, 1],
-                      sep=""),
-            "", sep="\n")
-        cat(gettext("Enter one or more numbers separated by spaces\n"))
-        res <- scan("", what=0, quiet=TRUE, nlines=1)
-        if(!length(res) || (length(res) == 1 && !res[1])) return(invisible())
-        res <- res[1 <= res && res <= nc]
+    if(length(ind)) res <- as.integer(ind)
+    else {
+        res <- integer(0)
+        if(graphics) {
+            ## return a list of row numbers.
+            if(.Platform$OS.type == "windows" || .Platform$GUI == "AQUA")
+                res <- match(select.list(a[, 1], a[default, 1], multiple = TRUE,
+                                         "Repositories"), a[, 1])
+            else if(.Platform$OS.type == "unix" &&
+                    capabilities("tcltk") && capabilities("X11"))
+                res <- match(tcltk::tk_select.list(a[, 1], a[default, 1],
+                                                   multiple = TRUE, "Repositories"),
+                             a[, 1])
+        }
+        if(!length(res)) {
+            ## text-mode fallback
+            cat(gettext("--- Please select repositories for use in this session ---\n"))
+            nc <- length(default)
+            cat("", paste(seq_len(nc), ": ",
+                          ifelse(default, "+", " "), " ", a[, 1],
+                          sep=""),
+                "", sep="\n")
+            cat(gettext("Enter one or more numbers separated by spaces\n"))
+            res <- scan("", what=0, quiet=TRUE, nlines=1)
+            if(!length(res) || (length(res) == 1 && !res[1])) return(invisible())
+            res <- res[1 <= res && res <= nc]
+        }
     }
     if(length(res)) {
         repos <- a[["URL"]]
@@ -653,3 +737,25 @@ compareVersion <- function(a, b)
     }
     done
 }
+
+.remove_stale_dups <- function(ap)
+{
+    ## Given a matrix from available.packages, return a copy
+    ## with no duplicate packages, being sure to keep the packages
+    ## with highest version number.
+    pkgs <- ap[ , "Package"]
+    dup_pkgs <- pkgs[duplicated(pkgs)]
+    stale_dups <- integer(length(dup_pkgs))
+    i <- 1L
+    for (dp in dup_pkgs) {
+        wh <- which(dp == pkgs)
+        vers <- package_version(ap[wh, "Version"])
+        keep_ver <- max(vers)
+        keep_idx = which(vers == keep_ver)[1L] # they might all be max
+        wh <- wh[-keep_idx]
+        end_i <- i + length(wh) - 1L
+        stale_dups[i:end_i] <- wh
+        i <- end_i + 1L
+    }
+    if(length(stale_dups)) ap[-stale_dups, ] else ap
+}
