@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> char here is handled as a whole string */
@@ -145,13 +145,14 @@ static int CountDLL = 0;
 
 static DllInfo LoadedDLL[MAX_NUM_DLLS];
 
-int addDLL(char *dpath, char *name, HINSTANCE handle);
-SEXP Rf_MakeDLLInfo(DllInfo *info);
+static int addDLL(char *dpath, char *name, HINSTANCE handle);
+static SEXP Rf_MakeDLLInfo(DllInfo *info);
 
 static SEXP createRSymbolObject(SEXP sname, DL_FUNC f,
 				R_RegisteredNativeSymbol *symbol,
 				Rboolean withRegistrationInfo);
 
+static DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path);
 
 attribute_hidden OSDynSymbol Rf_osDynSymbol;
 attribute_hidden OSDynSymbol *R_osDynSymbol = &Rf_osDynSymbol;
@@ -160,7 +161,7 @@ void R_init_base(DllInfo *); /* In Registration.c */
 DL_FUNC R_dlsym(DllInfo *dll, char const *name,
 		R_RegisteredNativeSymbol *symbol);
 
-void
+void attribute_hidden
 InitDynload()
 {
    DllInfo *dll;
@@ -170,11 +171,26 @@ InitDynload()
    InitFunctionHashing();
 }
 
+/* returns DllInfo used by the embedding application.
+   the underlying "(embedding)" entry is created if not present */
+DllInfo *R_getEmbeddingDllInfo() {
+    DllInfo *dll = R_getDllInfo("(embedding)");
+    if (dll == NULL) {
+	int which = addDLL(strdup("(embedding)"), "(embedding)", NULL);
+	dll = &LoadedDLL[which];
+	/* make sure we don't attempt dynamic lookup */
+	R_useDynamicSymbols(dll, FALSE);
+    }
+    return dll;
+}
+
+#ifdef UNUSED
 DllInfo *
 getBaseDllInfo()
 {
     return(&LoadedDLL[0]);
 }
+#endif
 
 Rboolean R_useDynamicSymbols(DllInfo *info, Rboolean value)
 {
@@ -185,17 +201,21 @@ Rboolean R_useDynamicSymbols(DllInfo *info, Rboolean value)
   return(old);
 }
 
-void R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
-		   Rf_DotCSymbol *sym);
-void R_addCallRoutine(DllInfo *info,
-		      const R_CallMethodDef * const croutine,
-		      Rf_DotCallSymbol *sym);
-void R_addFortranRoutine(DllInfo *info,
-			 const R_FortranMethodDef * const croutine,
-			 Rf_DotFortranSymbol *sym);
-void R_addExternalRoutine(DllInfo *info,
-			  const R_ExternalMethodDef * const croutine,
-	      	          Rf_DotExternalSymbol *sym);
+static void 
+R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
+	      Rf_DotCSymbol *sym);
+static void 
+R_addCallRoutine(DllInfo *info,
+		 const R_CallMethodDef * const croutine,
+		 Rf_DotCallSymbol *sym);
+static void 
+R_addFortranRoutine(DllInfo *info,
+		    const R_FortranMethodDef * const croutine,
+		    Rf_DotFortranSymbol *sym);
+static void 
+R_addExternalRoutine(DllInfo *info,
+		     const R_ExternalMethodDef * const croutine,
+		     Rf_DotExternalSymbol *sym);
 
 
 /*
@@ -236,11 +256,12 @@ R_registerRoutines(DllInfo *info, const R_CMethodDef * const croutines,
     if(info == NULL)
 	error(_("R_RegisterRoutines called with invalid DllInfo object."));
 
-
-    info->useDynamicLookup = TRUE; /* Default is to look in registered and then dynamic.
-                                      Potentially change in the future to be only registered
-                                      if there are any registered values.
-                                    */
+    /* Default is to look in registered and then dynamic (unless
+       the is no handle such as in "base" or "embedded")
+       Potentially change in the future to be only registered
+       if there are any registered values.
+    */
+    info->useDynamicLookup = (info->handle)?TRUE:FALSE;
 
     if(croutines) {
 	for(num=0; croutines[num].name != NULL; num++) {;}
@@ -311,7 +332,7 @@ R_setArgStyles(const R_FortranMethodDef * const croutine,
 	       sizeof(R_NativeArgStyle) * croutine->numArgs);
 }
 
-void attribute_hidden
+static void
 R_addFortranRoutine(DllInfo *info,
 		    const R_FortranMethodDef * const croutine,
 		    Rf_DotFortranSymbol *sym)
@@ -325,7 +346,7 @@ R_addFortranRoutine(DllInfo *info,
        R_setArgStyles(croutine, sym);
 }
 
-void attribute_hidden
+static void
 R_addExternalRoutine(DllInfo *info,
 		     const R_ExternalMethodDef * const croutine,
 		     Rf_DotExternalSymbol *sym)
@@ -337,7 +358,7 @@ R_addExternalRoutine(DllInfo *info,
 
 
 
-void attribute_hidden
+static void
 R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
 	      Rf_DotCSymbol *sym)
 {
@@ -351,7 +372,7 @@ R_addCRoutine(DllInfo *info, const R_CMethodDef * const croutine,
 
 }
 
-void attribute_hidden
+static void
 R_addCallRoutine(DllInfo *info, const R_CallMethodDef * const croutine,
 		 Rf_DotCallSymbol *sym)
 {
@@ -360,25 +381,31 @@ R_addCallRoutine(DllInfo *info, const R_CallMethodDef * const croutine,
     sym->numArgs = croutine->numArgs > -1 ? croutine->numArgs : -1;
 }
 
-void attribute_hidden
+static void
 Rf_freeCSymbol(Rf_DotCSymbol *sym)
 {
     free(sym->name);
 }
 
-void attribute_hidden
+static void
 Rf_freeCallSymbol(Rf_DotCallSymbol *sym)
 {
     free(sym->name);
 }
 
-void attribute_hidden
+static void
+Rf_freeExternalSymbol(Rf_DotCallSymbol *sym)
+{
+    free(sym->name);
+}
+
+static void
 Rf_freeFortranSymbol(Rf_DotFortranSymbol *sym)
 {
     free(sym->name);
 }
 
-void attribute_hidden
+static void
 Rf_freeDllInfo(DllInfo *info)
 {
     int i;
@@ -394,6 +421,11 @@ Rf_freeDllInfo(DllInfo *info)
 	    Rf_freeCallSymbol(info->CallSymbols+i);
 	free(info->CallSymbols);
     }
+    if(info->ExternalSymbols) {
+	for(i = 0; i < info->numExternalSymbols; i++)
+	    Rf_freeExternalSymbol(info->ExternalSymbols+i);
+	free(info->ExternalSymbols);
+    }
     if(info->FortranSymbols) {
 	for(i = 0; i < info->numFortranSymbols; i++)
 	    Rf_freeFortranSymbol(info->FortranSymbols+i);
@@ -402,16 +434,19 @@ Rf_freeDllInfo(DllInfo *info)
 }
 
 
+typedef void (*DllInfoUnloadCall)(DllInfo *);
+typedef DllInfoUnloadCall DllInfoInitCall;
+
 static Rboolean
 R_callDLLUnload(DllInfo *dllInfo)
 {
     char buf[1024];
-    DL_FUNC f;
+    DllInfoUnloadCall f;
     R_RegisteredNativeSymbol symbol;
     symbol.type = R_ANY_SYM;
 
     snprintf(buf, 1024, "R_unload_%s", dllInfo->name);
-    f = R_dlsym(dllInfo, buf, &symbol);
+    f = (DllInfoUnloadCall) R_dlsym(dllInfo, buf, &symbol);
     if(f)
        f(dllInfo);
 
@@ -517,7 +552,7 @@ static DllInfo* AddDLL(char *path, int asLocal, int now)
     */
     if(info) {
 	char *tmp;
-	DL_FUNC f;
+	DllInfoInitCall f;
 #ifdef HAVE_NO_SYMBOL_UNDERSCORE
 	tmp = (char*) malloc(sizeof(char)*(strlen("R_init_") +
 					   strlen(info->name)+ 1));
@@ -527,7 +562,7 @@ static DllInfo* AddDLL(char *path, int asLocal, int now)
 					   strlen(info->name)+ 2));
 	sprintf(tmp, "_%s%s","R_init_", info->name);
 #endif
-	f = (DL_FUNC) R_osDynSymbol->dlsym(info, tmp);
+	f = (DllInfoInitCall) R_osDynSymbol->dlsym(info, tmp);
 	free(tmp);
 	if(f)
 	    f(info);
@@ -537,8 +572,8 @@ static DllInfo* AddDLL(char *path, int asLocal, int now)
     return info;
 }
 
-attribute_hidden
-DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
+
+static DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
 {
     char *dpath,  DLLname[PATH_MAX], *p;
     DllInfo *info;
@@ -549,7 +584,7 @@ DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
     */
     info->useDynamicLookup = TRUE;
 
-    dpath = malloc(strlen(path)+1);
+    dpath = (char *) malloc(strlen(path)+1);
     if(dpath == NULL) {
 	strcpy(DLLerror, _("could not allocate space for 'path'"));
 	R_osDynSymbol->closeLibrary(handle);
@@ -579,11 +614,11 @@ DllInfo *R_RegisterDLL(HINSTANCE handle, const char *path)
     return(info);
 }
 
-int attribute_hidden
+static int
 addDLL(char *dpath, char *DLLname, HINSTANCE handle)
 {
     int ans = CountDLL;
-    char *name = malloc(strlen(DLLname)+1);
+    char *name = (char *) malloc(strlen(DLLname)+1);
     if(name == NULL) {
 	strcpy(DLLerror, _("could not allocate space for 'name'"));
 	if(handle)
@@ -608,7 +643,7 @@ addDLL(char *dpath, char *DLLname, HINSTANCE handle)
     return(ans);
 }
 
-Rf_DotCSymbol *
+static Rf_DotCSymbol *
 Rf_lookupRegisteredCSymbol(DllInfo *info, const char *name)
 {
     int i;
@@ -620,7 +655,7 @@ Rf_lookupRegisteredCSymbol(DllInfo *info, const char *name)
     return(NULL);
 }
 
-Rf_DotFortranSymbol *
+static Rf_DotFortranSymbol *
 Rf_lookupRegisteredFortranSymbol(DllInfo *info, const char *name)
 {
     int i;
@@ -632,7 +667,7 @@ Rf_lookupRegisteredFortranSymbol(DllInfo *info, const char *name)
     return((Rf_DotFortranSymbol*)NULL);
 }
 
-Rf_DotCallSymbol *
+static Rf_DotCallSymbol *
 Rf_lookupRegisteredCallSymbol(DllInfo *info, const char *name)
 {
     int i;
@@ -644,7 +679,7 @@ Rf_lookupRegisteredCallSymbol(DllInfo *info, const char *name)
     return((Rf_DotCallSymbol*)NULL);
 }
 
-Rf_DotExternalSymbol *
+static Rf_DotExternalSymbol *
 Rf_lookupRegisteredExternalSymbol(DllInfo *info, const char *name)
 {
     int i;
@@ -656,7 +691,7 @@ Rf_lookupRegisteredExternalSymbol(DllInfo *info, const char *name)
     return((Rf_DotExternalSymbol*)NULL);
 }
 
-DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
+static DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
 				 R_RegisteredNativeSymbol *symbol)
 {
     int fail = 0;
@@ -729,8 +764,9 @@ DL_FUNC R_getDLLRegisteredSymbol(DllInfo *info, const char *name,
     return((DL_FUNC) NULL);
 }
 
-DL_FUNC R_dlsym(DllInfo *info, char const *name,
-		R_RegisteredNativeSymbol *symbol)
+DL_FUNC attribute_hidden
+R_dlsym(DllInfo *info, char const *name,
+	R_RegisteredNativeSymbol *symbol)
 {
     char buf[MAXIDSIZE+1];
     DL_FUNC f;
@@ -754,6 +790,12 @@ DL_FUNC R_dlsym(DllInfo *info, char const *name,
 	buf[strlen(buf)] = '_';
     }
 #endif
+#ifdef HAVE_F77_EXTRA_UNDERSCORE
+    if(strchr(name, '_') && symbol && symbol->type == R_FORTRAN_SYM) {
+	buf[strlen(buf)+1] = '\0';
+	buf[strlen(buf)] = '_';
+    }
+#endif
 
     return (DL_FUNC) R_osDynSymbol->dlsym(info, buf);
 }
@@ -817,7 +859,7 @@ DL_FUNC R_FindSymbol(char const *name, char const *pkg,
 }
 
 
-static void GetFullDLLPath(SEXP call, char *buf, char *path)
+static void GetFullDLLPath(SEXP call, char *buf, const char *const path)
 {
     R_osDynSymbol->getFullDLLPath(call, buf, path);
 }
@@ -847,13 +889,12 @@ SEXP attribute_hidden do_dynload(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op,args);
     if (!isString(CAR(args)) || length(CAR(args)) < 1)
-	errorcall(call, _("character argument expected"));
-    GetFullDLLPath(call, buf, CHAR(STRING_ELT(CAR(args), 0)));
+	error(_("character argument expected"));
+    GetFullDLLPath(call, buf, translateChar(STRING_ELT(CAR(args), 0)));
     /* AddDLL does this DeleteDLL(buf); */
     info = AddDLL(buf, LOGICAL(CADR(args))[0], LOGICAL(CADDR(args))[0]);
     if(!info)
-	errorcall(call, _("unable to load shared library '%s':\n  %s"),
-		  buf, DLLerror);
+	error(_("unable to load shared library '%s':\n  %s"), buf, DLLerror);
     return(Rf_MakeDLLInfo(info));
 }
 
@@ -863,15 +904,15 @@ SEXP attribute_hidden do_dynunload(SEXP call, SEXP op, SEXP args, SEXP env)
 
     checkArity(op,args);
     if (!isString(CAR(args)) || length(CAR(args)) < 1)
-	errorcall(call, _("character argument expected"));
-    GetFullDLLPath(call, buf, CHAR(STRING_ELT(CAR(args), 0)));
+	error(_("character argument expected"));
+    GetFullDLLPath(call, buf, translateChar(STRING_ELT(CAR(args), 0)));
     if(!DeleteDLL(buf))
-	errorcall(call, _("dynamic/shared library '%s\' was not loaded"),
-		  buf);
+	error(_("dynamic/shared library '%s\' was not loaded"), buf);
     return R_NilValue;
 }
 
-int moduleCdynload(char *module, int local, int now)
+attribute_hidden
+int R_moduleCdynload(const char *module, int local, int now)
 {
     char dllpath[PATH_MAX], *p = getenv("R_HOME");
     DllInfo *res;
@@ -897,19 +938,15 @@ int moduleCdynload(char *module, int local, int now)
   NativeSymbol and can be used to relay symbols from
   one library to another.
  */
-SEXP attribute_hidden
+static SEXP
 Rf_MakeNativeSymbolRef(DL_FUNC f)
 {                     
   SEXP ref, klass;
 
-  /* The (void *) here is illegal C */
-  PROTECT(ref = R_MakeExternalPtr((void *) f, Rf_install("native symbol"),
-				  R_NilValue));
-
-  PROTECT(klass = allocVector(STRSXP, 1));
-  SET_STRING_ELT(klass, 0, mkChar("NativeSymbol"));
+  PROTECT(ref = R_MakeExternalPtrFn(f, Rf_install("native symbol"),
+				    R_NilValue));
+  PROTECT(klass = mkString("NativeSymbol"));
   setAttrib(ref, R_ClassSymbol, klass);
-
   UNPROTECT(2);
   return(ref);
 }
@@ -923,7 +960,7 @@ freeRegisteredNativeSymbolCopy(SEXP ref)
        free(ptr);
 }
 
-SEXP attribute_hidden
+static SEXP
 Rf_MakeRegisteredNativeSymbol(R_RegisteredNativeSymbol *symbol)
 {                     
   SEXP ref, klass;
@@ -935,13 +972,12 @@ Rf_MakeRegisteredNativeSymbol(R_RegisteredNativeSymbol *symbol)
   }
   *copy = *symbol;
 
-  PROTECT(ref = R_MakeExternalPtr((void *) copy, 
-                                  Rf_install("registered native symbol"),
-                                  R_NilValue));
+  PROTECT(ref = R_MakeExternalPtr(copy, 
+				  Rf_install("registered native symbol"),
+				  R_NilValue));
   R_RegisterCFinalizer(ref, freeRegisteredNativeSymbolCopy);
 
-  PROTECT(klass = allocVector(STRSXP, 1));
-  SET_STRING_ELT(klass, 0, mkChar("RegisteredNativeSymbol"));
+  PROTECT(klass = mkString("RegisteredNativeSymbol"));
   setAttrib(ref, R_ClassSymbol, klass);
 
   UNPROTECT(2);
@@ -982,7 +1018,7 @@ Rf_makeDllInfoReference(HINSTANCE inst)
  name of the DLL and whether we only look for symbols that have been
  registered in this DLL or do we also use dynamic lookup.
  */
-SEXP attribute_hidden
+static SEXP
 Rf_MakeDLLInfo(DllInfo *info)
 {
     SEXP ref, elNames, tmp;
@@ -1034,18 +1070,18 @@ Rf_MakeDLLInfo(DllInfo *info)
 SEXP attribute_hidden
 R_getSymbolInfo(SEXP sname, SEXP spackage, SEXP withRegistrationInfo)
 {
-    char *package, *name;
+    const char *package, *name;
     R_RegisteredNativeSymbol symbol = {R_ANY_SYM, {NULL}, NULL};
     SEXP sym = R_NilValue;
     DL_FUNC f = NULL;
 
     package = "";
 
-    name = CHAR(STRING_ELT(sname, 0));
+    name = translateChar(STRING_ELT(sname, 0));
 
     if(length(spackage)) {
 	if(TYPEOF(spackage) == STRSXP) {
-	    package = CHAR(STRING_ELT(spackage, 0));
+	    package = translateChar(STRING_ELT(spackage, 0));
 	}
 	else if(TYPEOF(spackage) == EXTPTRSXP &&
 		R_ExternalPtrTag(spackage) == Rf_install("DLLInfo")) {
@@ -1275,7 +1311,7 @@ SEXP attribute_hidden do_dynunload(SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 SEXP attribute_hidden
-R_getSymbolInfo(SEXP sname, SEXP spackage)
+R_getSymbolInfo(SEXP sname, SEXP spackage, SEXP withRegistrationInfo)
 {
     error(_("no dyn.load support in this R version"));
 }
@@ -1293,3 +1329,49 @@ R_getRegisteredRoutines(SEXP dll)
 }
 
 #endif
+
+/* Experimental interface for exporting and importing functions and
+   data from one package for use from C code in a package.  The
+   registration part probably ought to be integrated with the other
+   registrations.  The naming of these routines may be less than
+   ideal. */
+
+static SEXP CEntryTable = NULL;
+
+static SEXP get_package_CEntry_table(const char *package)
+{
+    SEXP penv, pname;
+
+    if (CEntryTable == NULL) {
+	CEntryTable = R_NewHashedEnv(R_NilValue, ScalarInteger(0));
+	R_PreserveObject(CEntryTable);
+    }
+    pname = install(package);
+    penv = findVarInFrame(CEntryTable, pname);
+    if (penv == R_UnboundValue) {
+	penv = R_NewHashedEnv(R_NilValue, ScalarInteger(0));
+	defineVar(pname, penv, CEntryTable);
+    }
+    return penv;
+}
+
+
+void R_RegisterCCallable(const char *package, const char *name, DL_FUNC fptr)
+{
+    SEXP penv = get_package_CEntry_table(package);
+    SEXP eptr = R_MakeExternalPtrFn(fptr, R_NilValue, R_NilValue);
+    PROTECT(eptr);
+    defineVar(install(name), eptr, penv);
+    UNPROTECT(1);
+}
+
+DL_FUNC R_GetCCallable(const char *package, const char *name)
+{
+    SEXP penv = get_package_CEntry_table(package);
+    SEXP eptr = findVarInFrame(penv, install(name));
+    if (eptr == R_UnboundValue)
+	error(_("function '%s' not provided by package '%s'"), name, package);
+    else if (TYPEOF(eptr) != EXTPTRSXP) 
+	error(_("table entry must be an external pointer"));
+    return R_ExternalPtrAddrFn(eptr);
+}
