@@ -1,8 +1,8 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2003   The R Development Core Team.
- *  Copyright (C) 2004        The R Foundation
+ *  Copyright (C) 1998-2007   The R Development Core Team.
+ *  Copyright (C) 2004-5        The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,10 +14,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *
  *  Symbolic Differentiation
@@ -89,9 +88,7 @@ static void InitDerivSymbols()
 
 static SEXP Constant(double x)
 {
-    SEXP s = allocVector(REALSXP, 1);
-    REAL(s)[0] = x;
-    return s;
+    return ScalarReal(x);
 }
 
 static int isZero(SEXP s)
@@ -115,7 +112,7 @@ static int isUminus(SEXP s)
 		return 1;
 	    else return 0;
 	default:
-	    error("invalid form in unary minus check");
+	    error(_("invalid form in unary minus check"));
 	    return -1;/* for -Wall */
 	}
     }
@@ -480,9 +477,9 @@ static SEXP D(SEXP expr, SEXP var)
 	}
 
 	else {
-	    SEXP u = deparse1(CAR(expr), 0);
-	    error("Function `%s' is not in the derivatives table",
-		  CHAR(STRING_ELT(u, 0)));
+	    SEXP u = deparse1(CAR(expr), 0, SIMPLEDEPARSE);
+	    error(_("Function '%s' is not in the derivatives table"),
+		  translateChar(STRING_ELT(u, 0)));
 	}
 
 	break;
@@ -580,7 +577,7 @@ static SEXP AddParens(SEXP expr)
     return expr;
 }
 
-SEXP do_D(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_D(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP expr, var;
     checkArity(op, args);
@@ -588,10 +585,10 @@ SEXP do_D(SEXP call, SEXP op, SEXP args, SEXP env)
     else expr = CAR(args);
     var = CADR(args);
     if (!isString(var) || length(var) < 1)
-	errorcall(call, "variable must be a character string");
+	error(_("variable must be a character string"));
     if (length(var) > 1)
-	warningcall(call, "only the first element is used as variable name");
-    var = install(CHAR(STRING_ELT(var, 0)));
+	warning(_("only the first element is used as variable name"));
+    var = install(translateChar(STRING_ELT(var, 0)));
     InitDerivSymbols();
     PROTECT(expr = D(expr, var));
     expr = AddParens(expr);
@@ -603,7 +600,7 @@ SEXP do_D(SEXP call, SEXP op, SEXP args, SEXP env)
 
 static void InvalidExpression(char *where)
 {
-    error("invalid expression in \"%s\"", where);
+    error(_("invalid expression in \"%s\""), where);
 }
 
 static int equal(SEXP expr1, SEXP expr2)
@@ -666,7 +663,7 @@ static int Accumulate2(SEXP expr, SEXP exprlist)
 static SEXP MakeVariable(int k, SEXP tag)
 {
     char buf[64];
-    snprintf(buf, 64, "%s%d", CHAR(STRING_ELT(tag, 0)), k);
+    snprintf(buf, 64, "%s%d", translateChar(STRING_ELT(tag, 0)), k);
     return install(buf);
 }
 
@@ -748,18 +745,15 @@ static SEXP CreateGrad(SEXP names)
     PROTECT(q = allocList(n));
     SETCADDR(dimnames, LCONS(p, q));
     UNPROTECT(1);
-    for(i=0 ; i<n ; i++) {
-	SETCAR(q, allocVector(STRSXP, 1));
-	SET_STRING_ELT(CAR(q), 0, STRING_ELT(names, i));
+    for(i = 0 ; i < n ; i++) {
+	SETCAR(q, ScalarString(STRING_ELT(names, i)));
 	q = CDR(q);
     }
     PROTECT(dim = lang3(R_NilValue, R_NilValue, R_NilValue));
     SETCAR(dim, install("c"));
     SETCADR(dim, lang2(install("length"), install(".value")));
-    SETCADDR(dim, allocVector(REALSXP, 1));
-    REAL(CADDR(dim))[0] = length(names);
-    PROTECT(data = allocVector(REALSXP, 1));
-    REAL(data)[0] = 0;
+    SETCADDR(dim, ScalarInteger(length(names))); /* was real? */
+    PROTECT(data = ScalarReal(0.));
     PROTECT(p = lang4(install("array"), data, dim, dimnames));
     p = lang3(install("<-"), install(".grad"), p);
     UNPROTECT(4);
@@ -777,21 +771,17 @@ static SEXP CreateHess(SEXP names)
     PROTECT(q = allocList(n));
     SETCADDR(dimnames, LCONS(p, q));
     UNPROTECT(1);
-    for(i=0 ; i<n ; i++) {
-	SETCAR(q, allocVector(STRSXP, 1));
-	SET_STRING_ELT(CAR(q), 0, STRING_ELT(names, i));
+    for(i = 0 ; i < n ; i++) {
+	SETCAR(q, ScalarString(STRING_ELT(names, i)));
 	q = CDR(q);
     }
     SETCADDDR(dimnames, duplicate(CADDR(dimnames)));
     PROTECT(dim = lang4(R_NilValue, R_NilValue, R_NilValue,R_NilValue));
     SETCAR(dim, install("c"));
     SETCADR(dim, lang2(install("length"), install(".value")));
-    SETCADDR(dim, allocVector(REALSXP, 1));
-    REAL(CADDR(dim))[0] = length(names);
-    SETCADDDR(dim, allocVector(REALSXP, 1));
-    REAL(CADDDR(dim))[0] = length(names);
-    PROTECT(data = allocVector(REALSXP, 1));
-    REAL(data)[0] = 0;
+    SETCADDR(dim, ScalarInteger(length(names)));
+    SETCADDDR(dim, ScalarInteger(length(names)));
+    PROTECT(data = ScalarReal(0.));
     PROTECT(p = lang4(install("array"), data, dim, dimnames));
     p = lang3(install("<-"), install(".hessian"), p);
     UNPROTECT(4);
@@ -802,8 +792,7 @@ static SEXP DerivAssign(SEXP name, SEXP expr)
 {
     SEXP ans, newname;
     PROTECT(ans = lang3(install("<-"), R_NilValue, expr));
-    PROTECT(newname = allocVector(STRSXP, 1));
-    SET_STRING_ELT(newname, 0, name);
+    PROTECT(newname = ScalarString(name));
     SETCADR(ans, lang4(install("["), install(".grad"), R_MissingArg, newname));
     UNPROTECT(2);
     return ans;
@@ -821,8 +810,7 @@ static SEXP HessAssign1(SEXP name, SEXP expr)
 {
     SEXP ans, newname;
     PROTECT(ans = lang3(install("<-"), R_NilValue, expr));
-    PROTECT(newname = allocVector(STRSXP, 1));
-    SET_STRING_ELT(newname, 0, name);
+    PROTECT(newname = ScalarString(name));
     SETCADR(ans, lang5(install("["), install(".hessian"), R_MissingArg,
 		       newname, newname));
     UNPROTECT(2);
@@ -832,10 +820,8 @@ static SEXP HessAssign1(SEXP name, SEXP expr)
 static SEXP HessAssign2(SEXP name1, SEXP name2, SEXP expr)
 {
     SEXP ans, newname1, newname2;
-    PROTECT(newname1 = allocVector(STRSXP, 1));
-    PROTECT(newname2 = allocVector(STRSXP, 1));
-    SET_STRING_ELT(newname1, 0, name1);
-    SET_STRING_ELT(newname2, 0, name2);
+    PROTECT(newname1 = ScalarString(name1));
+    PROTECT(newname2 = ScalarString(name2));
     ans = lang3(install("<-"),
 		lang5(install("["), install(".hessian"), R_MissingArg,
 		      newname1, newname2),
@@ -879,16 +865,14 @@ static SEXP Prune(SEXP lst)
     else return lst ;
 }
 
-SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
 {
 /* deriv.default(expr, namevec, function.arg, tag, hessian) */
     SEXP ans, ans2, expr, funarg, names, s;
     int f_index, *d_index, *d2_index;
     int i, j, k, nexpr, nderiv=0, hessian;
-    char *vmax;
     SEXP exprlist, tag;
     checkArity(op, args);
-    vmax = vmaxget();
     InitDerivSymbols();
     PROTECT(exprlist = LCONS(install("{"), R_NilValue));
     /* expr: */
@@ -899,7 +883,7 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
     /* namevec: */
     names = CAR(args);
     if (!isString(names) || (nderiv = length(names)) < 1)
-	errorcall(call, "invalid variable names");
+	error(_("invalid variable names"));
     args = CDR(args);
     /* function.arg: */
     funarg = CAR(args);
@@ -908,7 +892,7 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
     tag = CAR(args);
     if (!isString(tag) || length(tag) < 1
 	|| length(STRING_ELT(tag, 0)) < 1 || length(STRING_ELT(tag, 0)) > 60)
-	errorcall(call, "invalid tag");
+	error(_("invalid tag"));
     args = CDR(args);
     /* hessian: */
     hessian = asLogical(CAR(args));
@@ -922,14 +906,14 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
     UNPROTECT(1);
     for(i=0, k=0; i<nderiv ; i++) {
 	PROTECT(ans = duplicate(expr));
-	PROTECT(ans = D(ans, install(CHAR(STRING_ELT(names, i)))));
+	PROTECT(ans = D(ans, install(translateChar(STRING_ELT(names, i)))));
 	ans2 = duplicate(ans);	/* keep a temporary copy */
 	d_index[i] = FindSubexprs(ans, exprlist, tag); /* examine the derivative first */
 	ans = duplicate(ans2);	/* restore the copy */
 	if (hessian) {
 	    for(j = i; j < nderiv; j++) {
 		PROTECT(ans2 = duplicate(ans));
-		PROTECT(ans2 = D(ans2, install(CHAR(STRING_ELT(names, j)))));
+		PROTECT(ans2 = D(ans2, install(translateChar(STRING_ELT(names, j)))));
 		d2_index[k] = FindSubexprs(ans2, exprlist, tag);
 		k++;
 		UNPROTECT(2);
@@ -953,14 +937,13 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
 	    Accumulate2(MakeVariable(d_index[i], tag), exprlist);
 	    if (hessian) {
 		PROTECT(ans = duplicate(expr));
-		PROTECT(ans = D(ans, install(CHAR(STRING_ELT(names, i)))));
+		PROTECT(ans = D(ans, install(translateChar(STRING_ELT(names, i)))));
 		for (j = i; j < nderiv; j++) {
 		    if (d2_index[k]) {
 			Accumulate2(MakeVariable(d2_index[k], tag), exprlist);
 		    } else {
 			PROTECT(ans2 = duplicate(ans));
-			PROTECT(ans2 = D(ans2,
-					 install(CHAR(STRING_ELT(names, j)))));
+			PROTECT(ans2 = D(ans2, install(translateChar(STRING_ELT(names, j)))));
 			Accumulate2(ans2, exprlist);
 			UNPROTECT(2);
 		    }
@@ -970,7 +953,7 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	} else { /* the first derivative is constant or simple variable */
 	    PROTECT(ans = duplicate(expr));
-	    PROTECT(ans = D(ans, install(CHAR(STRING_ELT(names, i)))));
+	    PROTECT(ans = D(ans, install(translateChar(STRING_ELT(names, i)))));
 	    Accumulate2(ans, exprlist);
 	    UNPROTECT(2);
 	    if (hessian) {
@@ -979,8 +962,7 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
 			Accumulate2(MakeVariable(d2_index[k], tag), exprlist);
 		    } else {
 			PROTECT(ans2 = duplicate(ans));
-			PROTECT(ans2 = D(ans2,
-					 install(CHAR(STRING_ELT(names, j)))));
+			PROTECT(ans2 = D(ans2, install(translateChar(STRING_ELT(names, j)))));
 			if(isZero(ans2)) Accumulate2(R_MissingArg, exprlist);
 			else Accumulate2(ans2, exprlist);
 			UNPROTECT(2);
@@ -1060,7 +1042,7 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
 	PROTECT(ans = allocList(length(names)));
 	SET_FORMALS(funarg, ans);
 	for(i = 0; i < length(names); i++) {
-	    SET_TAG(ans, install(CHAR(STRING_ELT(names, i))));
+	    SET_TAG(ans, install(translateChar(STRING_ELT(names, i))));
 	    SETCAR(ans, R_MissingArg);
 	    ans = CDR(ans);
 	}
@@ -1074,6 +1056,5 @@ SEXP do_deriv(SEXP call, SEXP op, SEXP args, SEXP env)
 	/* funarg = lang2(install("expression"), exprlist); */
     }
     UNPROTECT(2);
-    vmaxset(vmax);
     return funarg;
 }
