@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2002   Robert Gentleman, Ross Ihaka and the R core team.
+ *  Copyright (C) 1997-2007   Robert Gentleman, Ross Ihaka and the R core team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> char here is either ASCII or handled as a whole */
@@ -781,7 +781,7 @@ int addContourLines(double *x, int nx, double *y, int ny,
 SEXP GEcontourLines(double *x, int nx, double *y, int ny,
 		    double *z, double *levels, int nl)
 {
-    char *vmax;
+    void *vmax;
     int i, nlines, len;
     double atom, zmin, zmax;
     SEGP* segmentDB;
@@ -866,8 +866,6 @@ SEXP GEcontourLines(double *x, int nx, double *y, int ny,
     return mainlist;
 }
 
-SEXP GEdrawContourLines();
-
 SEXP attribute_hidden do_contourLines(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP oargs, c, x, y, z;
@@ -1050,13 +1048,14 @@ static void contour(SEXP x, int nx, SEXP y, int ny, SEXP z,
 		buffer[0] = ' ';
 		if (!isNull(labels)) {
 		    int numl = length(labels);
-		    strcpy(&buffer[1], CHAR(STRING_ELT(labels, cnum % numl)));
+		    strcpy(&buffer[1], 
+			   translateChar(STRING_ELT(labels, cnum % numl)));
 		}
 		else {
 		    PROTECT(lab = allocVector(REALSXP, 1));
 		    REAL(lab)[0] = zc;
 		    lab = labelformat(lab);
-		    strcpy(&buffer[1], CHAR(STRING_ELT(lab, 0)));
+		    strcpy(&buffer[1], CHAR(STRING_ELT(lab, 0))); /* ASCII */
 		    UNPROTECT(1);
 		}
 		buffer[strlen(buffer)+1] = '\0';
@@ -1065,11 +1064,11 @@ static void contour(SEXP x, int nx, SEXP y, int ny, SEXP z,
 		if (vectorFonts) {
 		    /* 1, 1 => sans serif, basic font */
 		    labelDistance =
-			GVStrWidth((unsigned char *)buffer, typeface, fontindex,
-				   INCHES, dd);
+			GVStrWidth(buffer, typeface,
+				   fontindex, INCHES, dd);
 		    labelHeight =
-			GVStrHeight((unsigned char *)buffer, typeface, fontindex,
-				    INCHES, dd);
+			GVStrHeight(buffer, typeface,
+				    fontindex, INCHES, dd);
 		}
 		else {
 		    labelDistance = GStrWidth(buffer, INCHES, dd);
@@ -1375,7 +1374,7 @@ SEXP attribute_hidden do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
     GCheckState(dd);
 
     if (length(args) < 4)
-	errorcall(call, _("too few arguments"));
+	error(_("too few arguments"));
 
     oargs = args;
 
@@ -1412,7 +1411,7 @@ SEXP attribute_hidden do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
 
     method = asInteger(CAR(args)); args = CDR(args);
     if (method < 1 || method > 3)
-	errorcall(call, _("invalid '%s' value"), "method");
+	error(_("invalid '%s' value"), "method");
 
     PROTECT(vfont = FixupVFont(CAR(args)));
     if (!isNull(vfont)) {
@@ -1436,31 +1435,31 @@ SEXP attribute_hidden do_contour(SEXP call, SEXP op, SEXP args, SEXP env)
     args = CDR(args);
 
     if (nx < 2 || ny < 2)
-	errorcall(call, _("insufficient 'x' or 'y' values"));
+	error(_("insufficient 'x' or 'y' values"));
 
     if (nrows(z) != nx || ncols(z) != ny)
-	errorcall(call, _("dimension mismatch"));
+	error(_("dimension mismatch"));
 
     if (nc < 1)
-	errorcall(call, _("no contour values"));
+	error(_("no contour values"));
 
     for (i = 0; i < nx; i++) {
 	if (!R_FINITE(REAL(x)[i]))
-	    errorcall(call, _("missing 'x' values"));
+	    error(_("missing 'x' values"));
 	if (i > 0 && REAL(x)[i] < REAL(x)[i - 1])
-	    errorcall(call, _("increasing 'x' values expected"));
+	    error(_("increasing 'x' values expected"));
     }
 
     for (i = 0; i < ny; i++) {
 	if (!R_FINITE(REAL(y)[i]))
-	    errorcall(call, _("missing 'y' values"));
+	    error(_("missing 'y' values"));
 	if (i > 0 && REAL(y)[i] < REAL(y)[i - 1])
-	    errorcall(call, _("increasing 'y' values expected"));
+	    error(_("increasing 'y' values expected"));
     }
 
     for (i = 0; i < nc; i++)
 	if (!R_FINITE(REAL(c)[i]))
-	    errorcall(call, _("invalid NA contour values"));
+	    error(_("invalid NA contour values"));
 
     zmin = DBL_MAX;
     zmax = DBL_MIN;
@@ -1689,13 +1688,13 @@ SEXP attribute_hidden do_filledcontour(SEXP call, SEXP op, SEXP args, SEXP env)
     args = CDR(args);
 
     if (nx < 2 || ny < 2)
-	errorcall(call, _("insufficient 'x' or 'y' values"));
+	error(_("insufficient 'x' or 'y' values"));
 
     if (nrows(sz) != nx || ncols(sz) != ny)
-	errorcall(call, _("dimension mismatch"));
+	error(_("dimension mismatch"));
 
     if (nc < 1)
-	errorcall(call, _("no contour values"));
+	error(_("no contour values"));
 
     PROTECT(scol = FixupCol(CAR(args), R_TRANWHITE));
     ncol = length(scol);
@@ -1753,16 +1752,15 @@ SEXP attribute_hidden do_filledcontour(SEXP call, SEXP op, SEXP args, SEXP env)
     GMode(0, dd);
     Rf_gpptr(dd)->col = colsave;
     Rf_gpptr(dd)->xpd = xpdsave;
-    R_Visible = 0;
     UNPROTECT(1);
     if (GRecording(call, dd))
 	recordGraphicOperation(op, oargs, dd);
     return R_NilValue;
 
  badxy:
-    errorcall(call, _("invalid x / y values or limits"));
+    error(_("invalid x / y values or limits"));
  badlev:
-    errorcall(call, _("invalid contour levels: must be strictly increasing"));
+    error(_("invalid contour levels: must be strictly increasing"));
     return R_NilValue;  /* never used; to keep -Wall happy */
 }
 
@@ -1829,7 +1827,6 @@ SEXP attribute_hidden do_image(SEXP call, SEXP op, SEXP args, SEXP env)
     GMode(0, dd);
     Rf_gpptr(dd)->col = colsave;
     Rf_gpptr(dd)->xpd = xpdsave;
-    R_Visible = 0;
     UNPROTECT(1);
     if (GRecording(call, dd))
 	recordGraphicOperation(op, oargs, dd);
@@ -2207,7 +2204,7 @@ static double labelAngle(double x1, double y1, double x2, double y2) {
 
 static void PerspAxis(double *x, double *y, double *z,
 		      int axis, int axisType, int nTicks, int tickType,
-		      char *label, DevDesc *dd) {
+		      const char *label, DevDesc *dd) {
     Vector3d u1, u2, u3, v1, v2, v3;
     double tickLength = .03; /* proportion of axis length */
     double min, max, d_frac;
@@ -2215,6 +2212,10 @@ static void PerspAxis(double *x, double *y, double *z,
     double axp[3];
     int nint, i;
     SEXP at, lab;
+    double cexsave = Rf_gpptr(dd)->cex;
+    int fontsave = Rf_gpptr(dd)->font;
+
+
     switch (axisType) {
     case 0:
 	min = x[0];	max = x[1];	range = x;	break;
@@ -2309,10 +2310,16 @@ static void PerspAxis(double *x, double *y, double *z,
     TransVector(u2, VT, v2);
     TransVector(u3, VT, v3);
     /* Draw axis label */
+    /* change in 2.5.0 to use cex.lab and font.lab */
+    Rf_gpptr(dd)->cex = Rf_gpptr(dd)->cexbase * Rf_gpptr(dd)->cexlab;
+    Rf_gpptr(dd)->font = Rf_gpptr(dd)->fontlab;
     GText(v3[0]/v3[3], v3[1]/v3[3], USER, label, .5, .5,
 	  labelAngle(v1[0]/v1[3], v1[1]/v1[3], v2[0]/v2[3], v2[1]/v2[3]),
 	  dd);
     /* Draw axis ticks */
+    /* change in 2.5.0 to use cex.axis and font.axis */
+    Rf_gpptr(dd)->cex = Rf_gpptr(dd)->cexbase * Rf_gpptr(dd)->cexaxis;
+    Rf_gpptr(dd)->font = Rf_gpptr(dd)->fontaxis;
     switch (tickType) {
     case 1: /* "simple": just an arrow parallel to axis, indicating direction
 	       of increase */
@@ -2359,12 +2366,15 @@ static void PerspAxis(double *x, double *y, double *z,
 	    GLine(v1[0]/v1[3], v1[1]/v1[3],
 		  v2[0]/v2[3], v2[1]/v2[3], USER, dd);
 	    /* Draw tick label */
-	    GText(v3[0]/v3[3], v3[1]/v3[3], USER, CHAR(STRING_ELT(lab, i)),
+	    GText(v3[0]/v3[3], v3[1]/v3[3], USER,
+		  translateChar(STRING_ELT(lab, i)),
 		  .5, .5, 0, dd);
 	}
 	UNPROTECT(2);
 	break;
     }
+    Rf_gpptr(dd)->cex = cexsave;
+    Rf_gpptr(dd)->font = fontsave;
 }
 
 /* Determine the transformed (x, y) coordinates (in USER space)
@@ -2376,7 +2386,7 @@ static void PerspAxis(double *x, double *y, double *z,
  * has the lowest x-value to decide which of the z-axes to label
  */
 static void PerspAxes(double *x, double *y, double *z,
-                      char *xlab, char *ylab, char *zlab,
+                      const char *xlab, const char *ylab, const char *zlab,
 		      int nTicks, int tickType, DevDesc *dd) {
     int xAxis=0, yAxis=0, zAxis=0; /* -Wall */
     int xpdsave;
@@ -2452,43 +2462,43 @@ SEXP attribute_hidden do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
     DevDesc *dd;
 
     if (length(args) < 24)
-	errorcall(call, _("too few parameters"));
+	error(_("too few parameters"));
     gcall = call;
     originalArgs = args;
 
     PROTECT(x = coerceVector(CAR(args), REALSXP));
-    if (length(x) < 2) errorcall(call, _("invalid '%s' argument"), "x");
+    if (length(x) < 2) error(_("invalid '%s' argument"), "x");
     args = CDR(args);
 
     PROTECT(y = coerceVector(CAR(args), REALSXP));
-    if (length(y) < 2) errorcall(call, _("invalid '%s' argument"), "x");
+    if (length(y) < 2) error(_("invalid '%s' argument"), "y");
     args = CDR(args);
 
     PROTECT(z = coerceVector(CAR(args), REALSXP));
     if (!isMatrix(z) || nrows(z) != length(x) || ncols(z) != length(y))
-	errorcall(call, _("invalid '%s' argument"), "z");
+	error(_("invalid '%s' argument"), "z");
     args = CDR(args);
 
     PROTECT(xlim = coerceVector(CAR(args), REALSXP));
-    if (length(xlim) != 2) errorcall(call, _("invalid '%s' argument"), "xlim");
+    if (length(xlim) != 2) error(_("invalid '%s' argument"), "xlim");
     args = CDR(args);
 
     PROTECT(ylim = coerceVector(CAR(args), REALSXP));
-    if (length(ylim) != 2) errorcall(call, _("invalid '%s' argument"), "ylim");
+    if (length(ylim) != 2) error(_("invalid '%s' argument"), "ylim");
     args = CDR(args);
 
     PROTECT(zlim = coerceVector(CAR(args), REALSXP));
-    if (length(zlim) != 2) errorcall(call, _("invalid '%s' argument"), "zlim");
+    if (length(zlim) != 2) error(_("invalid '%s' argument"), "zlim");
     args = CDR(args);
 
     /* Checks on x/y/z Limits */
 
     if (!LimitCheck(REAL(xlim), &xc, &xs))
-	errorcall(call, _("invalid 'x' limits"));
+	error(_("invalid 'x' limits"));
     if (!LimitCheck(REAL(ylim), &yc, &ys))
-	errorcall(call, _("invalid 'y' limits"));
+	error(_("invalid 'y' limits"));
     if (!LimitCheck(REAL(zlim), &zc, &zs))
-	errorcall(call, _("invalid 'z' limits"));
+	error(_("invalid 'z' limits"));
 
     theta = asReal(CAR(args));	args = CDR(args);
     phi	  = asReal(CAR(args));	args = CDR(args);
@@ -2533,15 +2543,15 @@ SEXP attribute_hidden do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (!R_FINITE(theta) || !R_FINITE(phi) || !R_FINITE(r) || !R_FINITE(d) ||
 	d < 0 || r < 0)
-	errorcall(call, _("invalid viewing parameters"));
+	error(_("invalid viewing parameters"));
     if (!R_FINITE(expand) || expand < 0)
-	errorcall(call, _("invalid '%s' value"), "expand");
+	error(_("invalid '%s' value"), "expand");
     if (scale == NA_LOGICAL)
 	scale = 0;
     if ((nTicks == NA_INTEGER) || (nTicks < 0))
-	errorcall(call, _("invalid '%s' value"), "nticks");
+	error(_("invalid '%s' value"), "nticks");
     if ((tickType == NA_INTEGER) || (tickType < 1) || (tickType > 2))
-	errorcall(call, _("invalid '%s' value"), "ticktype");
+	error(_("invalid '%s' value"), "ticktype");
 
     dd = CurrentDevice();
 
@@ -2549,11 +2559,11 @@ SEXP attribute_hidden do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
 
     PROTECT(col = FixupCol(col, Rf_gpptr(dd)->bg));
     ncol = LENGTH(col);
-    if (ncol < 1) errorcall(call, _("invalid '%s' specification"), "col");
+    if (ncol < 1) error(_("invalid '%s' specification"), "col");
     if(!R_OPAQUE(INTEGER(col)[0])) DoLighting = FALSE;
     PROTECT(border = FixupCol(border, Rf_gpptr(dd)->fg));
     if (length(border) < 1) 
-	errorcall(call, _("invalid '%s' specification"), "border");
+	error(_("invalid '%s' specification"), "border");
 
     GSetState(1, dd);
     GSavePars(dd);
@@ -2607,9 +2617,9 @@ SEXP attribute_hidden do_persp(SEXP call, SEXP op, SEXP args, SEXP env)
 	    SEXP xl = STRING_ELT(xlab, 0), yl = STRING_ELT(ylab, 0),
 		zl = STRING_ELT(zlab, 0);
 	    PerspAxes(REAL(xlim), REAL(ylim), REAL(zlim),
-		      (xl == NA_STRING)? "" : CHAR(xl),
-		      (yl == NA_STRING)? "" : CHAR(yl),
-		      (zl == NA_STRING)? "" : CHAR(zl),
+		      (xl == NA_STRING)? "" : translateChar(xl),
+		      (yl == NA_STRING)? "" : translateChar(yl),
+		      (zl == NA_STRING)? "" : translateChar(zl),
 		      nTicks, tickType, dd);
 	}
     }
