@@ -1,10 +1,11 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2000	The R Development Core Team
+ *  Copyright (C) 1998-2008	The R Development Core Team
  *
  *  This source code module:
  *  Copyright (C) 1997, 1998 Paul Murrell and Ross Ihaka
+ *  Copyright (C) 1998-2008	The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +18,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8-FIXME>
@@ -30,15 +31,16 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include <Defn.h>
 
 #include <ctype.h>
 #ifdef SUPPORT_MBCS
+# include <R_ext/rlocale.h>
 # include <wchar.h>
 # include <wctype.h>
 #endif
 
 
-#include <Defn.h>
 #include <Rmath.h>
 #include <Graphics.h>
 
@@ -589,7 +591,17 @@ static BBOX DrawBBox(BBOX bbox, double xoffset, double yoffset,
     y[3] = ConvertedY(mc, dd);
     gc->col = mc->BoxColor;
     gc->lty = LTY_SOLID;
-    gc->lwd = 1;
+    /*
+     * We used to just force lwd = 1, which is a reasonable sanity check
+     * because if the user had set lwd = 10 for something, your
+     * mathematical annotation would look REALLY bad.
+     * Unfortunately, this meant that the user could not set a nice
+     * small lwd to get a finer line;  we always bumped it back up to 1.
+     * NOW we just reduce lwd to 1 (as a sanity check) ONLY if 
+     * lwd is greater than 1.
+     */
+    if (gc->lwd > 1)
+	gc->lwd = 1;
     GEPolyline(5, x, y, gc, dd);
     PMoveTo(xsaved, ysaved, mc);
     gc->col = savedcol;
@@ -606,15 +618,15 @@ typedef struct {
 
 /* Determine a match between symbol name and string. */
 
-static int NameMatch(SEXP expr, char *aString)
+static int NameMatch(SEXP expr, const char *aString)
 {
     if (!isSymbol(expr)) return 0;
     return !strcmp(CHAR(PRINTNAME(expr)), aString);
 }
 
-static int StringMatch(SEXP expr, char *aString)
+static int StringMatch(SEXP expr, const char *aString)
 {
-    return !strcmp(CHAR(STRING_ELT(expr, 0)), aString);
+    return !strcmp(translateChar(STRING_ELT(expr, 0)), aString);
 }
 /* Code to determine the ascii code corresponding */
 /* to an element of a mathematical expression. */
@@ -699,6 +711,7 @@ SymbolTable[] = {
     { "Eta",		 72 },
     { "Iota",		 73 },
     { "theta1",		 74 },
+    { "vartheta",	 74 },
     { "Kappa",		 75 },
     { "Lambda",		 76 },
     { "Mu",		 77 },
@@ -711,6 +724,8 @@ SymbolTable[] = {
     { "Tau",		 84 },
     { "Upsilon",	 85 },
     { "sigma1",		 86 },
+    { "varsigma",	 86 },
+    { "stigma",		 86 },
     { "Omega",		 87 },
     { "Xi",		 88 },
     { "Psi",		 89 },
@@ -733,6 +748,7 @@ SymbolTable[] = {
     { "eta",		104 },
     { "iota",		105 },
     { "phi1",		106 },
+    { "varphi",		106 },
     { "kappa",		107 },
     { "lambda",		108 },
     { "mu",		109 },
@@ -860,6 +876,7 @@ static int SymbolCode(SEXP expr)
     return 0;
 }
 
+/* this is the one really used: */
 static int TranslatedSymbol(SEXP expr)
 {
     int code = SymbolCode(expr);
@@ -983,7 +1000,9 @@ static BBOX RenderGap(double gap, int draw, mathContext *mc,
     return MakeBBox(0, 0, gap);
 }
 
-/* Draw a Symbol from the Special Font */
+/* Draw a Symbol from the Special Font:
+   this is assumed to be 8-bit encoded in Adobe Symbol.
+ */
 
 static BBOX RenderSymbolChar(int ascii, int draw, mathContext *mc,
 			     R_GE_gcontext *gc, GEDevDesc *dd)
@@ -997,7 +1016,6 @@ static BBOX RenderSymbolChar(int ascii, int draw, mathContext *mc,
 	prev = SetFont(SymbolFont, gc);
     bbox = GlyphBBox(ascii, gc, dd);
     if (draw) {
-	/* <UTF8-FIXME> */
 	asciiStr[0] = ascii;
 	asciiStr[1] = '\0';
 	GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), asciiStr,
@@ -1009,49 +1027,93 @@ static BBOX RenderSymbolChar(int ascii, int draw, mathContext *mc,
     return bbox;
 }
 
-/* Draw a Symbol String in "Math Mode */
+/* Draw a Symbol String in "Math Mode" */
 /* This code inserts italic corrections after */
 /* every character. */
 
-static BBOX RenderSymbolStr(char *str, int draw, mathContext *mc,
+static BBOX RenderSymbolStr(const char *str, int draw, mathContext *mc,
 			    R_GE_gcontext *gc, GEDevDesc *dd)
 {
-    char chr[2];
+    char chr[7] = "";
+    const char *s = str;
     BBOX glyphBBox;
     BBOX resultBBox = NullBBox();
     double lastItalicCorr = 0;
     FontType prevfont = GetFont(gc);
     FontType font = prevfont;
-    chr[1] = '\0';
+
     if (str) {
-	/* <UTF8-FIXME> perhaps */
-	char *s = str;
-	while (*s) {
-	    if (isdigit((int)*s) && font != PlainFont) {
-		font = PlainFont;
-		SetFont(PlainFont, gc);
+#ifdef SUPPORT_MBCS
+	if(mbcslocale) {  /* need to advance by character, not byte */
+	    wchar_t wc;
+	    mbstate_t mb_st;
+	    size_t res;
+
+	    memset(&mb_st, 0, sizeof(mb_st));
+	    while (*s) {
+		wc = 0;
+		res = mbrtowc(&wc, s, MB_LEN_MAX, &mb_st);
+		if(res == -1) error("invalid multibyte string '%s'", s);
+		if (iswdigit(wc) && font != PlainFont) {
+		    font = PlainFont;
+		    SetFont(PlainFont, gc);
+		}
+		else if (font != prevfont) {
+		    font = prevfont;
+		    SetFont(prevfont, gc);
+		}
+		glyphBBox = GlyphBBox((int)wc, gc, dd);
+		if (UsingItalics(gc))
+		    bboxItalic(glyphBBox) =
+			ItalicFactor * bboxHeight(glyphBBox);
+		else
+		    bboxItalic(glyphBBox) = 0;
+		if (draw) {
+		    memset(chr, 0, sizeof(chr));
+		    res  = wcrtomb(chr, wc, &mb_st);
+		    if(res == -1) error("invalid multibyte string");
+		    PMoveAcross(lastItalicCorr, mc);
+		    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr,
+			   0.0, 0.0, mc->CurrentAngle, gc,
+			   dd);
+		    PMoveAcross(bboxWidth(glyphBBox), mc);
+		}
+		bboxWidth(resultBBox) += lastItalicCorr;
+		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
+		lastItalicCorr = bboxItalic(glyphBBox);
+		s += res;
 	    }
-	    else if (font != prevfont) {
-		font = prevfont;
-		SetFont(prevfont, gc);
+	} else
+#endif
+	{
+	    while (*s) {
+		if (isdigit((int)*s) && font != PlainFont) {
+		    font = PlainFont;
+		    SetFont(PlainFont, gc);
+		}
+		else if (font != prevfont) {
+		    font = prevfont;
+		    SetFont(prevfont, gc);
+		}
+		glyphBBox = GlyphBBox(*s, gc, dd);
+		if (UsingItalics(gc))
+		    bboxItalic(glyphBBox) =
+			ItalicFactor * bboxHeight(glyphBBox);
+		else
+		    bboxItalic(glyphBBox) = 0;
+		if (draw) {
+		    chr[0] = *s;
+		    PMoveAcross(lastItalicCorr, mc);
+		    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr,
+			   0.0, 0.0, mc->CurrentAngle, gc,
+			   dd);
+		    PMoveAcross(bboxWidth(glyphBBox), mc);
+		}
+		bboxWidth(resultBBox) += lastItalicCorr;
+		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
+		lastItalicCorr = bboxItalic(glyphBBox);
+		s++;
 	    }
-	    glyphBBox = GlyphBBox(*s, gc, dd);
-	    if (UsingItalics(gc))
-		bboxItalic(glyphBBox) = ItalicFactor * bboxHeight(glyphBBox);
-	    else
-		bboxItalic(glyphBBox) = 0;
-	    if (draw) {
-		chr[0] = *s;
-		PMoveAcross(lastItalicCorr, mc);
-		GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), chr,
-		       0.0, 0.0, mc->CurrentAngle, gc,
-		       dd);
-		PMoveAcross(bboxWidth(glyphBBox), mc);
-	    }
-	    bboxWidth(resultBBox) += lastItalicCorr;
-	    resultBBox = CombineBBoxes(resultBBox, glyphBBox);
-	    lastItalicCorr = bboxItalic(glyphBBox);
-	    s++;
 	}
 	if (font != prevfont)
 	    SetFont(prevfont, gc);
@@ -1067,15 +1129,19 @@ static BBOX RenderChar(int ascii, int draw, mathContext *mc,
 		       R_GE_gcontext *gc, GEDevDesc *dd)
 {
     BBOX bbox;
-    char asciiStr[2];
+    char asciiStr[7];
+
     bbox = GlyphBBox(ascii, gc, dd);
     if (draw) {
-	/* <UTF8-FIXME> This appears only to get called with values
-	   for hat, tilde and ring.  The latter appears to be hardcoded
-	   in Latin-1.
-	 */
-	asciiStr[0] = ascii;
-	asciiStr[1] = '\0';
+        memset(asciiStr, 0, sizeof(asciiStr));
+#ifdef SUPPORT_MBCS
+	if(mbcslocale) {
+	    size_t res = wcrtomb(asciiStr, ascii, NULL);
+	    if(res == -1)
+		error("invalid character in current multibyte locale");
+	} else
+#endif
+	    asciiStr[0] = ascii;
 	GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), asciiStr,
 	       0.0, 0.0, mc->CurrentAngle, gc,
 	       dd);
@@ -1085,35 +1151,38 @@ static BBOX RenderChar(int ascii, int draw, mathContext *mc,
 }
 
 /* This gets called on strings and PRINTNAMES */
-static BBOX RenderStr(char *str, int draw, mathContext *mc,
+static BBOX RenderStr(const char *str, int draw, mathContext *mc,
 		      R_GE_gcontext *gc, GEDevDesc *dd)
 {
     BBOX glyphBBox;
     BBOX resultBBox = NullBBox();
+
     if (str) {
 #ifdef SUPPORT_MBCS
-	int n = strlen(str), used;
-	wchar_t wc;
-	char *p = str;
-	mbstate_t mb_st;
-	mbs_init(&mb_st);
-	while ((used = Mbrtowc(&wc, p, n, &mb_st)) > 0) {
-	    glyphBBox = GlyphBBox(wc, gc, dd);
-	    resultBBox = CombineBBoxes(resultBBox, glyphBBox);
-	    p += used; n -= used;
-	}
-#else
-	char *s = str;
-	while (*s) {
-	    glyphBBox = GlyphBBox(*s, gc, dd);
-	    resultBBox = CombineBBoxes(resultBBox, glyphBBox);
-	    s++;
-	}
+	if(mbcslocale && gc->fontface != 5) {
+	    int n = strlen(str), used;
+	    wchar_t wc;
+	    const char *p = str;
+	    mbstate_t mb_st;
+	    mbs_init(&mb_st);
+	    while ((used = Mbrtowc(&wc, p, n, &mb_st)) > 0) {
+		glyphBBox = GlyphBBox(wc, gc, dd);
+		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
+		p += used; n -= used;
+	    }
+	} else
 #endif
+	{
+	    const char *s = str;
+	    while (*s) {
+		glyphBBox = GlyphBBox(*s, gc, dd);
+		resultBBox = CombineBBoxes(resultBBox, glyphBBox);
+		s++;
+	    }
+	}
 	if (draw) {
 	    GEText(ConvertedX(mc ,dd), ConvertedY(mc, dd), str,
-		   0.0, 0.0, mc->CurrentAngle, gc,
-		   dd);
+		   0.0, 0.0, mc->CurrentAngle, gc, dd);
 	    PMoveAcross(bboxWidth(resultBBox), mc);
 	}
 	if (UsingItalics(gc))
@@ -1166,7 +1235,7 @@ static BBOX RenderNumber(SEXP expr, int draw, mathContext *mc,
 static BBOX RenderString(SEXP expr, int draw, mathContext *mc,
 			 R_GE_gcontext *gc, GEDevDesc *dd)
 {
-    return RenderStr(CHAR(STRING_ELT(expr, 0)), draw, mc, gc, dd);
+    return RenderStr(translateChar(STRING_ELT(expr, 0)), draw, mc, gc, dd);
 }
 
 /* Code for Ellipsis (ldots, cdots, ...) */
@@ -1275,6 +1344,7 @@ static SymTab BinTable[] = {
     { "%/%",		0270 },
     { "%intersection%", 0307 },
     { "%union%",	0310 },
+    { "%.%",            0327 }, /* cdot or dotmath */
     { NULL,		   0 }
 };
 
@@ -1332,7 +1402,8 @@ static BBOX RenderSlash(int draw, mathContext *mc, R_GE_gcontext *gc,
 	y[1] = ConvertedY(mc, dd);
 	PMoveUp(-height, mc);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(2, x, y, gc, dd);
 	PMoveAcross(0.5 * width, mc);
 	gc->lty = savedlty;
@@ -1591,7 +1662,8 @@ static BBOX RenderWideTilde(SEXP expr, int draw, mathContext *mc,
 	x[NTILDE + 2] = ConvertedX(mc, dd);
 	y[NTILDE + 2] = ConvertedY(mc, dd);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(NTILDE + 3, x, y, gc, dd);
 	PMoveTo(savedX + totalwidth, savedY, mc);
 	gc->lty = savedlty;
@@ -1634,7 +1706,8 @@ static BBOX RenderWideHat(SEXP expr, int draw, mathContext *mc,
 	x[2] = ConvertedX(mc, dd);
 	y[2] = ConvertedY(mc, dd);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(3, x, y, gc, dd);
 	PMoveTo(savedX + width, savedY, mc);
 	gc->lty = savedlty;
@@ -1671,7 +1744,8 @@ static BBOX RenderBar(SEXP expr, int draw, mathContext *mc,
 	x[1] = ConvertedX(mc, dd);
 	y[1] = ConvertedY(mc, dd);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(2, x, y, gc, dd);
 	PMoveTo(savedX + width, savedY, mc);
 	gc->lty = savedlty;
@@ -1787,11 +1861,16 @@ static void NumDenomVShift(BBOX numBBox, BBOX denomBBox,
 	phi = theta;
     }
     delta = (*u - bboxDepth(numBBox)) - (a + 0.5 * theta);
+    /*
+     * Numerators and denominators on fractions appear too far from
+     * horizontal bar. 
+     * Reread of Knuth suggests removing "+ theta" components below.
+     */ 
     if (delta < phi)
-	*u += (phi - delta) + theta;
+	*u += (phi - delta); /* + theta; */
     delta = (a + 0.5 * theta) - (bboxHeight(denomBBox) - *v);
     if (delta < phi)
-	*v += (phi - delta) + theta;
+	*v += (phi - delta); /* + theta; */
 }
 
 static void NumDenomHShift(BBOX numBBox, BBOX denomBBox,
@@ -1862,7 +1941,8 @@ static BBOX RenderFraction(SEXP expr, int rule, int draw,
 	    x[1] = ConvertedX(mc, dd);
 	    y[1] = ConvertedY(mc, dd);
 	    gc->lty = LTY_SOLID;
-	    gc->lwd = 1;
+	    if (gc->lwd > 1)
+		gc->lwd = 1;
 	    GEPolyline(2, x, y, gc, dd);
 	    PMoveUp(-AxisHeight(gc, dd), mc);
 	    gc->lty = savedlty;
@@ -1903,7 +1983,8 @@ static BBOX RenderUnderline(SEXP expr, int draw, mathContext *mc,
         x[1] = ConvertedX(mc, dd);
         y[1] = ConvertedY(mc, dd);
         gc->lty = LTY_SOLID;
-        gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
         GEPolyline(2, x, y, gc, dd);
         PMoveUp(depth, mc);
         gc->lty = savedlty;
@@ -2525,7 +2606,8 @@ static BBOX RenderRadical(SEXP expr, int draw, mathContext *mc,
 	x[4] = ConvertedX(mc, dd);
 	y[4] = ConvertedY(mc, dd);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(5, x, y, gc, dd);
 	PMoveTo(savedX, savedY, mc);
 	gc->lty = savedlty;
@@ -2574,7 +2656,8 @@ static BBOX RenderAbs(SEXP expr, int draw, mathContext *mc,
 	x[1] = ConvertedX(mc, dd);
 	y[1] = ConvertedY(mc, dd);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(2, x, y, gc, dd);
 	PMoveUp(-height, mc);
 	gc->lty = savedlty;
@@ -2594,7 +2677,8 @@ static BBOX RenderAbs(SEXP expr, int draw, mathContext *mc,
 	x[1] = ConvertedX(mc, dd);
 	y[1] = ConvertedY(mc, dd);
 	gc->lty = LTY_SOLID;
-	gc->lwd = 1;
+	if (gc->lwd > 1)
+	    gc->lwd = 1;
 	GEPolyline(2, x, y, gc, dd);
 	PMoveUp(-height, mc);
 	gc->lty = savedlty;
@@ -2764,6 +2848,34 @@ static BBOX RenderPlain(SEXP expr, int draw, mathContext *mc,
     return bbox;
 }
 
+/*----------------------------------------------------------------------
+ *
+ *  Code for SymbolFace (i.e. font = 5) Expressions
+ *
+ *  This makes the default font an Adobe Symbol Encoded font
+ *  (provides access to any character in the Adobe Symbol Font
+ *   encoding via strings like "\042" for the universal ["for all"]
+ *   symbol, without the need for separate special names for each
+ *   of these symbols).
+ *
+ */
+
+static int SymbolFaceAtom(SEXP expr)
+{
+    return NameAtom(expr) &&
+	NameMatch(expr, "symbol");
+}
+
+static BBOX RenderSymbolFace(SEXP expr, int draw, mathContext *mc,
+                             R_GE_gcontext *gc, GEDevDesc *dd)
+{
+    BBOX bbox;
+    int prevfont = SetFont(SymbolFont, gc);
+    bbox = RenderElement(CADR(expr), draw, mc, gc, dd);
+    SetFont(prevfont, gc);
+    return bbox;
+}
+
 /*----------------------------------------------------------------------
  *
  *  Code for Bold Italic Expressions
@@ -3006,6 +3118,8 @@ static BBOX RenderFormula(SEXP expr, int draw, mathContext *mc,
 	return RenderItalic(expr, draw, mc, gc, dd);
     else if (PlainAtom(head))
 	return RenderPlain(expr, draw, mc, gc, dd);
+    else if (SymbolFaceAtom(head))
+	return RenderSymbolFace(expr, draw, mc, gc, dd);
     else if (BoldItalicAtom(head))
 	return RenderBoldItalic(expr, draw, mc, gc, dd);
     else if (StyleAtom(head))
@@ -3199,13 +3313,14 @@ void GEMathText(double x, double y, SEXP expr,
     mc.CosAngle = cos(rot);
     mc.SinAngle = sin(rot);
     RenderElement(expr, 1, &mc, gc, dd);
-}/* GMathText */
+}/* GEMathText */
 
 
 /********************************
  * Code below here ...
  * ... should be moved to base.c and
  * ... is part of the base graphics API NOT the graphics engine API
+ * used in plot.c
  ********************************
  */
 double GExpressionWidth(SEXP expr, GUnit units, DevDesc *dd)
@@ -3232,6 +3347,8 @@ double GExpressionHeight(SEXP expr, GUnit units, DevDesc *dd)
 	return GConvertYUnits(height, DEVICE, units, dd);
 }
 
+/* NOT true: used in plot.c */
+
 /* This is just here to satisfy the Rgraphics.h API.
  * This allows new graphics API (GraphicsDevice.h, GraphicsEngine.h)
  * to be developed alongside.
