@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2005-6   The R Development Core Team
+ *  Copyright (C) 2005-7   The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -12,16 +12,15 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 51 Franklin Street
- *  Fifth Floor, Boston, MA 02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /*  This file was contributed by Ei-ji Nakama.
  *  See also the comments in R_ext/rlocale.h.
  *
- *  It provides replacements for the wctype functions on 
+ *  It provides replacements for the wctype functions on
  *  Windows (where they are not correct in e.g. Japanese)
  *  AIX (missing)
  *  MacOS X in CJK (where these just call the ctype functions)
@@ -36,10 +35,6 @@
 #include <config.h>
 #endif
 
-#ifdef HAVE_GLIBC2
-# define _GNU_SOURCE /* iswblank is a GNU extension, also in C99 */
-#endif
-
 #include <string.h>
 #include <stdlib.h>
 
@@ -63,7 +58,7 @@ void Ri18n_iswctype(void)
 
 #else /* SUPPORT_MBCS */
 
-#define IN_RLOCALE_C 1 /* used in rlocale.h */ 
+#define IN_RLOCALE_C 1 /* used in rlocale.h */
 #include <R_ext/rlocale.h>
 #include "rlocale_data.h"
 
@@ -74,8 +69,8 @@ void Ri18n_iswctype(void)
 #include <limits.h>
 #include <R_ext/Riconv.h>
 
-static int wcwidthsearch(int wint, const struct interval_wcwidth *table, 
-			 int max, int locale) 
+static int wcwidthsearch(int wint, const struct interval_wcwidth *table,
+			 int max, int locale)
 {
     int min = 0;
     int mid;
@@ -132,18 +127,18 @@ static cjk_locale_name_t cjk_locale_name[] = {
 int Ri18n_wcwidth(wchar_t c)
 {
     char lc_str[128];
-    unsigned int i;
+    unsigned int i, j;
 
     static char *lc_cache = "";
     static int lc = 0;
 
     if (0 != strcmp(setlocale(LC_CTYPE, NULL), lc_cache)) {
 	strncpy(lc_str, setlocale(LC_CTYPE, NULL), sizeof(lc_str));
-	for (i = 0; i < strlen(lc_str) && i < sizeof(lc_str); i++)
+	for (i = 0, j = strlen(lc_str); i < j && i < sizeof(lc_str); i++)
 	    lc_str[i] = toupper(lc_str[i]);
-	for (i = 0; i < (sizeof(cjk_locale_name)/sizeof(cjk_locale_name_t)); 
+	for (i = 0; i < (sizeof(cjk_locale_name)/sizeof(cjk_locale_name_t));
 	     i++) {
-	    if (0 == strncmp(cjk_locale_name[i].name, lc_str, 
+	    if (0 == strncmp(cjk_locale_name[i].name, lc_str,
 			     strlen(cjk_locale_name[i].name))) {
 		lc = cjk_locale_name[i].locale;
 		break;
@@ -170,7 +165,7 @@ int Ri18n_wcswidth (const wchar_t *s, size_t n)
 }
 
 #if defined(Win32) || defined(_AIX) || defined(__APPLE_CC__)
-static int wcsearch(int wint, const struct interval *table, int max) 
+static int wcsearch(int wint, const struct interval *table, int max)
 {
     int min = 0;
     int mid;
@@ -191,14 +186,6 @@ static int wcsearch(int wint, const struct interval *table, int max)
 }
 #endif
 
-/* what was this for ?
-#ifdef Win32
-# include <windows.h>
-# include <winnls.h>
-#else
-# include <langinfo.h>
-#endif
-*/
 
 /*********************************************************************
  *  There is MacOS with a CSI(CodeSet Independence) system.
@@ -212,7 +199,6 @@ static const char UNICODE[] = "UCS-4BE";
 #else
 static const char UNICODE[] = "UCS-4LE";
 #endif
-char *locale2charset(const char *locale);
 
 #define ISWFUNC(ISWNAME) static int Ri18n_isw ## ISWNAME (wint_t wc) \
 {	                                                             \
@@ -232,12 +218,12 @@ char *locale2charset(const char *locale);
   memset(mb_buf, 0, sizeof(mb_buf));				     \
   memset(ucs4_buf, 0, sizeof(ucs4_buf));			     \
   wcrtomb( mb_buf, wc, NULL);					     \
-  if((void *)(-1) != (cd = Riconv_open((char*)UNICODE, fromcode))) { \
+  if((void *)(-1) != (cd = Riconv_open(UNICODE, fromcode))) {	     \
       wc_len = sizeof(ucs4_buf);		                     \
       _wc_buf = (char *)ucs4_buf;		       		     \
       mb_len = strlen(mb_buf);					     \
       _mb_buf = (char *)mb_buf;					     \
-      rc = Riconv(cd, (char **)&_mb_buf, (size_t *)&mb_len,	     \
+      rc = Riconv(cd, (const char **)&_mb_buf, (size_t *)&mb_len,	     \
 	          (char **)&_wc_buf, (size_t *)&wc_len);       	     \
       Riconv_close(cd);						     \
       wc = ucs4_buf[0];                                              \
@@ -328,7 +314,7 @@ static const Ri18n_wctype_func_l Ri18n_wctype_func[] = {
     {NULL,     0,     NULL}
 };
 
-wctype_t Ri18n_wctype(const char *name) 
+wctype_t Ri18n_wctype(const char *name)
 {
     int i;
 
