@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2005   The R Development Core Team
+ *  Copyright (C) 1997-2007   The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *
  *  Vector and List Subsetting
@@ -61,6 +61,11 @@ static SEXP ExtractSubset(SEXP x, SEXP result, SEXP indx, SEXP call)
 	    ii--;
 	switch (mode) {
 	case LGLSXP:
+            if (0 <= ii && ii < nx && ii != NA_LOGICAL)
+                LOGICAL(result)[i] = LOGICAL(x)[ii];
+            else
+                LOGICAL(result)[i] = NA_INTEGER;
+            break;
 	case INTSXP:
 	    if (0 <= ii && ii < nx && ii != NA_INTEGER)
 		INTEGER(result)[i] = INTEGER(x)[ii];
@@ -138,7 +143,7 @@ static SEXP VectorSubset(SEXP x, SEXP s, SEXP call)
 
     if (isMatrix(s) && isArray(x) && (isInteger(s) || isReal(s)) &&
 	    ncols(s) == length(attrib)) {
-	s = mat2indsub(attrib, s);
+	s = mat2indsub(attrib, s, call);
 	UNPROTECT(1);
 	PROTECT(s);
     }
@@ -146,7 +151,7 @@ static SEXP VectorSubset(SEXP x, SEXP s, SEXP call)
     /* Convert to a vector of integer subscripts */
     /* in the range 1:length(x). */
 
-    PROTECT(indx = makeSubscript(x, s, &stretch));
+    PROTECT(indx = makeSubscript(x, s, &stretch, call));
     n = LENGTH(indx);
 
     /* Allocate the result. */
@@ -175,6 +180,13 @@ static SEXP VectorSubset(SEXP x, SEXP s, SEXP call)
 	setAttrib(result, R_NamesSymbol, nattrib);
 	UNPROTECT(1);
     }
+    if (result != R_NilValue && (attrib = getAttrib(x, R_SrcrefSymbol)) != R_NilValue) {
+	nattrib = allocVector(VECSXP, n);
+	PROTECT(nattrib); /* seems unneeded */
+	nattrib = ExtractSubset(attrib, nattrib, indx, call);
+	setAttrib(result, R_SrcrefSymbol, nattrib);
+	UNPROTECT(1);
+    }    
     UNPROTECT(3);
     return result;
 }
@@ -241,7 +253,7 @@ static SEXP MatrixSubset(SEXP x, SEXP s, SEXP call, int drop)
 		    RAW(result)[ij] = (Rbyte) 0;
 		    break;
 		default:
-		    error(_("matrix subscripting not handled for this type"));
+		    errorcall(call, _("matrix subscripting not handled for this type"));
 		    break;
 		}
 	    }
@@ -249,6 +261,8 @@ static SEXP MatrixSubset(SEXP x, SEXP s, SEXP call, int drop)
 		iijj = ii + jj * nr;
 		switch (TYPEOF(x)) {
 		case LGLSXP:
+                    LOGICAL(result)[ij] = LOGICAL(x)[iijj];
+                    break;
 		case INTSXP:
 		    INTEGER(result)[ij] = INTEGER(x)[iijj];
 		    break;
@@ -268,7 +282,7 @@ static SEXP MatrixSubset(SEXP x, SEXP s, SEXP call, int drop)
 		    RAW(result)[ij] = RAW(x)[iijj];
 		    break;
 		default:
-		    error(_("matrix subscripting not handled for this type"));
+		    errorcall(call, _("matrix subscripting not handled for this type"));
 		    break;
 		}
 	    }
@@ -327,7 +341,7 @@ static SEXP ArraySubset(SEXP x, SEXP s, SEXP call, int drop)
     int i, j, k, ii, jj, mode, n;
     int **subs, *indx, *offset, *bound;
     SEXP dimnames, dimnamesnames, p, q, r, result, xdims;
-    char *vmaxsave;
+    void *vmaxsave;
 
     mode = TYPEOF(x);
     xdims = getAttrib(x, R_DimSymbol);
@@ -425,7 +439,7 @@ static SEXP ArraySubset(SEXP x, SEXP s, SEXP call, int drop)
 		RAW(result)[i] = (Rbyte) 0;
 	    break;
 	default:
-	    error(_("array subscripting not handled for this type"));
+	    errorcall(call, _("array subscripting not handled for this type"));
 	    break;
 	}
 	if (n > 1) {
@@ -484,7 +498,8 @@ static SEXP ArraySubset(SEXP x, SEXP s, SEXP call, int drop)
 	setAttrib(result, R_DimNamesSymbol, xdims);
 	UNPROTECT(1);
     }
-    copyMostAttrib(x, result);
+    /* This was removed for matrices in 1998
+       copyMostAttrib(x, result); */
     /* Free temporary memory */
     vmaxset(vmaxsave);
     if (drop)
@@ -494,27 +509,61 @@ static SEXP ArraySubset(SEXP x, SEXP s, SEXP call, int drop)
 }
 
 
+/* Returns and removes a named argument from argument list args.
+   The search ends as soon as a matching argument is found.  If
+   the argument is not found, the argument list is not modified
+   and R_NilValue is returned.
+ */
+static SEXP ExtractArg(SEXP args, SEXP arg_sym)
+{
+    SEXP arg, prev_arg;
+    int found = 0;
+
+    for (arg = prev_arg = args; arg != R_NilValue; arg = CDR(arg)) {
+	if(TAG(arg) == arg_sym) {
+            if (arg == prev_arg) /* found at head of args */
+                args = CDR(args);
+            else
+                SETCDR(prev_arg, CDR(arg));
+            found = 1;
+            break;
+	}
+	else  prev_arg = arg;
+    }
+    return found ? CAR(arg) : R_NilValue;
+}
+
 /* Extracts the drop argument, if present, from the argument list.
    The object being subsetted must be the first argument. */
 static void ExtractDropArg(SEXP el, int *drop)
 {
-    SEXP last = el;
-    for (el = CDR(el); el != R_NilValue; el = CDR(el)) {
-	if(TAG(el) == R_DropSymbol) {
-	    *drop = asLogical(CAR(el));
-	    if (*drop == NA_LOGICAL) *drop = 1;
-	    SETCDR(last, CDR(el));
-	}
-	else last = el;
-    }
+    SEXP dropArg = ExtractArg(el, R_DropSymbol);
+    *drop = asLogical(dropArg);
+    if (*drop == NA_LOGICAL) *drop = 1;
 }
 
 
+/* Extracts and, if present, removes the 'exact' argument from the
+   argument list.  An integer code giving the desired exact matching
+   behavior is returned:
+       0  not exact
+       1  exact
+      -1  not exact, but warn when partial matching is used
+ */
+static int ExtractExactArg(SEXP args)
+{
+    SEXP argval = ExtractArg(args, R_ExactSymbol);
+    int exact = -1;
+    exact = asLogical(argval);
+    if (exact == NA_LOGICAL) exact = -1;
+    return exact;
+}
+
 
 /* The "[" subset operator.
  * This provides the most general form of subsetting. */
 
-SEXP do_subset(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_subset(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
 
@@ -532,15 +581,13 @@ SEXP do_subset(SEXP call, SEXP op, SEXP args, SEXP rho)
     return do_subset_dflt(call, op, ans, rho);
 }
 
-SEXP do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, ax, px, x, subs;
     int drop, i, nsubs, type;
 
     /* By default we drop extents of length 1 */
 
-    PROTECT(args);
-
     /* Handle case of extracting a single element from a simple vector
        directly to improve speed for this simple case. */
     if (CDDR(args) == R_NilValue) {
@@ -555,12 +602,8 @@ SEXP do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		case INTSXP: i = (LENGTH(s) == 1) ? INTEGER(s)[0] : -1; break;
 		default:  i = -1;
 		}
-		if (i >= 1 && i <= LENGTH(x)) {
-		    ans = allocVector(REALSXP, 1);
-		    REAL(ans)[0] = REAL(x)[i-1];
-		    UNPROTECT(1);
-		    return ans;
-		}
+		if (i >= 1 && i <= LENGTH(x))
+		    return ScalarReal( REAL(x)[i-1] );
 		break;
 	    case INTSXP:
 		switch (TYPEOF(s)) {
@@ -568,18 +611,16 @@ SEXP do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		case INTSXP: i = (LENGTH(s) == 1) ? INTEGER(s)[0] : -1; break;
 		default:  i = -1;
 		}
-		if (i >= 1 && i <= LENGTH(x)) {
-		    ans = allocVector(INTSXP, 1);
-		    INTEGER(ans)[0] = INTEGER(x)[i-1];
-		    UNPROTECT(1);
-		    return ans;
-		}
+		if (i >= 1 && i <= LENGTH(x))
+		    return ScalarInteger( INTEGER(x)[i-1] );
 		break;
 	    default: break;
 	    }
 	}
     }
 
+    PROTECT(args);
+
     drop = 1;
     ExtractDropArg(args, &drop);
     x = CAR(args);
@@ -675,6 +716,7 @@ SEXP do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	setAttrib(ans, R_DimSymbol, getAttrib(ax, R_DimSymbol));
 	setAttrib(ans, R_DimNamesSymbol, getAttrib(ax, R_DimNamesSymbol));
 	setAttrib(ans, R_NamesSymbol, getAttrib(ax, R_NamesSymbol));
+	SET_NAMED(ans, NAMED(ax)); /* PR#7924 */
     }
     else {
 	PROTECT(ans);
@@ -691,7 +733,7 @@ SEXP do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 /* The [[ subset operator.  It needs to be fast. */
 /* The arguments to this call are evaluated on entry. */
 
-SEXP do_subset2(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_subset2(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
 
@@ -710,14 +752,23 @@ SEXP do_subset2(SEXP call, SEXP op, SEXP args, SEXP rho)
     return do_subset2_dflt(call, op, ans, rho);
 }
 
-SEXP do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
+SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, dims, dimnames, indx, subs, x;
     int i, ndims, nsubs, offset = 0;
-    int drop = 1;
+    int drop = 1, pok, exact = -1;
 
     PROTECT(args);
     ExtractDropArg(args, &drop);
+    /* Is partial matching ok?  When the exact arg is NA, a warning is
+       issued if partial matching occurs.
+     */
+    exact = ExtractExactArg(args);
+    if (exact == -1)
+        pok = exact;
+    else
+        pok = !exact;
+
     x = CAR(args);
 
     /* This code was intended for compatibility with S, */
@@ -742,14 +793,15 @@ SEXP do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* split out ENVSXP for now */
     if( TYPEOF(x) == ENVSXP ) {
       if( nsubs != 1 || !isString(CAR(subs)) || length(CAR(subs)) != 1 )
-	error(_("wrong arguments for subsetting an environment"));
-      ans = findVarInFrame(x, install(CHAR(STRING_ELT(CAR(subs),
-						      0))));
-        if( TYPEOF(ans) == PROMSXP ) {
+	errorcall(call, _("wrong arguments for subsetting an environment"));
+      ans = findVarInFrame(x, install(translateChar(STRING_ELT(CAR(subs), 0))));
+      if( TYPEOF(ans) == PROMSXP ) {
 	    PROTECT(ans);
 	    ans = eval(ans, R_GlobalEnv);
 	    UNPROTECT(1);
-      	}   	
+      } else {
+	    SET_NAMED(ans, 2);
+      }
       
       UNPROTECT(1);
       if(ans == R_UnboundValue )
@@ -769,19 +821,18 @@ SEXP do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if(isVectorList(x) && length(CAR(subs)) > 1) {
 	    for(i = 0; i < len - 1; i++) {
 		if(!isVectorList(x))
-		    error(_("recursive indexing failed at level %d\n"), i+1);
+		    errorcall(call, _("recursive indexing failed at level %d\n"), i+1);
 		offset = get1index(CAR(subs), getAttrib(x, R_NamesSymbol),
-				   length(x), /*partial ok*/TRUE, i);
+				   length(x), pok, i, call);
 		if(offset < 0 || offset >= length(x))
-		    error(_("no such index at level %d\n"), i+1);
+		    errorcall(call, _("no such index at level %d\n"), i+1);
 		x = VECTOR_ELT(x, offset);
 	    }
 	}
 	offset = get1index(CAR(subs), getAttrib(x, R_NamesSymbol),
-			   length(x), /*partial ok*/TRUE, i);
+			   length(x), pok, i, call);
 	if (offset < 0 || offset >= length(x)) {
-	    /* a bold attempt to get the same */
-	    /* behaviour for $ and [[ */
+	    /* a bold attempt to get the same behaviour for $ and [[ */
 	    if (offset < 0 && (isNewList(x) ||
 			       isExpression(x) ||
 			       isList(x) ||
@@ -806,7 +857,7 @@ SEXP do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    INTEGER(indx)[i] =
 		get1index(CAR(subs), (i < ndn) ? VECTOR_ELT(dimnames, i) :
 			  R_NilValue,
-			  INTEGER(indx)[i], /*partial ok*/TRUE, -1);
+			  INTEGER(indx)[i], pok, -1, call);
 	    subs = CDR(subs);
 	    if (INTEGER(indx)[i] < 0 ||
 		INTEGER(indx)[i] >= INTEGER(dims)[i])
@@ -824,7 +875,10 @@ SEXP do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (NAMED(x) > NAMED(ans))
 	    SET_NAMED(ans, NAMED(x));
     } else if(isVectorList(x)) {
-	ans = duplicate(VECTOR_ELT(x, offset));
+	/* did unconditional duplication before 2.4.0 */
+	ans = VECTOR_ELT(x, offset);
+	if (NAMED(x) > NAMED(ans))
+	    SET_NAMED(ans, NAMED(x));
     } else {
 	ans = allocVector(TYPEOF(x), 1);
 	switch (TYPEOF(x)) {
@@ -853,17 +907,19 @@ SEXP do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
-/* A helper to partially match tags against a candidate. */
-/* Returns: */
-static
 enum pmatch {
     NO_MATCH,
     EXACT_MATCH,
     PARTIAL_MATCH
-}
+};
+
+/* A helper to partially match tags against a candidate. */
+/* Returns: */
+static
+enum pmatch
 pstrmatch(SEXP target, SEXP input, int slen)
 {
-    char *st="";
+    const char *st = "";
 
     if(target == R_NilValue)
 	return NO_MATCH;
@@ -873,15 +929,11 @@ pstrmatch(SEXP target, SEXP input, int slen)
 	st = CHAR(PRINTNAME(target));
 	break;
     case CHARSXP:
-	st = CHAR(target);
+	st = translateChar(target);
 	break;
     }
-    if(strncmp(st, CHAR(input), slen) == 0) {
-	if (strlen(st) == slen)
-	    return EXACT_MATCH;
-	else
-	    return PARTIAL_MATCH;
-    }
+    if(strncmp(st, translateChar(input), slen) == 0)
+	return (strlen(st) == slen) ?  EXACT_MATCH : PARTIAL_MATCH;
     else return NO_MATCH;
 }
 
@@ -890,7 +942,7 @@ pstrmatch(SEXP target, SEXP input, int slen)
    We need to be sure to only evaluate the first argument.
    The second will be a symbol that needs to be matched, not evaluated.
 */
-SEXP do_subset3(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_subset3(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP input, nlist, ans;
 
@@ -906,11 +958,16 @@ SEXP do_subset3(SEXP call, SEXP op, SEXP args, SEXP env)
     else if(isString(nlist) )
 	SET_STRING_ELT(input, 0, STRING_ELT(nlist, 0));
     else {
-	errorcall_return(call, _("invalid subscript type"));
+	errorcall(call,_("invalid subscript type '%s'"), 
+		  type2char(TYPEOF(nlist)));
     }
 
     /* replace the second argument with a string */
-    SETCADR(args, input);
+
+    /* Previously this was SETCADR(args, input); */
+    /* which could cause problems when nlist was */
+    /* ..., as in PR#8718 */
+    PROTECT(args = CONS(CAR(args), CONS(input, R_NilValue)));
 
     /* If the first argument is an object and there is */
     /* an approriate method, we dispatch to that method, */
@@ -919,15 +976,16 @@ SEXP do_subset3(SEXP call, SEXP op, SEXP args, SEXP env)
     /* evaluation retains any missing argument indicators. */
 
     if(DispatchOrEval(call, op, "$", args, env, &ans, 0, 0)) {
-	UNPROTECT(1);
+	UNPROTECT(2);
 	return(ans);
     }
 
-    UNPROTECT(1);
-    return R_subset3_dflt(CAR(ans), STRING_ELT(input, 0));
+    UNPROTECT(2);
+    return R_subset3_dflt(CAR(ans), STRING_ELT(input, 0), call);
 }
 
-SEXP R_subset3_dflt(SEXP x, SEXP input)
+/* used in eval.c */
+SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
 {
     SEXP y, nlist;
     int slen;
@@ -936,13 +994,13 @@ SEXP R_subset3_dflt(SEXP x, SEXP input)
     PROTECT(input);
 
     /* Optimisation to prevent repeated recalculation */
-    slen = strlen(CHAR(input));
+    slen = strlen(translateChar(input));
 
     /* If this is not a list object we return NULL. */
     /* Or should this be allocVector(VECSXP, 0)? */
 
     if (isPairList(x)) {
-	SEXP xmatch=R_NilValue;
+	SEXP xmatch = R_NilValue;
 	int havematch;
 	UNPROTECT(2);
 	havematch = 0;
@@ -950,8 +1008,7 @@ SEXP R_subset3_dflt(SEXP x, SEXP input)
 	    switch(pstrmatch(TAG(y), input, slen)) {
 	    case EXACT_MATCH:
 		y = CAR(y);
-		if (NAMED(x) > NAMED(y))
-		    SET_NAMED(y, NAMED(x));
+		if (NAMED(x) > NAMED(y)) SET_NAMED(y, NAMED(x));
 		return y;
 	    case PARTIAL_MATCH:
 		havematch++;
@@ -961,10 +1018,23 @@ SEXP R_subset3_dflt(SEXP x, SEXP input)
 		break;
 	    }
 	}
-	if (havematch == 1) {
+	if (havematch == 1) { /* unique partial match */
+	    if(R_warn_partial_match_dollar) {
+		const char *st = "";
+		SEXP target = TAG(y);
+		switch (TYPEOF(target)) {
+		case SYMSXP:
+		    st = CHAR(PRINTNAME(target));
+		    break;
+		case CHARSXP:
+		    st = translateChar(target);
+		    break;
+		}
+		warningcall(call, _("partial match of '%s' to '%s'"),
+			    translateChar(input), st);
+	    }
 	    y = CAR(xmatch);
-	    if (NAMED(x) > NAMED(y))
-		SET_NAMED(y, NAMED(x));
+	    if (NAMED(x) > NAMED(y)) SET_NAMED(y, NAMED(x));
 	    return y;
 	}
 	return R_NilValue;
@@ -984,22 +1054,43 @@ SEXP R_subset3_dflt(SEXP x, SEXP input)
 		return y;
 	    case PARTIAL_MATCH:
 		havematch++;
+		if (havematch == 1) {
+		    /* partial matches can cause aliasing in eval.c:evalseq 
+                       This is overkill, but alternative ways to prevent 
+                       the aliasing appear to be even worse */
+		    y = VECTOR_ELT(x,i);
+		    SET_NAMED(y,2);
+		    SET_VECTOR_ELT(x,i,y);
+		}
 		imatch = i;
 		break;
 	    case NO_MATCH:
 		break;
 	    }
 	}
-	if(havematch ==1) {
+	if(havematch == 1) { /* unique partial match */
+	    if(R_warn_partial_match_dollar) {
+		const char *st = "";
+		SEXP target = STRING_ELT(nlist, imatch);
+		switch (TYPEOF(target)) {
+		case SYMSXP:
+		    st = CHAR(PRINTNAME(target));
+		    break;
+		case CHARSXP:
+		    st = translateChar(target);
+		    break;
+		}
+		warningcall(call, _("partial match of '%s' to '%s'"),
+			    translateChar(input), st);
+	    }
 	    y = VECTOR_ELT(x, imatch);
-	    if (NAMED(x) > NAMED(y))
-		SET_NAMED(y, NAMED(x));
+	    if (NAMED(x) > NAMED(y)) SET_NAMED(y, NAMED(x));
 	    return y;
 	}
 	return R_NilValue;
     }
     else if( isEnvironment(x) ){
-      	y = findVarInFrame(x, install(CHAR(input)));
+      	y = findVarInFrame(x, install(translateChar(input)));
       	if( TYPEOF(y) == PROMSXP ) {
 	    PROTECT(y);
 	    y = eval(y, R_GlobalEnv);
@@ -1013,6 +1104,12 @@ SEXP R_subset3_dflt(SEXP x, SEXP input)
 	}
       return R_NilValue;
     }
+    else if( isVectorAtomic(x) ){
+        warningcall(call, "$ operator is invalid for atomic vectors, returning NULL");
+    }
+    else if( IS_S4_OBJECT(x) ){
+        warningcall(call, "$ operator not defined for this S4 class, returning NULL");
+    }
     UNPROTECT(2);
     return R_NilValue;
 }
