@@ -15,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8>
@@ -31,9 +31,10 @@
 #include <config.h>
 #endif
 
-#define COMPILING_R 1 /* for Rinlinedfuns.h included via Defn.h */
 #include <Defn.h>
-#include <Rmath.h>
+#undef COMPILING_R
+
+#define imax2(x, y) ((x < y) ? y : x)
 #include <Print.h>
 
 #ifdef HAVE_UNISTD_H
@@ -42,9 +43,7 @@
 
 #ifdef __cplusplus
 #include "Clinkage.h"
-#endif
 
-#ifdef __cplusplus
 extern "C" {
 #endif
 void F77_SYMBOL(rwarnc)(char *msg, int *nchar);
@@ -54,7 +53,7 @@ void F77_SYMBOL(rexitc)(char *msg, int *nchar);
 }
 #endif
 
-/* Many small functions are included from Rinlinedfuns.h */
+/* Many small functions are included from ../include/Rinlinedfuns.h */
 
 Rboolean tsConform(SEXP x, SEXP y)
 {
@@ -71,14 +70,6 @@ Rboolean tsConform(SEXP x, SEXP y)
     return FALSE;
 }
 
-const static char * const truenames[] = {
-    "T",
-    "True",
-    "TRUE",
-    "true",
-    (char *) 0,
-};
-
 int nrows(SEXP s)
 {
     SEXP t;
@@ -125,12 +116,20 @@ void attribute_hidden internalTypeCheck(SEXP call, SEXP s, SEXPTYPE type)
     }
 }
 
+const static char * const truenames[] = {
+    "T",
+    "True",
+    "TRUE",
+    "true",
+    (char *) NULL,
+};
+
 const static char * const falsenames[] = {
     "F",
     "False",
     "FALSE",
     "false",
-    (char *) 0,
+    (char *) NULL,
 };
 
 SEXP asChar(SEXP x)
@@ -165,13 +164,27 @@ SEXP asChar(SEXP x)
 	    default:
 		return NA_STRING;
 	    }
-	}
-	else if(TYPEOF(x) == SYMSXP)
+	} else if(TYPEOF(x) == CHARSXP) {
+	    return x;
+	} else if(TYPEOF(x) == SYMSXP)
 	    return PRINTNAME(x);
     }
     return NA_STRING;
 }
 
+Rboolean isUnordered(SEXP s)
+{
+    return (TYPEOF(s) == INTSXP
+	    && inherits(s, "factor")
+	    && !inherits(s, "ordered"));
+}
+
+Rboolean isOrdered(SEXP s)
+{
+    return (TYPEOF(s) == INTSXP
+	    && inherits(s, "factor")
+	    && inherits(s, "ordered"));
+}
 
 
 const static struct {
@@ -209,11 +222,11 @@ TypeTable[] = {
     { "numeric",	REALSXP	   },
     { "name",		SYMSXP	   },
 
-    { (char *)0,	-1	   }
+    { (char *)NULL,	-1	   }
 };
 
 
-SEXPTYPE str2type(char *s)
+SEXPTYPE str2type(const char *s)
 {
     int i;
     for (i = 0; TypeTable[i].str; i++) {
@@ -237,13 +250,13 @@ SEXP type2str(SEXPTYPE t)
     return R_NilValue; /* for -Wall */
 }
 
-char * type2char(SEXPTYPE t)
+const char *type2char(SEXPTYPE t)
 {
     int i;
 
     for (i = 0; TypeTable[i].str; i++) {
 	if (TypeTable[i].type == t)
-	    return (char *) TypeTable[i].str;
+	    return TypeTable[i].str;
     }
     error(_("type %d is unimplemented in '%s'"), t, "type2char");
     return ""; /* for -Wall */
@@ -257,13 +270,13 @@ SEXP type2symbol(SEXPTYPE t)
        character string and to the symbol would be better */
     for (i = 0; TypeTable[i].str; i++) {
 	if (TypeTable[i].type == t)
-	    return install((char *)&TypeTable[i].str);
+	    return install((char *) &TypeTable[i].str);
     }
     error(_("type %d is unimplemented in '%s'"), t, "type2symbol");
     return R_NilValue; /* for -Wall */
 }
 
-void UNIMPLEMENTED_TYPEt(char *s, SEXPTYPE t)
+void UNIMPLEMENTED_TYPEt(const char *s, SEXPTYPE t)
 {
     int i;
 
@@ -274,7 +287,7 @@ void UNIMPLEMENTED_TYPEt(char *s, SEXPTYPE t)
     error(_("unimplemented type (%d) in '%s'\n"), t, s);
 }
 
-void UNIMPLEMENTED_TYPE(char *s, SEXP x)
+void UNIMPLEMENTED_TYPE(const char *s, SEXP x)
 {
     UNIMPLEMENTED_TYPEt(s, TYPEOF(x));
 }
@@ -300,25 +313,25 @@ static const char UCS2ENC[] = "UCS-2LE";
 /* Note: this does not terminate out, as all current uses are to look
  * at 'out' a wchar at a time, and sometimes just one char.
  */
-size_t mbcsToUcs2(char *in, ucs2_t *out, int nout)
+size_t mbcsToUcs2(const char *in, ucs2_t *out, int nout)
 {
     void   *cd = NULL ;
-    char   *i_buf, *o_buf;
+    const char *i_buf;
+    char *o_buf;
     size_t  i_len, o_len, status, wc_len;
 
     /* out length */
     wc_len = mbstowcs(NULL, in, 0);
     if (out == NULL || (int)wc_len < 0) return wc_len;
 
-    if ((void*)-1 == (cd = Riconv_open((char *)UCS2ENC, "")))
+    if ((void*)-1 == (cd = Riconv_open(UCS2ENC, "")))
 	return (size_t) -1;
 
-    i_buf = in;
+    i_buf = (char *)in;
     i_len = strlen(in); /* not including terminator */
     o_buf = (char *)out;
     o_len = nout * sizeof(ucs2_t);
-    status = Riconv(cd, (char **)&i_buf, (size_t *)&i_len,
-		    (char **)&o_buf, (size_t *)&o_len);
+    status = Riconv(cd, &i_buf, (size_t *)&i_len, &o_buf, (size_t *)&o_len);
 
     Riconv_close(cd);
     if (status == (size_t)-1) {
@@ -344,7 +357,7 @@ size_t mbcsToUcs2(char *in, ucs2_t *out, int nout)
 #endif
 
 /* This one is not in Rinternals.h, but is used in internet module */
-Rboolean isBlankString(char *s)
+Rboolean isBlankString(const char *s)
 {
 #ifdef SUPPORT_MBCS
     if(mbcslocale) {
@@ -369,7 +382,7 @@ Rboolean StringBlank(SEXP x)
 
 /* Function to test whether a string is a true value */
 
-Rboolean StringTrue(char *name)
+Rboolean StringTrue(const char *name)
 {
     int i;
     for (i = 0; truenames[i]; i++)
@@ -378,7 +391,7 @@ Rboolean StringTrue(char *name)
     return FALSE;
 }
 
-Rboolean StringFalse(char *name)
+Rboolean StringFalse(const char *name)
 {
     int i;
     for (i = 0; falsenames[i]; i++)
@@ -408,16 +421,22 @@ SEXP attribute_hidden EnsureString(SEXP s)
 }
 
 /* used in modules */
-void checkArity(SEXP op, SEXP args)
+void Rf_checkArityCall(SEXP op, SEXP args, SEXP call)
 {
-    if (PRIMARITY(op) >= 0 && PRIMARITY(op) != length(args))
-	error(P_("%d argument passed to '%s' which requires %d",
-		 "%d arguments passed to '%s' which requires %d",
-		 length(args)),
-	      length(args), PRIMNAME(op), PRIMARITY(op));
+    if (PRIMARITY(op) >= 0 && PRIMARITY(op) != length(args)) {
+	if (PRIMINTERNAL(op))
+	    error(P_("%d argument passed to .Internal(%s) which requires %d",
+		     "%d arguments passed to .Internal(%s) which requires %d",
+		     length(args)),
+		  length(args), PRIMNAME(op), PRIMARITY(op));
+	else
+	    errorcall(call, P_("%d argument passed to '%s' which requires %d",
+			       "%d arguments passed to '%s' which requires %d",
+			       length(args)),
+		      length(args), PRIMNAME(op), PRIMARITY(op));
+    }
 }
 
-
 SEXP nthcdr(SEXP s, int n)
 {
     if (isList(s) || isLanguage(s) || isFrame(s) || TYPEOF(s) == DOTSXP ) {
@@ -435,7 +454,6 @@ SEXP nthcdr(SEXP s, int n)
 
 SEXP attribute_hidden do_nargs(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP t;
     RCNTXT *cptr;
     int nargs = NA_INTEGER;
     for (cptr = R_GlobalContext; cptr != NULL; cptr = cptr->nextcontext) {
@@ -444,9 +462,7 @@ SEXP attribute_hidden do_nargs(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    break;
 	}
     }
-    t = allocVector(INTSXP, 1);
-    *INTEGER(t) = nargs;
-    return (t);
+    return ScalarInteger(nargs);
 }
 
 
@@ -538,7 +554,6 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
     int nx = 0, ny = 0, i, j, k, nans = 0, nx_lone = 0, ny_lone = 0;
     int all_x = 0, all_y = 0, ll = 0/* "= 0" : for -Wall */;
     int *ix, *iy, tmp, nnx, nny, i0, j0;
-    char *vmax = vmaxget();
 
     checkArity(op, args);
     xi = CAR(args);
@@ -548,9 +563,9 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
     if ( !isInteger(yi) || !(ny = LENGTH(yi)) )
 	error(_("invalid '%s' argument"), "yinds");
     if(!LENGTH(ans = CADDR(args)) || NA_LOGICAL == (all_x = asLogical(ans)))
-	errorcall(call, _("'all.x' must be TRUE or FALSE"));
+	error(_("'all.x' must be TRUE or FALSE"));
     if(!LENGTH(ans = CADDDR(args))|| NA_LOGICAL == (all_y = asLogical(ans)))
-	errorcall(call, _("'all.y' must be TRUE or FALSE"));
+	error(_("'all.y' must be TRUE or FALSE"));
 
     /* 0. sort the indices */
     ix = (int *) R_alloc(nx, sizeof(int));
@@ -606,7 +621,6 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    }
     }
 
-    vmaxset(vmax);
     PROTECT(ansnames = allocVector(STRSXP, 4));
     SET_STRING_ELT(ansnames, 0, mkChar("xi"));
     SET_STRING_ELT(ansnames, 1, mkChar("yi"));
@@ -620,7 +634,8 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 /* Functions for getting and setting the working directory. */
 #ifdef Win32
-#include <windows.h>
+# define WIN32_LEAN_AND_MEAN 1
+# include <windows.h>
 #endif
 
 SEXP static intern_getwd()
@@ -650,7 +665,7 @@ SEXP attribute_hidden do_getwd(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 
 #if defined(Win32) && defined(_MSC_VER)
-#include <direct.h> /* for chdir */
+# include <direct.h> /* for chdir, via io.h */
 #endif
 
 SEXP attribute_hidden do_setwd(SEXP call, SEXP op, SEXP args, SEXP rho)
@@ -660,7 +675,7 @@ SEXP attribute_hidden do_setwd(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     checkArity(op, args);
     if (!isPairList(args) || !isValidString(s = CAR(args)))
-	errorcall(call, _("character argument expected"));
+	error(_("character argument expected"));
 
     /* get current directory to return */
     wd = intern_getwd();
@@ -669,7 +684,7 @@ SEXP attribute_hidden do_setwd(SEXP call, SEXP op, SEXP args, SEXP rho)
 #ifdef HAVE_CHDIR
     if(chdir(path) < 0)
 #endif
-	errorcall(call, _("cannot change working directory"));
+	error(_("cannot change working directory"));
     return(wd);
 }
 
@@ -679,17 +694,18 @@ SEXP attribute_hidden do_basename(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, s = R_NilValue;	/* -Wall */
     char  buf[PATH_MAX], *p, fsp = FILESEP[0];
+    const char *pp;
     int i, n;
 
     checkArity(op, args);
     if (TYPEOF(s = CAR(args)) != STRSXP)
-	errorcall(call, _("a character vector argument expected"));
+	error(_("a character vector argument expected"));
     PROTECT(ans = allocVector(STRSXP, n = LENGTH(s)));
     for(i = 0; i < n; i++) {
-	p = R_ExpandFileName(translateChar(STRING_ELT(s, i)));
-	if (strlen(p) > PATH_MAX - 1)
-	    errorcall(call, _("path too long"));
-	strcpy (buf, p);
+	pp = R_ExpandFileName(translateChar(STRING_ELT(s, i)));
+	if (strlen(pp) > PATH_MAX - 1)
+	    error(_("path too long"));
+	strcpy (buf, pp);
 #ifdef Win32
 	R_fixslash(buf);
 #endif
@@ -716,17 +732,18 @@ SEXP attribute_hidden do_dirname(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, s = R_NilValue;	/* -Wall */
     char buf[PATH_MAX], *p, fsp = FILESEP[0];
+    const char *pp;
     int i, n;
 
     checkArity(op, args);
     if (TYPEOF(s = CAR(args)) != STRSXP)
-	errorcall(call, _("a character vector argument expected"));
+	error(_("a character vector argument expected"));
     PROTECT(ans = allocVector(STRSXP, n = LENGTH(s)));
     for(i = 0; i < n; i++) {
-	p = R_ExpandFileName(translateChar(STRING_ELT(s, i)));
-	if (strlen(p) > PATH_MAX - 1)
-	    errorcall(call, _("path too long"));
-	strcpy (buf, p);
+	pp = R_ExpandFileName(translateChar(STRING_ELT(s, i)));
+	if (strlen(pp) > PATH_MAX - 1)
+	    error(_("path too long"));
+	strcpy (buf, pp);
 #ifdef Win32
 	R_fixslash(buf);
 #endif
@@ -760,33 +777,32 @@ SEXP attribute_hidden do_encodeString(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, s;
     int i, len, w, quote = 0, justify, na;
-    char *cs;
+    const char *cs;
     Rboolean findWidth;
 
     checkArity(op, args);
     if (TYPEOF(x = CAR(args)) != STRSXP)
-	errorcall(call, _("a character vector argument expected"));
+	error(_("a character vector argument expected"));
     if(isNull(CADR(args))) w = NA_INTEGER;
     else {
 	w = asInteger(CADR(args));
 	if(w != NA_INTEGER && w < 0)
-	    errorcall(call, _("invalid '%s' value"), "width");
+	    error(_("invalid '%s' value"), "width");
     }
     findWidth = (w == NA_INTEGER);
     s = CADDR(args);
     if(LENGTH(s) != 1 || TYPEOF(s) != STRSXP)
-	errorcall(call, _("invalid '%s' value"), "quote");
+	error(_("invalid '%s' value"), "quote");
     cs = translateChar(STRING_ELT(s, 0));
     if(strlen(cs) > 0) quote = cs[0];
     if(strlen(cs) > 1)
-	warningcall(call,
-		    _("only the first character of 'quote' will be used"));
+	warning(_("only the first character of 'quote' will be used"));
     justify = asInteger(CADDDR(args));
     if(justify == NA_INTEGER || justify < 0 || justify > 3)
-	errorcall(call, _("invalid '%s' value"), "justify");
+	error(_("invalid '%s' value"), "justify");
     if(justify == 3) w = 0;
     na = asLogical(CAD4R(args));
-    if(na == NA_LOGICAL) errorcall(call, _("invalid '%s' value"), "na.encode");
+    if(na == NA_LOGICAL) error(_("invalid '%s' value"), "na.encode");
 
     len = LENGTH(x);
     if(findWidth && justify < 3) {
@@ -816,7 +832,7 @@ SEXP attribute_hidden do_encoding(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     checkArity(op, args);
     if (TYPEOF(x = CAR(args)) != STRSXP)
-	errorcall(call, _("a character vector argument expected"));
+	error(_("a character vector argument expected"));
     n = LENGTH(x);
     PROTECT(ans = allocVector(STRSXP, n));
     for (i = 0; i < n; i++) {
@@ -833,47 +849,49 @@ SEXP attribute_hidden do_setencoding(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP x, enc, tmp;
     int i, m, n;
-    char *this;
+    const char *this;
 
     checkArity(op, args);
     if (TYPEOF(x = CAR(args)) != STRSXP)
-	errorcall(call, _("a character vector argument expected"));
+	error(_("a character vector argument expected"));
     if (TYPEOF(enc = CADR(args)) != STRSXP)
-	errorcall(call, _("a character vector argument expected"));
+	error(_("a character vector 'value' expected"));
     m = LENGTH(enc);
     if(m == 0)
-	errorcall(call, _("'value must be of positive length"));
+	error(_("'value' must be of positive length"));
     if(NAMED(x)) x = duplicate(x);
     PROTECT(x);
     n = LENGTH(x);
     for(i = 0; i < n; i++) {
-	tmp = STRING_ELT(x, i);
-	UNSET_LATIN1(tmp);
-	UNSET_UTF8(tmp);
+	int ienc = 0;
 	this = CHAR(STRING_ELT(enc, i % m)); /* ASCII */
-	if(streql(this, "latin1")) SET_LATIN1(tmp);
-	else if(streql(this, "UTF-8")) SET_UTF8(tmp);
-	SET_STRING_ELT(x, i, tmp);
+	if(streql(this, "latin1")) ienc = LATIN1_MASK;
+	else if(streql(this, "UTF-8")) ienc = UTF8_MASK;
+	tmp = STRING_ELT(x, i);
+	if (! ((ienc == LATIN1_MASK && IS_LATIN1(tmp)) ||
+	       (ienc == UTF8_MASK && IS_UTF8(tmp)) ||
+	       (ienc == 0 && ! IS_LATIN1(tmp) && ! IS_UTF8(tmp))))
+	    SET_STRING_ELT(x, i, mkCharEnc(CHAR(tmp), ienc));
     }
     UNPROTECT(1);
     return x;
 }
 
-void attribute_hidden markKnown(SEXP x, SEXP ref)
+SEXP attribute_hidden markKnown(const char *s, SEXP ref)
 {
-    if(TYPEOF(x) != CHARSXP)
-	error("invalid use of 'markKnown'");
+    int ienc = 0;
     if(IS_LATIN1(ref) || IS_UTF8(ref)) {
-	if(known_to_be_latin1) SET_LATIN1(x);
-	if(known_to_be_utf8) SET_UTF8(x);
+	if(known_to_be_latin1) ienc = LATIN1_MASK;
+	if(known_to_be_utf8) ienc = UTF8_MASK;
     }
+    return mkCharEnc(s, ienc);
 }
 
 /* Note: this is designed to be fast and valid only for UTF-8 strings.
    It is also correct in EUC-* locales. */
-Rboolean utf8strIsASCII(char *str)
+Rboolean utf8strIsASCII(const char *str)
 {
-    char *p;
+    const char *p;
     for(p = str; *p; p++)
 	if((unsigned int)*p > 0x7F) return FALSE;
     return TRUE;
@@ -901,24 +919,24 @@ size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, mbstate_t *ps)
 
     if(n <= 0 || !*s) return (size_t)0;
     used = mbrtowc(wc, s, n, ps);
-    if((int)used < 0) error(_("invalid multibyte string"));
+    if((int) used < 0) error(_("invalid multibyte string at '%s'"), s);
     return used;
 }
 
-Rboolean mbcsValid(char *str)
+Rboolean mbcsValid(const char *str)
 {
     return  ((int)mbstowcs(NULL, str, 0) >= 0);
 }
 
 /* We do this conversion ourselves to do our own error recovery */
-void mbcsToLatin1(char *in, char *out)
+void mbcsToLatin1(const char *in, char *out)
 {
     wchar_t *wbuff;
     int i;
     size_t res = mbstowcs(NULL, in, 0), mres;
 
     if(res == (size_t)(-1)) {
-	warning(_("invalid input in mbcsToLatin1"));
+	warning(_("invalid input '%s' in mbcsToLatin1"), in);
 	*out = '\0';
 	return;
     }
@@ -926,8 +944,8 @@ void mbcsToLatin1(char *in, char *out)
     R_CheckStack();
     if(!wbuff) error(_("allocation failure in 'mbcsToLatin1'"));
     mres = mbstowcs(wbuff, in, res+1);
-    if(mres == (size_t)-1)
-	error(_("invalid input in 'mbcsToLatin1'"));
+    if(mres == (size_t)-1) /* we checked above, so should not get here */
+	error("invalid input in 'mbcsToLatin1'");
     for(i = 0; i < res; i++) {
 	/* here we do assume Unicode wchars */
 	if(wbuff[i] > 0xFF) out[i] = '.';
@@ -971,7 +989,7 @@ char *Rf_strrchr(const char *s, int c)
 int utf8clen(char c) { return 1;}
 size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, void *ps)
 { return (size_t)(-1);}
-Rboolean mbcsValid(char *str) { return TRUE; }
+Rboolean mbcsValid(const char *str) { return TRUE; }
 void mbcsToLatin1(char *in, char *out) {}
 #undef Rf_strchr
 char *Rf_strchr(const char *s, int c)
@@ -1033,7 +1051,7 @@ void F77_SYMBOL(rexitc)(char *msg, int *nchar)
     }
     strncpy(buf, msg, nc);
     buf[nc] = '\0';
-    error(buf);
+    error("%s", buf);
 }
 
 void F77_SYMBOL(rwarnc)(char *msg, int *nchar)
@@ -1046,7 +1064,7 @@ void F77_SYMBOL(rwarnc)(char *msg, int *nchar)
     }
     strncpy(buf, msg, nc);
     buf[nc] = '\0';
-    warning(buf);
+    warning("%s", buf);
 }
 
 void F77_SYMBOL(rchkusr)(void)
@@ -1054,3 +1072,15 @@ void F77_SYMBOL(rchkusr)(void)
     R_CheckUserInterrupt();
 }
 
+/* Return a copy of a string using memory from R_alloc */
+char *acopy_string(const char *in)
+{
+    char *out;
+    int len = strlen(in);
+    if (len > 0) {
+        out = (char *) R_alloc(1+strlen(in), sizeof(char));
+        strcpy(out, in);
+    } else
+        out = "";
+    return out;
+}
