@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2001  The R Development Core Team
+ *  Copyright (C) 1998--2007  The R Development Core Team
  *  based on code (C) 1979 and later Royal Statistical Society
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
 
  * Reference:
@@ -32,12 +32,7 @@
 #include "dpq.h"
 
 /* set the exponent of accu to -2r-2 for r digits of accuracy */
-#ifdef OLD
-#define acu 1.0e-32
-#define lower 0.0001
-#define upper 0.9999
-
-#else/*---- NEW ---- -- still fails for p = 1e11, q=.5*/
+/*---- NEW ---- -- still fails for p = 1e11, q=.5*/
 
 #define fpu 3e-308
 /* acu_min:  Minimal value for accuracy 'acu' which will depend on (a,p);
@@ -46,14 +41,11 @@
 #define lower fpu
 #define upper 1-2.22e-16
 
-#endif
-
 #define const1 2.30753
 #define const2 0.27061
 #define const3 0.99229
 #define const4 0.04481
 
-static volatile double xtrunc;/* not a real global .. delicate though! */
 
 double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 {
@@ -62,25 +54,21 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
     double acu;
     volatile double xinbta;
 
-    /* define accuracy and initialize */
-
-    xinbta = alpha;
-
     /* test for admissibility of parameters */
 
 #ifdef IEEE_754
     if (ISNAN(p) || ISNAN(q) || ISNAN(alpha))
 	return p + q + alpha;
 #endif
-    R_Q_P01_check(alpha);
-
     if(p < 0. || q < 0.) ML_ERR_return_NAN;
 
+    R_Q_P01_boundaries(alpha, 0, 1);
+
     p_ = R_DT_qIv(alpha);/* lower_tail prob (in any case) */
 
-    if (p_ == 0. || p_ == 1.)
-	return p_;
+    /* initialize */
 
+    xinbta = alpha;
     logbeta = lbeta(p, q);
 
     /* change tail if necessary;  afterwards   0 < a <= 1/2	 */
@@ -93,7 +81,7 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 
     /* calculate the initial approximation */
 
-    r = sqrt(-log(a * a));
+    r = sqrt(-2 * log(a));
     y = r - (const1 + const2 * r) / (1. + (const3 + const4 * r) * r);
     if (pp > 1 && qq > 1) {
 	r = (y * y - 3.) / 6.;
@@ -107,7 +95,7 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 	t = 1. / (9. * qq);
 	t = r * pow(1. - t + y * sqrt(t), 3.0);
 	if (t <= 0.)
-	    xinbta = 1. - exp((log((1. - a) * qq) + logbeta) / qq);
+	    xinbta = 1. - exp((log1p(-a)+ log(qq) + logbeta) / qq);
 	else {
 	    t = (4. * pp + r - 2.) / t;
 	    if (t <= 1.)
@@ -139,12 +127,12 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
      * NEW: 'acu' accuracy NOT for squared adjustment, but simple;
      * ---- i.e.,  "new acu" = sqrt(old acu)
 
-     */
+    */
     acu = fmax2(acu_min, pow(10., -13 - 2.5/(pp * pp) - 0.5/(a * a)));
     tx = prev = 0.;	/* keep -Wall happy */
 
     for (i_pb=0; i_pb < 1000; i_pb++) {
-	y = pbeta_raw(xinbta, pp, qq, /*lower_tail = */ TRUE);
+	y = pbeta_raw(xinbta, pp, qq, /*lower_tail = */ TRUE, FALSE);
 	/* y = pbeta_raw2(xinbta, pp, qq, logbeta) -- to SAVE CPU; */
 #ifdef IEEE_754
 	if(!R_FINITE(y))
@@ -154,7 +142,7 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 		ML_ERR_return_NAN;
 
 	y = (y - a) *
-	    exp(logbeta + r * log(xinbta) + t * log(1 - xinbta));
+	    exp(logbeta + r * log(xinbta) + t * log1p(-xinbta));
 	if (y * yprev <= 0.)
 	    prev = fmax2(fabs(adj),fpu);
 	g = 1;
@@ -171,18 +159,13 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 	    }
 	    g /= 3;
 	}
-	xtrunc = tx;	/* this prevents trouble with excess FPU */
-				/* precision on some machines. */
-	if (xtrunc == xinbta)
-	    goto L_converged;
+	if (fabs(tx - xinbta) < 1e-15*xinbta) goto L_converged;
 	xinbta = tx;
 	yprev = y;
     }
     /*-- NOT converged: Iteration count --*/
-    ML_ERROR(ME_PRECISION);
+    ML_ERROR(ME_PRECISION, "qbeta");
 
- L_converged:
-    if (swap_tail)
-	return 1 - xinbta;
-    return xinbta;
+L_converged:
+    return swap_tail ? 1 - xinbta : xinbta;
 }
