@@ -22,13 +22,11 @@ rErr <- function(approx, true, eps = .Options$rErr.eps)
 	   true - approx)     # absolute error (e.g. when true=0)
 }
 ## Numerical equality: Here want "rel.error" almost always:
-All.eq <- function(x,y)
+All.eq <- function(x,y) {
     all.equal.numeric(x,y, tolerance= 64*.Machine$double.eps,
-                      scale = {r <- mean(abs(x),na.rm=TRUE)
-                               if(r > 0) r else 0})
-
-if(!interactive())
-    .Random.seed <- c(0,rep(7654, 3))
+                      scale = max(0, mean(abs(x), na.rm=TRUE)))
+}
+if(!interactive()) set.seed(123)
 
 ## The prefixes of ALL the PDQ & R functions
 PDQRinteg <- c("binom", "geom", "hyper", "nbinom", "pois","signrank","wilcox")
@@ -56,11 +54,10 @@ for(n in rbinom(n1, size = 2*n0, p = .4)) {
 	    tst <- all.equal(if(k==n || p==0) 1 else
 			     pf((k+1)/(n-k)*(1-p)/p, df1=2*(n-k), df2=2*(k+1)),
 			     sum(dbinom(0:k, size = n, prob = p)))
-	    if(!(is.logical(tst1) && tst1) ||
-	       !(is.logical(tst)  && tst ) ) {
+	    if(!isTRUE(tst1) || !isTRUE(tst)) {
 		cat("n=", n,"; p =",format(p),".  k =",k)
-		if(!is.logical(tst1)) cat("; tst1=",tst1)
-		if(!is.logical(tst )) cat("; tst=", tst)
+		if(!isTRUE(tst1)) cat("; tst1=",tst1)
+		if(!isTRUE(tst )) cat("; tst=", tst)
 		cat("\n")
 	    }
 	}
@@ -69,7 +66,7 @@ for(n in rbinom(n1, size = 2*n0, p = .4)) {
 }
 
 ##__ 2. Geometric __
-for(pr in seq(0,1,len=15)) {
+for(pr in seq(1e-10,1,len=15)) { # p=0 is not a distribution
     print(All.eq((dg <- dgeom(0:10, pr)),
 		 pr * (1-pr)^(0:10)))
     print(All.eq(cumsum(dg), pgeom(0:10, pr)))
@@ -103,13 +100,13 @@ for(lambda in rexp(n1))
     for(k in rpois(n2, lambda)) {
 	tst <- all.equal(1 - pchisq(2*lambda, 2*(1+ 0:k)),
 			 pp <- cumsum(dpois(0:k, lambda=lambda)), tol= 100*Meps)
-	if(!(is.logical(tst) && tst))
+	if(!isTRUE(tst))
 	    cat("lambda=", format(lambda),".  k =",k, " --> tst=", tst,"\n")
 	tst2 <- all.equal(pp, ppois(0:k, lambda=lambda), tol = 100*Meps)
-	if(!(is.logical(tst2) && tst2))
+	if(!isTRUE(tst2))
 	    cat("lambda=", format(lambda),".  k =",k, " --> tst2=", tst2,"\n")
 	tst3 <- all.equal(1 - pp, ppois(0:k, lambda=lambda, lower.tail=FALSE))
-	if(!(is.logical(tst3) && tst3))
+	if(!isTRUE(tst3))
 	    cat("lambda=", format(lambda),".  k =",k, " --> tst3=", tst3,"\n")
     }
 
@@ -117,8 +114,8 @@ for(lambda in rexp(n1))
 ##__ 6. SignRank __
 for(n in rpois(32, lam=8)) {
     x <- -1:(n + 4)
-    eq <- All.eq(psignrank(x, n), cumsum(dsignrank(x, n)))
-    if(!is.logical(eq) || !eq) print(eq)
+    if(!isTRUE(eq <- All.eq(psignrank(x, n), cumsum(dsignrank(x, n)))))
+        print(eq)
 }
 
 ##__ 7. Wilcoxon (symmetry & cumulative) __
@@ -129,8 +126,8 @@ for(n in rpois(5, lam=6))
 	fx <- dwilcox(x, n, m)
 	Fx <- pwilcox(x, n, m)
 	is.sym <- is.sym & all(fx == dwilcox(x, m, n))
-	eq <- All.eq(Fx, cumsum(fx))
-	if(!is.logical(eq) || !eq) print(eq)
+	if(!isTRUE(eq <- All.eq(Fx, cumsum(fx))))
+            print(eq)
     }
 is.sym
 
@@ -145,18 +142,18 @@ for(sh in round(rlnorm(30),2)) {
 	tst <- all.equal((d1 <- dgamma(	 x,   shape = sh, scale = sig)),
 			 (d2 <- dgamma(x/sig, shape = sh, scale = 1) / sig),
 			 tol = 1e-14)## __ad interim__ was 1e-15
-	if(!(is.logical(tst) && tst))
+	if(!isTRUE(tst))
 	    cat("ERROR: dgamma() doesn't scale:",tst,"\n",
 		"  x =", formatC(x),"\n	 shape,scale=",formatC(c(sh, sig)),"\n")
 	tst <- All.eq(d1, (d3 <- 1/(Ga * sig^sh) * x^(sh-1) * exp(-x/sig)))
-	if(!(is.logical(tst) && tst))
+	if(!isTRUE(tst))
 	    cat("NOT Equal:",tst,"\n x =", formatC(x),
 		"\n  shape,scale=",formatC(c(sh, sig)),"\n")
     }
 }
 pgamma(1,Inf,scale=Inf) == 0
+## Also pgamma(Inf,Inf) == 1 for which NaN was slightly more appropriate
 all(is.nan(c(pgamma(Inf,  1,scale=Inf),
-             pgamma(Inf,Inf,scale=1),
              pgamma(Inf,Inf,scale=Inf))))
 scLrg <- c(2,100, 1e300*c(.1, 1,10,100), 1e307, xMax, Inf)
 stopifnot(pgamma(Inf, 1, scale=xMax) == 1,
@@ -171,7 +168,7 @@ p <- 7e-4; df <- 0.9
 abs(1-c(pchisq(qchisq(p, df),df)/p, # was 2.31e-8 for R <= 1.8.1
         pchisq(qchisq(1-p, df,lower=FALSE),df,lower=FALSE)/(1-p),# was 1.618e-11
         pchisq(qchisq(log(p), df,log=TRUE),df, log=TRUE)/log(p), # was 3.181e-9
-        pchisq(qchisq(log(1-p),df,log=T,lower=F),df, log=T,lower=F)/log(1-p),
+        pchisq(qchisq(log(1-p),df,log=T,lower=F),df, log=T,lower=F)/log(1-p)
         )# 32b-i386: (2.2e-16, 0,0, 3.3e-16); Opteron: (2.2e-16, 0,0, 2.2e-15)
     ) < 1e-14
 
@@ -191,14 +188,16 @@ for(df in c(0.1, 0.5, 1.5, 4.7, 10, 20,50,100)) {
 }
 
 ## p ~= 1 (<==> 1-p ~= 0) -- gave infinite loop in R <= 1.8.1 -- PR#6421
+options(warn=-1) # ignore warnings from R's version of log1p
 psml <- 2^-(10:54)
 q0 <- qchisq(psml,    df=1.2, ncp=10, lower.tail=FALSE)
-q1 <- qchisq(1 -psml, df=1.2, ncp=10)
+q1 <- qchisq(1-psml, df=1.2, ncp=10) # inaccurate in the tail
 p0 <- pchisq(q0, df=1.2, ncp=10, lower.tail=FALSE)
 p1 <- pchisq(q1, df=1.2, ncp=10, lower.tail=FALSE)
-iO <- 1:10
-all.equal(q0[iO], q1[iO])
-all.equal(p0[iO], psml[iO], tol = 0.08)# bad tol (0.0744 on 386-Linux).
+iO <- 1:30
+all.equal(q0[iO], q1[iO], 1e-5)
+all.equal(p0[iO], psml[iO])
+options(warn=0)
 
 ##--- Beta (need more):
 
@@ -233,7 +232,7 @@ all.equal(qnorm(-1e5, log = TRUE), -447.1974945)
 z <- rnorm(1000); all.equal(pnorm(z),  1 - pnorm(-z), tol= 1e-15)
 z <- c(-Inf,Inf,NA,NaN, rt(1000, df=2))
 z.ok <- z > -37.5 | !is.finite(z)
-for(df in 1:10) if(!is.logical(all.equal(pt(z, df), 1 - pt(-z,df), tol= 1e-15)))
+for(df in 1:10) if(!isTRUE(all.equal(pt(z, df), 1 - pt(-z,df), tol= 1e-15)))
     cat("ERROR -- df = ", df, "\n")
 All.eq(pz <- pnorm(z), 1 - pnorm(z, lower=FALSE))
 All.eq(pz,		 pnorm(-z, lower=FALSE))
@@ -260,7 +259,8 @@ all.equal(z[ok], qnorm(pz[ok]), tol= 1e-12)
 
 ###===== Random numbers -- first, just output:
 
-.Random.seed <- c(0, 17292, 29447, 24113)
+set.seed(123)
+# .Random.seed <- c(0L, 17292L, 29447L, 24113L)
 n <- 20
 ## for(pre in PDQR) { n <- paste("r",pre,sep=""); cat(n,": "); str(get(n))}
 (Rbeta	  <- rbeta    (n, shape1 = .8, shape2 = 2) )
@@ -444,8 +444,19 @@ All.eq(1, -1e-17/ pexp(qexp(-1e-17, log=TRUE),log=TRUE))
 abs(pgamma(30,100, lower=FALSE, log=TRUE) + 7.3384686328784e-24) < 1e-36
 All.eq(1, pcauchy(-1e20)           /  3.18309886183791e-21)
 All.eq(1, pcauchy(+1e15, log=TRUE) / -3.18309886183791e-16)## PR#6756
-for(x in 10^c(15,25,50,100,200))
-    print(all.equal(pt(-x, df=1), pcauchy(-x), tol = 1e-15))
+x <- 10^(ex <- c(1,2,5*(1:5),50,100,200,300,Inf))
+for(a in x[ex > 10]) ## improve pt() : cbind(x,t= pt(-x, df=1), C=pcauchy(-x))
+    print(all.equal(pt(-a, df=1), pcauchy(-a), tol = 1e-15))
+## for PR#7902:
+ex <- -c(rev(1/x), ex)
+All.eq(-x, qcauchy(pcauchy(-x)))
+All.eq(+x, qcauchy(pcauchy(+x, log=TRUE), log=TRUE))
+All.eq(1/x, pcauchy(qcauchy(1/x)))
+All.eq(ex,  pcauchy(qcauchy(ex, log=TRUE), log=TRUE))
+II <- c(-Inf,Inf)
+stopifnot(pcauchy(II) == 0:1, qcauchy(0:1) == II,
+          pcauchy(II, log=TRUE) == c(-Inf,0),
+          qcauchy(c(-Inf,0), log=TRUE) == II)
 
 pr <- 1e-23 ## PR#6757
 stopifnot(all.equal(pr^ 12, pbinom(11, 12, prob= pr,lower=FALSE),
@@ -466,11 +477,139 @@ all.equal(pgamma(x, 10, log = TRUE) - 10*log(x),
 all.equal(pgamma(x, 0.1, log = TRUE) - 0.1*log(x),
           rep(0.0498724412598364, length(x)), tol = 1e-13)# 7e-16 (i386)
 
-All.eq(dpois(  10, 2e-308, log=TRUE), -7100.13502718914)
-All.eq(dpois(  20, 3e-308, log=TRUE), -14204.2875435307)
+All.eq(dpois(  10*1:2, 3e-308, log=TRUE),
+       c(-7096.08037610806, -14204.2875435307))
 All.eq(dpois(1e20, 1e-290, log=TRUE), -7.12801378828154e+22)
 ## all gave -Inf in R 2.0.1
 
 
+## Inf df in pf etc.
+# apparently pf(df2=Inf) worked in 2.0.1 (undocumented) but df did not.
+x <- c(1/pi, 1, pi)
+oo <- options(digits = 8)
+df(x, 3, 1e6)
+df(x, 3, Inf)
+pf(x, 3, 1e6)
+pf(x, 3, Inf)
+
+df(x, 1e6, 5)
+df(x, Inf, 5)
+pf(x, 1e6, 5)
+pf(x, Inf, 5)
+
+df(x, Inf, Inf)# (0, Inf, 0)  - since 2.1.1
+pf(x, Inf, Inf)# (0, 1/2, 1)
+
+pf(x, 5, Inf, ncp=0)
+pf(x, 5, 1e6, ncp=1)
+pf(x, 5, 1e7, ncp=1)
+all.equal(pf(x, 5, 1e8, ncp=1), tol = 1e-6,
+          c(0.0659330751, 0.4708802996, 0.9788764591))
+pf(x, 5, Inf, ncp=1)
+
+dt(1, Inf)
+dt(1, Inf, ncp=0)
+dt(1, Inf, ncp=1)
+dt(1, 1e6, ncp=1)
+dt(1, 1e7, ncp=1)
+dt(1, 1e8, ncp=1)
+dt(1, 1e10, ncp=1) # = Inf
+## Inf valid as from 2.1.1: df(x, 1e16, 5) was way off in 2.0.1.
+
+sml.x <- c(10^-c(2:8,100), 0)
+cbind(x = sml.x, `dt(x,*)` = dt(sml.x, df = 2, ncp=1))
+## small 'x' used to suffer from cancellation
+options(oo)
+
+## pf() with large df1 or df2
+## (was said to be PR#7099, but that is about non-central pchisq)
+nu <- 2^seq(25, 34, 0.5)
+target <- pchisq(1, 1) # 0.682...
+y <- pf(1, 1, nu)
+stopifnot(All.eq(pf(1, 1, Inf), target),
+          diff(c(y, target)) > 0, # i.e. pf(1, 1, *) is monotone increasing
+          abs(y[1] - (target - 7.21129e-9)) < 1e-11) # computed value
+## non-monotone in R <= 2.1.0
+
+stopifnot(pgamma(Inf, 1.1) == 1)
+## didn't not terminate in R 2.1.x (only)
+
+## qgamma(q, *) should give {0,Inf} for q={0,1}
+sh <- c(1.1, 0.5, 0.2, 0.15, 1e-2, 1e-10)
+stopifnot(Inf == qgamma(1, sh))
+stopifnot(0   == qgamma(0, sh))
+## the first gave Inf, NaN, and 99.425 in R 2.1.1 and earlier
+
+f2 <- c(0.5, 1:4)
+stopifnot(df(0, 1, f2) == Inf,
+          df(0, 2, f2) == 1,
+          df(0, 3, f2) == 0)
+## only the last one was ok in R 2.2.1 and earlier
+
+x0 <- -2 * 10^-c(22,10,7,5)
+stopifnot(pbinom(x0, size = 3, prob = 0.1) == 0,
+          dbinom(x0, 3, 0.1) == 0) # d*() warns about non-integer
+## very small negatives were rounded to 0 in R 2.2.1 and earlier
+
+## dbeta(*, ncp):
+a <- rlnorm(100)
+stopifnot(All.eq(a, dbeta(0, 1, a, ncp=0)),
+          dbeta(0, 0.9, 2.2, ncp = c(0, a)) == Inf
+          )
+## the first gave 0, the 2nd NaN in R <= 2.3.0
+
+## df(*, ncp):
+x <- seq(0, 10, length=101)
+h <- 1e-7
+dx.h <- (pf(x+h, 7, 5, ncp= 2.5) - pf(x-h, 7, 5, ncp= 2.5)) / (2*h)
+stopifnot(all.equal(dx.h, df(x, 7, 5, ncp= 2.5), tol = 1e-6),# (1.50 | 1.65)e-8
+          All.eq(df(0, 2, 4, ncp=x), df(1e-300, 2, 4, ncp=x))
+          )
+
+## qt(p ~ 0, df=1) - PR#9804
+p <- 10^(-10:-20)
+qtp <- qt(p, df = 1)
+## relative error < 10^-14 :
+stopifnot(abs(1 - p / pt(qtp, df=1)) < 1e-14)
+
+## Similarly for df = 2 --- both for p ~ 0  *and*  p ~ 1/2
+## P ~ 0
+stopifnot(all.equal(qt(-740, df=2, log=TRUE), -exp(370)/sqrt(2)))
+## P ~ 1 (=> p ~ 0.5):
+p.5 <- 0.5 + 2^(-5*(5:8))
+p.5 - 0.5
+stopifnot(all.equal(qt(p.5, df = 2),
+		    c(8.429369702179e-08, 2.634178031931e-09,
+		      8.231806349784e-11, 2.572439484308e-12)))
+## qt(<large>, log = TRUE)  is now more finite and monotone (again!):
+stopifnot(all.equal(qt(-1000, df = 4, log=TRUE),
+                    -4.930611e108, tol = 1e-6))
+qtp <- qt(-(20:850), df=1.2, log=TRUE, lower=FALSE)
+##almost: stopifnot(all(abs(5/6 - diff(log(qtp))) < 1e-11))
+stopifnot(abs(5/6 - quantile(diff(log(qtp)), pr=c(0,0.995))) < 1e-11)
+
+## close to df=1 (where Taylor steps are important!):
+all.equal(-20, pt(qt(-20, df=1.02, log=TRUE),
+                          df=1.02, log=TRUE), tol = 1e-12)
+stopifnot(diff(lq <- log(qt(-2^-(10:600), df=1.1, log=TRUE))) > 0.6)
+lq1 <- log(qt(-2^-(20:600), df=1, log=TRUE))
+lq2 <- log(qt(-2^-(20:600), df=2, log=TRUE))
+stopifnot(mean(abs(diff(lq1) - log(2)      )) < 1e-8,
+	  mean(abs(diff(lq2) - log(sqrt(2)))) < 4e-8)
+
+## pbeta(*, log=TRUE) {toms708} -- now improved tail behavior
+x <- c(.01, .10, .25, .40, .55, .71, .98)
+pbval <- c(-0.04605755624088, -0.3182809860569, -0.7503593555585,
+           -1.241555830932, -1.851527837938, -2.76044482378, -8.149862739881)
+all.equal(pbeta(x, 0.8, 2, lower=FALSE, log=TRUE), pbval)
+all.equal(pbeta(1-x, 2, 0.8, log=TRUE), pbval)
+qq <- 2^(0:1022)
+df.set <- c(0.1, 0.2, 0.5, 1, 1.2, 2.2, 5, 10, 20, 50, 100, 500)
+for(nu in df.set) {
+    pqq <- pt(-qq, df = nu, log=TRUE)
+    stopifnot(is.finite(pqq))
+}
+
+
 
 cat("Time elapsed: ", proc.time() - .ptime,"\n")
